@@ -50,17 +50,19 @@
 #else
 #define KONOHA_NAME     "konoha"
 #define KONOHA_MAJOR_VERSION  0
-#define KONOHA_MIDDLE_VERSION 5
-#define KONOHA_MINER_VERSION  0
-#define KONOHA_VERSION  "0.5"
+#define KONOHA_MINER_VERSION  7
+#define KONOHA_PATCH_VERSION  0
+#define KONOHA_VERSION  "0.7b"
 #endif
 
-#define KONOHA_XCODE    "toguro"
+#define LIBKONOHA_VERSION "0.7"
+
+#define KONOHA_XCODE    "BAKUMAN"
 #define KONOHA_URL		"http://konoha.sourceforge.jp/"
 #define KONOHA_URLBASE  "kttp://konoha.sourceforge.jp/wiki/"
 
-#define KONOHA_BUILDID_LATEST  625
-#define KONOHA_BUILDID_TOOOLD  625
+#define KONOHA_BUILDID_LATEST  645
+#define KONOHA_BUILDID_TOOOLD  645
 #define KNH_CHECK_VERSION(id)  ((id) <= KONOHA_BUILDID_TOOOLD)
 
 /* ======================================================================== */
@@ -210,6 +212,10 @@
 #define KONOHA_PLATFORM KONOHA_OS "_32"
 #endif
 
+#ifdef __SSE2__
+#define KNH_USING_SSE2
+#endif
+
 /* ======================================================================== */
 /* [OPTIONS_DEPS] */
 
