@@ -1,13 +1,32 @@
+/****************************************************************************
+ * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
+ *
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ *           (c) 2008-      Konoha Software Foundation
+ * All rights reserved.
+ *
+ * You may choose one of the following two licenses when you use konoha.
+ * See www.konohaware.org/license.html for further information.
+ *
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
+ * (2) Konoha Software Foundation License 1.0
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
+ * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+ * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ ****************************************************************************/
+
 #ifndef KONOHA_DEPS_H_
 #define KONOHA_DEPS_H_
-#include<konoha/konoha_t.h>
-
-#ifdef KNH_USING_STDC
-#include<stdlib.h>
-#include<string.h>
-#include<setjmp.h>
-#endif
-
 
 /* ======================================================================== */
 /* [UNIX] */
@@ -20,35 +39,35 @@
 #endif
 
 #ifdef KONOHA_ON_LINUX
-#define		KNH_USING_POSIX 1
-#define		KNH_USING_UNIX  1
+#define     KNH_USING_POSIX 1
+#define     KNH_USING_UNIX  1
 #define     KNH_USING_SOCKET 1
-#define 	KNH_USING_THREAD   1
+#define     KNH_USING_THREAD   1
 #define     KNH_USING_PTHREAD  1
-#define		KONOHA_OS_ENCODING "UTF-8"
-#define 	KNH_USING_KONOHAGET 1
-	#ifdef HAVE_ICONV_H
-	#define		KNH_USING_ICONV 1
-	#endif
+#define     KONOHA_OS_ENCODING "UTF-8"
+#define     KNH_USING_KONOHAGET 1
+#ifdef HAVE_ICONV_H
+#define     KNH_USING_ICONV 1
+#endif
 //#ifdef HAVE_REGEX_H
-#define		KNH_USING_REGEX 1
+#define     KNH_USING_REGEX 1
 //#endif
-#define		KONOHA_OS_LINEFEED "\n"
-#define		KONOHA_OS_DLLEXT   ".so"
+#define     KONOHA_OS_LINEFEED "\n"
+#define     KONOHA_OS_DLLEXT   ".so"
 #endif
 
 #ifdef KONOHA_ON_MACOSX
-#define 	KNH_USING_POSIX    1
-#define 	KNH_USING_UNIX     1
-#define 	KNH_USING_SOCKET   1
-#define 	KNH_USING_ICONV    1
-#define 	KNH_USING_REGEX    1
-#define 	KNH_USING_SQLITE3  1
-#define 	KNH_USING_THREAD   1
-#define     KNH_USING_PTHREAD  1
-#define 	KONOHA_OS_ENCODING "UTF-8"
-#define 	KONOHA_OS_LINEFEED "\n"
-#define 	KONOHA_OS_DLLEXT ".dylib"
+#define     KNH_USING_POSIX    1
+#define     KNH_USING_UNIX     1
+#define     KNH_USING_SOCKET   1
+#define     KNH_USING_ICONV    1
+#define     KNH_USING_REGEX    1
+//#define     KNH_USING_SQLITE3  1
+//#define     KNH_USING_THREAD   1
+//#define     KNH_USING_PTHREAD  1
+#define     KONOHA_OS_ENCODING "UTF-8"
+#define     KONOHA_OS_LINEFEED "\n"
+#define     KONOHA_OS_DLLEXT ".dylib"
 #endif
 
 /* ------------------------------------------------------------------------ */
@@ -89,12 +108,12 @@
 #define     KNH_USING_MATH     1
 #define 	KONOHA_OS_LINEFEED "\r\n"
 #ifdef KNHAPI_IMPORTS
-#define 	KNHAPI(T)     __declspec(dllimport) T __cdecl
+#define 	KNHAPI(T__)     __declspec(dllimport) T__ __cdecl
 #else
-#define 	KNHAPI(T)     __declspec(dllexport) T __cdecl
+#define 	KNHAPI(T__)     __declspec(dllexport) T__ __cdecl
 #endif
 
-#define 	KNH_EXPORTS(T)     __declspec(dllexport) T __cdecl
+#define 	KNH_EXPORTS(T__)     __declspec(dllexport) T__ __cdecl
 
 #define 	KONOHA_OS_DLLEXT ".dll"
 #define		KONOHA_OS_FILE_SEPARATOR '\\'
@@ -106,11 +125,11 @@
 #define 	KNH_USING_WINDOWS  1
 #define 	KONOHA_OS_LINEFEED "\r\n"
 #ifdef KNHAPI_IMPORTS
-#define 	KNHAPI(T) __declspec(dllimport) T __cdecl
+#define 	KNHAPI(T__) __declspec(dllimport) T__ __cdecl
 #else
-#define 	KNHAPI(T) __declspec(dllexport) T __cdecl
+#define 	KNHAPI(T__) __declspec(dllexport) T__ __cdecl
 #endif
-#define 	KNH_EXPORTS(T)  __declspec(dllexport) T __cdecl
+#define 	KNH_EXPORTS(T__)  __declspec(dllexport) T__ __cdecl
 #define 	KONOHA_OS_DLLEXT ".dll"
 #define		KONOHA_OS_FILE_SEPARATOR '\\'
 #ifndef HAVE_CONFIG_H
@@ -145,7 +164,7 @@
 
 #ifndef KNHAPI_
 #ifdef KNHAPI
-#define KNHAPI_(T)       KNHAPI(T)
+#define KNHAPI_(T__)       KNHAPI(T__)
 #else
 #define KNHAPI_(T)       T KNH_CC_FASTCALL
 #endif
@@ -157,7 +176,7 @@
 
 #ifndef KNHAPI
 #ifdef  KONOHA_ON_WINDOWS
-#define KNHAPI(T)         T __declspec(dllexport)
+#define KNHAPI(T__)         T__ __declspec(dllexport)
 #else
 #define KNHAPI(T)         T
 #endif
@@ -166,8 +185,8 @@
 
 #ifdef KONOHA_MONOLITHIC
 #undef KNHAPI
-#define KNHAPI(T)        T
-#define KNH_EXPORTS(T)   T
+#define KNHAPI(T__)        T__
+#define KNH_EXPORTS(T__)   T__
 #endif
 
 #ifndef KONOHA_OS_FILE_SEPARATOR
@@ -227,13 +246,28 @@
 /* ------------------------------------------------------------------------ */
 
 #ifdef ALT_SORT
-#define  knh_sort(b,s,w,f)     ALT_sort(b,s,w,f)
-#define  knh_qsort(b,s,w,f)    ALT_qsort(b,s,w,f)
+#define  knh_sort(b,s,w,f)            ALT_sort(b,s,w,f)
+#define  knh_qsort(b,s,w,f)           ALT_qsort(b,s,w,f)
+#else /* ALT_SORT */
+
+#ifdef KNH_EXT_QSORT
+//#define  knh_qsort(b,s,w,f)           knh_sort(b,s,w,f)
 #else
-#define  knh_sort(b,s,w,f)     qsort(b,s,w,f)
-#define  knh_qsort(b,s,w,f)    qsort(b,s,w,f)
+#define  knh_qsort(b,s,w,f)           qsort(b,s,w,f)
 #endif
 
+#ifdef KONOHA_ON_MACOSX
+#define  knh_qsort_r(b,s,w,thunk,f)   qsort_r(b,s,w,thunk,f)
+#elif defined(KONOHA_ON_LINUX)
+#define  knh_qsort_r(b,s,w,thunk,f)   qsort_r(b,s,w,f,thunk)
+#elif defined(KONOHA_ON_WINDOWS)
+#define  knh_qsort_r(b,s,w,thunk,f)   qsort_s(b,s,w,f,thunk)
+#else /* FIXME KONOHA_ON_UNKNOWN ?? */
+//#define  knh_qsort_r(b,s,w,thunk,f)   qsort_r(b,s,w,f,thunk)
+#endif
+
+#endif /* ALT_SORT */
+
 /* ======================================================================== */
 
 #if defined(KNH_USING_POSIX) && !defined(KONOHA_ON_LKM)
