@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -41,11 +41,11 @@ extern "C" {
 /* [common] */
 
 static
-knh_hcode_t knh_stack_hashCode(Ctx *ctx, knh_sfp_t *sfp)
+knh_hashcode_t knh_stack_hashCode(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_class_t bcid = (sfp[0].o)->h.bcid;
 	if(CLASS_Boolean <= bcid && bcid <= CLASS_Float) {
-		return (knh_hcode_t)(sfp[0].i)->n.data;
+		return (knh_hashcode_t)(sfp[0].i)->n.data;
 	}
 	return ctx->share->StructTable[bcid].fhashCode(ctx, sfp[0].o);
 }
@@ -57,17 +57,17 @@ int knh_stack_equals(Ctx *ctx, knh_sfp_t *sfp, Object *o)
 {
 	knh_class_t bcid = (sfp[0].o)->h.bcid;
 	if(CLASS_Boolean <= bcid && bcid <= CLASS_Float) {
-		return ((sfp[0].i)->n.data == ((Int*)o)->n.data);
+		return ((sfp[0].i)->n.data == ((knh_Int_t*)o)->n.data);
 	}
 	return (knh_Object_compareTo(ctx, sfp[0].o, o) == 0);
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method This! HashMap.new(Int init) */
+//## method This! HashMap.new(Int? initCapacity);
 
-static METHOD knh__HashMap_new(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD HashMap_new(Ctx *ctx, knh_sfp_t *sfp)
 {
-	HashMap *o = (HashMap*)sfp[0].o;
+	knh_HashMap_t *o = (knh_HashMap_t*)sfp[0].o;
 	int init = IS_NULL(sfp[1].o) ? KNH_HASH_INITSIZE: p_int(sfp[1]);
 	if(init > 0) {
 		ctx->share->StructTable[STRUCT_HashMap].finit(ctx, sfp[0].o, init);
@@ -76,12 +76,12 @@ static METHOD knh__HashMap_new(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method Any2 HashMap.get(Any1 key) */
+//## method T2 HashMap.get(T1 key);
 
-static METHOD knh__HashMap_get(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD HashMap_get(Ctx *ctx, knh_sfp_t *sfp)
 {
-	HashMap *o = (HashMap*)sfp[0].o;
-	knh_hcode_t hcode = knh_stack_hashCode(ctx, sfp + 1);
+	knh_HashMap_t *o = (knh_HashMap_t*)sfp[0].o;
+	knh_hashcode_t hcode = knh_stack_hashCode(ctx, sfp + 1);
 	knh_uintptr_t h =  hcode % DP(o)->hmax;
 	knh_hashentry_t *e = DP(o)->array[h];
 
@@ -97,12 +97,12 @@ static METHOD knh__HashMap_get(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method Boolean! HashMap.opHas(Any1 key) */
+//## method Boolean! HashMap.opHas(T1 key);
 
-static METHOD knh__HashMap_opHas(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD HashMap_opHas(Ctx *ctx, knh_sfp_t *sfp)
 {
-	HashMap *o = (HashMap*)sfp[0].o;
-	knh_hcode_t hcode = knh_stack_hashCode(ctx, sfp + 1);
+	knh_HashMap_t *o = (knh_HashMap_t*)sfp[0].o;
+	knh_hashcode_t hcode = knh_stack_hashCode(ctx, sfp + 1);
 	knh_uintptr_t h =  hcode % DP(o)->hmax;
 	knh_hashentry_t *e = DP(o)->array[h];
 
@@ -118,45 +118,41 @@ static METHOD knh__HashMap_opHas(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void HashMap.remove(Any1 key) */
+//## method void HashMap.remove(T1 key);
 
-static METHOD knh__HashMap_remove(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD HashMap_remove(Ctx *ctx, knh_sfp_t *sfp)
 {
-	if(IS_IMM(sfp[0].o)) {
-		knh_Hash_t *o = (knh_Hash_t*)sfp[0].o;
-		knh_hcode_t hcode = knh_stack_hashCode(ctx, sfp + 1);
-		knh_uintptr_t h =  hcode % DP(o)->hmax;
-		knh_hashentry_t *e = DP(o)->array[h];
-		knh_hashentry_t **prev_next = &(DP(o)->array[h]);
+	knh_Hash_t *o = (knh_Hash_t*)sfp[0].o;
+	knh_hashcode_t hcode = knh_stack_hashCode(ctx, sfp + 1);
+	knh_uintptr_t h =  hcode % DP(o)->hmax;
+	knh_hashentry_t *e = DP(o)->array[h];
+	knh_hashentry_t **prev_next = &(DP(o)->array[h]);
 
-		while(e != NULL) {
-			if(e->hcode == hcode
-					&& knh_Object_cid(sfp[1].o) == knh_Object_cid(e->key)
-					&& knh_stack_equals(ctx, sfp + 1, e->key)) {
-				prev_next[0] = e->next;
-				knh_hashentry_collect(ctx, o, e);
-				DP(o)->size--;
-			}
-			prev_next = &(e->next);
-			e = e->next;
+	while(e != NULL) {
+		if(e->hcode == hcode
+				&& knh_Object_cid(sfp[1].o) == knh_Object_cid(e->key)
+				&& knh_stack_equals(ctx, sfp + 1, e->key)) {
+			prev_next[0] = e->next;
+			knh_hashentry_collect(ctx, o, e);
+			DP(o)->size--;
 		}
+		prev_next = &(e->next);
+		e = e->next;
 	}
 	KNH_RETURN_void(ctx, sfp);
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void HashMap.set(Any1! key, Any2 value) */
+//## method void HashMap.set(T1! key, T2 value);
 
-static METHOD knh__HashMap_set(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD HashMap_set(Ctx *ctx, knh_sfp_t *sfp)
 {
-	if(IS_IMM(sfp[0].o)) {
-		if(IS_NULL(sfp[2].o)) {
-			knh__HashMap_remove(ctx, sfp);
-			return ;
-		}
-
+	if(IS_NULL(sfp[2].o)) {
+		HashMap_remove(ctx, sfp);
+	}
+	else {
 		knh_Hash_t *o = (knh_Hash_t*)sfp[0].o;
-		knh_hcode_t hcode = knh_stack_hashCode(ctx, sfp + 1);
+		knh_hashcode_t hcode = knh_stack_hashCode(ctx, sfp + 1);
 		knh_uintptr_t h =  hcode % DP(o)->hmax;
 		knh_hashentry_t *e = DP(o)->array[h];
 
@@ -182,117 +178,17 @@ static METHOD knh__HashMap_set(Ctx *ctx, knh_sfp_t *sfp)
 			DP(o)->array[h] = e;
 			DP(o)->size++;
 		}
+		KNH_RETURN_void(ctx, sfp);
 	}
-	KNH_RETURN_void(ctx, sfp);
-}
-
-/* ======================================================================== */
-/* @method void HashMap.%k(OutputStream w, String m) */
-
-void knh_HashMap__k(Ctx *ctx, knh_Hash_t *o, OutputStream *w, String *m)
-{
-	size_t pos = 0, c = 0;
-	size_t max = (KNH_HASH_TABLESIZE / o->hashop->size) * DP(o)->tables_size;
-	knh_putc(ctx, w, '{');
-	while(pos < max) {
-		knh_hashentry_t *e = knh_hashentry_at((knh_Hash_t*)o, pos);
-		if(e != NULL) {
-			if(c > 0) {
-				knh_write_delim(ctx,w);
-			}
-			knh_format(ctx, w, METHODN__k, e->key, KNH_NULL);
-			knh_putc(ctx, w, ':');
-			knh_putc(ctx, w, ' ');
-			knh_format(ctx, w, METHODN__k, e->value, KNH_NULL);
-			c++;
-		}
-		pos++;
-	}
-	knh_putc(ctx, w, '}');
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void HashMap.%dump(OutputStream w, String m) */
-
-void knh_HashMap__dump(Ctx *ctx, HashMap *o, OutputStream *w, String *m)
-{
-	knh_HashMap__k(ctx, (knh_Hash_t*)o, w, m);
-}
-
-/* ======================================================================== */
-/* [mapping] */
-
-static
-ITRNEXT knh_HashMap_var_next(Ctx *ctx, knh_sfp_t *sfp, int n)
-{
-	Iterator *it = sfp[0].it;
-	knh_Hash_t *o = (knh_Hash_t*)DP(it)->source;
-	KNH_ASSERT(IS_bHashMap(o));
-	size_t pos = DP(it)->pos;
-	size_t max = (KNH_HASH_TABLESIZE / o->hashop->size) * DP(o)->tables_size;
-	while(pos < max) {
-		knh_hashentry_t *e = knh_hashentry_at(o, pos);
-		if(e != NULL) {
-			DP(it)->pos = pos + 1;
-			KNH_ITRNEXT(ctx, sfp, n, e->key);
-		}
-		pos++;
-	}
-	KNH_ITREND(ctx, sfp, n);
-}
-
-/* ------------------------------------------------------------------------ */
-/* @map HashMap Iterator! @Final*/
-
-MAPPER knh_HashMap_Iterator(Ctx *ctx, knh_sfp_t *sfp)
-{
-	HashMap *o = (HashMap*)sfp[0].o;
-	KNH_MAPPED(ctx, sfp, new_Iterator(ctx, ctx->share->ClassTable[o->h.cid].p1, UP(o), knh_HashMap_var_next));
-}
-
-/* ------------------------------------------------------------------------ */
-/* @method Any1.. HashMap.opItr() */
-
-static METHOD knh__HashMap_opItr(Ctx *ctx, knh_sfp_t *sfp)
-{
-	HashMap *o = (HashMap*)sfp[0].o;
-	KNH_RETURN(ctx, sfp, new_Iterator(ctx, ctx->share->ClassTable[o->h.cid].p1, UP(o), knh_HashMap_var_next));
-}
-
-///* ------------------------------------------------------------------------ */
-//
-//static
-//Object* knh_HashMap_tuple2_next(Ctx *ctx, Iterator *it)
-//{
-//	HashMap *o = (HashMap*)DP(it)->source;
-//	KNH_ASSERT(IS_bHashMap(o));
-//	size_t pos = DP(it)->pos, capacity = knh_hashmapentry_capacity(ctx, o);
-//	while(pos < capacity) {
-//		knh_hashmapentry_t *e = knh_hashmapentry_at(o, pos);
-//		if(e != NULL) {
-//			DP(it)->pos = , pos+1);
-//			return new_Tuple2(ctx, e->key, e->value);
-//		}
-//		pos++;
-//	}
-//	return KNH_VOID;
-//}
-//
-///* ------------------------------------------------------------------------ */
-///* @map HashMap Tuple2..! */
-//
-//Iterator* knh_HashMap_Tuple2__(Ctx *ctx, HashMap *o, Mapper *mpr)
-//{
-//	return new_Iterator(ctx, CLASS_Tuple2, o, knh_HashMap_tuple2_next);
-//}
-//
-///* ------------------------------------------------------------------------ */
 
 ///* ======================================================================== */
 ///* [constructor] */
 //
 ///* ------------------------------------------------------------------------ */
-///* @method HashSet! HashSet.new(Int initialCapacity=0) */
+////## method HashSet! HashSet.new(Int initialCapacity=0);
 //
 //HashSet *knh_HashSet_new(Ctx *ctx, HashSet *b, size_t initialCapacity)
 //{
@@ -307,10 +203,10 @@ static METHOD knh__HashMap_opItr(Ctx *ctx, knh_sfp_t *sfp)
 ///* [method] */
 //
 ///* ------------------------------------------------------------------------ */
-///* @method Boolean HashSet.opHas(Any1 key) */
+////## method Boolean HashSet.opHas(T1 key);
 //
 //
-//knh_bool_t knh_HashSet_opHas(Ctx *ctx, HashSet *b, knh_hcode_t key, Object *keyobj)
+//knh_bool_t knh_HashSet_opHas(Ctx *ctx, HashSet *b, knh_hashcode_t key, Object *keyobj)
 //{
 //	return (knh_HashSet_get(ctx, b, key, keyobj) != 0);
 //}
@@ -330,7 +226,7 @@ static METHOD knh__HashMap_opItr(Ctx *ctx, knh_sfp_t *sfp)
 ///* ======================================================================== */
 ///* [movabletext] */
 //
-///* @method void HashSet.%dump(OutputStream w, String m) */
+////## method void HashSet.%dump(OutputStream w, String m);
 //
 //void knh_HashSet__dump(Ctx *ctx, HashSet *b, OutputStream *w, String *m)
 //{
@@ -347,7 +243,7 @@ static METHOD knh__HashMap_opItr(Ctx *ctx, knh_sfp_t *sfp)
 ////				break;
 ////			}
 ////			knh_Object__dump(ctx, cur->h.keyobj, w, lv+1);
-////			if(cur->value != 1) knh_printf(ctx, w, "<%d>", cur->value);
+////			if(cur->value != 1) knh_printf(ctx, w, "<%d>", (knh_intptr_t)cur->value);
 ////			c++;
 ////			cur = next;
 ////		}
