@@ -40,6 +40,17 @@ extern "C" {
 /* ======================================================================== */
 /* [toBoolean] */
 
+/* ------------------------------------------------------------------------ */
+//## @Const @Final mapper String Boolean;
+
+static MAPPER String_Boolean(Ctx *ctx, knh_sfp_t *sfp)
+{
+	knh_bytes_t t = knh_String_tobytes(sfp[0].s);
+	int b = (knh_bytes_equalsIgnoreCase(t, STEXT("true")) ||
+				knh_bytes_equalsIgnoreCase(t, STEXT("yes")));
+	KNH_MAPPED_Boolean(ctx, sfp, b);
+}
+
 /* ======================================================================== */
 /* [toInt] */
 
@@ -58,7 +69,7 @@ static MAPPER Float_Int(Ctx *ctx, knh_sfp_t *sfp)
 static MAPPER String_Int(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_int_t v = 0;
-	if(knh_bytes_parseint(__tobytes(sfp[0].s), &v)) {
+	if(knh_bytes_parseint(knh_String_tobytes(sfp[0].s), &v)) {
 		KNH_MAPPED_Int(ctx, sfp, v);
 	}
 	KNH_MAPPED(ctx, sfp, KNH_NULL);
@@ -81,8 +92,12 @@ static MAPPER Int_Float(Ctx *ctx, knh_sfp_t *sfp)
 
 static MAPPER String_Float(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_float_t f = KNH_FLOAT_ZERO;
-	if(knh_bytes_parsefloat(__tobytes(sfp[0].s), &f)) {
+#ifndef KNH_USING_NOFLOAT
+	knh_float_t f = 0.0;
+#else 
+	knh_float_t f = 0;
+#endif
+	if(knh_bytes_parsefloat(knh_String_tobytes(sfp[0].s), &f)) {
 		KNH_MAPPED_Float(ctx, sfp, f);
 	}
 	else {
@@ -147,7 +162,7 @@ static MAPPER Bytes_String(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_bytes_t t = knh_Bytes_tobytes((knh_Bytes_t*)sfp[0].o);
 	knh_String_t *s = TS_EMPTY;
-	if(knh_bytes_checkENCODING(t) == 0) {
+	if(knh_bytes_checkENCODING(t)) {
 		s = new_String(ctx, t, NULL);
 	}
 	KNH_MAPPED(ctx, sfp, s);
@@ -162,7 +177,7 @@ static MAPPER Bytes_String(Ctx *ctx, knh_sfp_t *sfp)
 static MAPPER String_Bytes(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_Bytes_t *b = new_Bytes(ctx, knh_String_strlen(sfp[0].s) + 1);
-	knh_Bytes_write(ctx, b, __tobytes(sfp[0].s));
+	knh_Bytes_write(ctx, b, knh_String_tobytes(sfp[0].s));
 	KNH_MAPPED(ctx, sfp, b);
 }
 
@@ -197,7 +212,7 @@ static MAPPER Iterator_Array(Ctx *ctx, knh_sfp_t *sfp)
 	knh_Iterator_t *it = sfp[0].it;
 	knh_Array_t *a = new_Array(ctx, ctx->share->ClassTable[it->h.cid].p1, 0);
 	while(it->fnext_1(ctx, sfp, 1)) {
-		knh_stack_boxing(ctx, sfp + 1);
+		knh_sfp_boxing(ctx, sfp + 1);
 		knh_Array_add(ctx, a, sfp[1].o);
 	}
 	KNH_MAPPED(ctx, sfp, a);
