@@ -55,15 +55,15 @@ void knh_ObjectField_setValue(Ctx *ctx, knh_ObjectField_t *of, knh_index_t idx,
 {
 	if(IS_NULL(value)) {
 		if(IS_NNTYPE(type)) return;
-		goto L_SET_;
+		goto L_SET;
 	}
 	else {
 		knh_class_t tcid = CLASS_type(type);
 		knh_class_t scid = knh_Object_cid(value);
-		DBG2_ASSERT_cid(tcid);
-		DBG2_ASSERT_cid(scid);
+		KNH_ASSERT_cid(tcid);
+		KNH_ASSERT_cid(scid);
 		if(scid == tcid || knh_class_instanceof(ctx, scid, tcid)) {
-			goto L_SET_;
+			goto L_SET;
 		}
 		DBG2_P("COERCION %s -> %s", CLASSN(scid), CLASSN(tcid));
 		TODO();
@@ -75,7 +75,7 @@ void knh_ObjectField_setValue(Ctx *ctx, knh_ObjectField_t *of, knh_index_t idx,
 		return ;
 	}
 
-	L_SET_:;
+	L_SET:;
 #ifdef KNH_USING_UNBOXFIELD
 	if(IS_ubxint(type)) {
 		knh_int_t *data = (knh_int_t*)(of->fields + idx);
@@ -109,14 +109,14 @@ static METHOD Object_new__dictmap(Ctx *ctx, knh_sfp_t *sfp)
 	size_t i;
 	for(i = 0; i < ac; i+= 2) {
 		if(IS_bString(v[i].s)) {
-			knh_fieldn_t fn = knh_getfnq(ctx, __tobytes(v[i].s), FIELDN_NONAME);
+			knh_fieldn_t fn = knh_getfnq(ctx, knh_String_tobytes(v[i].s), FIELDN_NONAME);
 			if(fn == FIELDN_NONAME) continue;
 			knh_index_t idx = knh_Class_queryField(ctx, cid, fn);
 			if(idx == -1) continue;
 			knh_cfield_t *cf = knh_Class_fieldAt(ctx, cid, idx);
 			knh_type_t type = knh_pmztype_totype(ctx, cf->type, cid);
 			if(type == TYPE_void) continue;
-			DBG2_P("[%d] %s%s %s", (int)(idx), TYPEQN(type), __tochar(v[i].s));
+			DBG2_P("[%d] %s%s %s", (int)(idx), TYPEQN(type), knh_String_tochar(v[i].s));
 			knh_ObjectField_setValue(ctx, of, idx, type, v[i+1].o);
 		}
 	}
@@ -125,7 +125,7 @@ static METHOD Object_new__dictmap(Ctx *ctx, knh_sfp_t *sfp)
 
 /* ------------------------------------------------------------------------ */
 /* [Bytes] */
-//## method This! Bytes.new(Int? init);
+//## method This! Bytes.new(Int init);
 
 static METHOD Bytes_new(Ctx *ctx, knh_sfp_t *sfp)
 {
@@ -151,7 +151,7 @@ static METHOD String_new(Ctx *ctx, knh_sfp_t *sfp)
 		s = new_String(ctx, knh_Bytes_tobytes(sfp[1].ba), NULL);
 	}
 	else {
-		knh_BytesConv_t *bc = new_BytesConv__in(ctx, __tochar(sfp[2].s));
+		knh_BytesConv_t *bc = new_BytesConv__in(ctx, knh_String_tochar(sfp[2].s));
 		KNH_SETv(ctx, sfp[3].o, bc);
 		s = new_String__bconv(ctx, knh_Bytes_tobytes(sfp[1].ba), bc);
 	}
@@ -165,7 +165,7 @@ static
 METHOD Regex_new(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_Regex_t *o = (knh_Regex_t*)sfp[0].o;
-	knh_bytes_t p = __tobytes(sfp[1].s);
+	knh_bytes_t p = knh_String_tobytes(sfp[1].s);
 	knh_index_t loc = knh_bytes_index(p, ':');
 	KNH_SETv(ctx, o->pattern, sfp[1].s);
 	if(loc == -1) {
@@ -177,7 +177,7 @@ METHOD Regex_new(Ctx *ctx, knh_sfp_t *sfp)
 	o->reg = o->df->regmalloc(ctx);
 	{
 		char *ptn = (char*)(knh_bytes_last(p, loc+1).buf);
-		char *opt = IS_NULL(sfp[2].o) ? "" : __tochar(sfp[2].s);
+		char *opt = IS_NULL(sfp[2].o) ? "" : knh_String_tochar(sfp[2].s);
 		o->df->regcomp(ctx, o->reg, ptn, opt);
 	}
 	KNH_RETURN(ctx, sfp, sfp[0].o);
@@ -190,9 +190,9 @@ METHOD Regex_new(Ctx *ctx, knh_sfp_t *sfp)
 static METHOD Pair_new__init(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_Pair_t *o = (knh_Pair_t*)sfp[0].o;
-	knh_stack_boxing(ctx, sfp + 1);
+	knh_sfp_boxing(ctx, sfp + 1);
 	KNH_SETv(ctx, o->first, sfp[1].o);
-	knh_stack_boxing(ctx, sfp + 2);
+	knh_sfp_boxing(ctx, sfp + 2);
 	KNH_SETv(ctx, o->second, sfp[2].o);
 	KNH_RETURN(ctx, sfp, o);
 }
@@ -206,7 +206,7 @@ static METHOD Tuple_new__init(Ctx *ctx, knh_sfp_t *sfp)
 	knh_sfp_t *v = sfp + 1;
 	int i, ac = knh_stack_argc(ctx, v);
 	for(i = 0; i < ac; i++) {
-		knh_stack_boxing(ctx, v + i);
+		knh_sfp_boxing(ctx, v + i);
 	}
 	DBG2_ASSERT(t->size == 0);
 	if(ac <= 3) {
@@ -220,9 +220,9 @@ static METHOD Tuple_new__init(Ctx *ctx, knh_sfp_t *sfp)
 	}
 	else {
 		t->size = ac;
-		t->list = (knh_Object_t**)KNH_MALLOC(ctx, sizeof(void*) * t->size);
+		t->list = KNH_MALLOC(ctx, sizeof(void*) * t->size);
 		for(i = 0; i < t->size; i++) {
-			KNH_INITv(t->list[i], v[i].o);
+			KNH_INITv(t->list[i], v[0].o);
 		}
 	}
 	KNH_RETURN(ctx, sfp, t);
@@ -234,9 +234,9 @@ static METHOD Tuple_new__init(Ctx *ctx, knh_sfp_t *sfp)
 static METHOD Range_new__init(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_Range_t *r = (knh_Range_t*)sfp[0].o;
-	knh_stack_boxing(ctx, sfp + 1);
+	knh_sfp_boxing(ctx, sfp + 1);
 	KNH_SETv(ctx, r->start, sfp[1].o);
-	knh_stack_boxing(ctx, sfp + 2);
+	knh_sfp_boxing(ctx, sfp + 2);
 	KNH_SETv(ctx, r->end, sfp[2].o);
 	KNH_RETURN(ctx, sfp, r);
 }
@@ -261,9 +261,7 @@ static METHOD Array_new(Ctx *ctx, knh_sfp_t *sfp)
 	if(init > 0) {
 		knh_Array_grow(ctx, o, init, KNH_NULL);
 	}
-	//if (!IS_NULL(sfp[1].o)) {
-	//	o->size = init;
-	//}
+	o->size = 0;
 	KNH_RETURN(ctx, sfp, o);
 }
 
@@ -287,7 +285,7 @@ static METHOD Array_new__array(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-//## @Hidden method This! Array.new:init(T1? value, ...);
+//## @Hidden method This! Array.new:init(T1 value, ...);
 
 static METHOD Array_new__init(Ctx *ctx, knh_sfp_t *sfp)
 {
@@ -298,7 +296,7 @@ static METHOD Array_new__init(Ctx *ctx, knh_sfp_t *sfp)
 		knh_Array_grow(ctx, o, ac, KNH_NULL);
 	}
 	for(i = 0; i < ac; i++) {
-		knh_stack_boxing(ctx, v + i);
+		knh_sfp_boxing(ctx, v + i);
 		knh_Array_add(ctx, o, v[i].o);
 	}
 	KNH_RETURN(ctx, sfp, o);
@@ -336,7 +334,7 @@ static METHOD IArray_new__array(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-//## @Hidden method This! IArray.new:init(Int? value, ...);
+//## @Hidden method This! IArray.new:init(Int! value, ...);
 
 static METHOD IArray_new__init(Ctx *ctx, knh_sfp_t *sfp)
 {
@@ -383,7 +381,11 @@ static METHOD FArray_new(Ctx *ctx, knh_sfp_t *sfp)
 	knh_FArray_t *o = (knh_FArray_t*)sfp[0].o;
 	long init = IS_NULL(sfp[1].o) ? KNH_FARRAY_INITSIZE: p_int(sfp[1]);
 	if(init > 0) {
-		knh_FArray_grow(ctx, o, init, KNH_FLOAT_ZERO);
+#ifndef KNH_USING_NOFLOAT
+		knh_FArray_grow(ctx, o, init, 0.0);
+#else
+		knh_FArray_grow(ctx, o, init, 0);
+#endif
 	}
 	o->size = 0;
 	KNH_RETURN(ctx, sfp, o);
@@ -397,7 +399,11 @@ static METHOD FArray_new__array(Ctx *ctx, knh_sfp_t *sfp)
 	knh_FArray_t *o = (knh_FArray_t*)sfp[0].o;
 	knh_int_t init = sfp[1].ivalue;
 	if(0 < init && init < LONG_MAX) {
-		knh_FArray_grow(ctx, o, init, KNH_FLOAT_ZERO);
+#ifndef KNH_USING_NOFLOAT
+		knh_FArray_grow(ctx, o, init, 0.0);
+#else
+		knh_FArray_grow(ctx, o, init, 0);
+#endif
 		o->size = init;
 	}
 	else if(init < 0) {
@@ -407,7 +413,7 @@ static METHOD FArray_new__array(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-//## @Hidden method This! FArray.new:init(Float? value, ...);
+//## @Hidden method This! FArray.new:init(Float! value, ...);
 
 static METHOD FArray_new__init(Ctx *ctx, knh_sfp_t *sfp)
 {
@@ -415,7 +421,11 @@ static METHOD FArray_new__init(Ctx *ctx, knh_sfp_t *sfp)
 	knh_sfp_t *v = sfp + 1;
 	int i, ac = knh_stack_argc(ctx, v);
 	if(ac > 0) {
-		knh_FArray_grow(ctx, o, ac, KNH_FLOAT_ZERO);
+#ifndef KNH_USING_NOFLOAT
+		knh_FArray_grow(ctx, o, ac, 0.0);
+#else
+		knh_FArray_grow(ctx, o, ac, 0);
+#endif
 	}
 	for(i = 0; i < ac; i++) {
 		knh_FArray_add(ctx, o, v[i].fvalue);
@@ -448,7 +458,7 @@ static METHOD DictMap_new__dictmap(Ctx *ctx, knh_sfp_t *sfp)
 	size_t i;
 	for(i = 0; i < ac; i+=2) {
 		if(IS_bString(v[i].s)) {
-			knh_stack_boxing(ctx, v + i + 1);
+			knh_sfp_boxing(ctx, v + i + 1);
 			knh_DictMap_append(ctx, o, v[i].s, v[i+1].o);
 		}
 	}
@@ -469,7 +479,7 @@ static METHOD Exception_new(Ctx *ctx, knh_sfp_t *sfp)
 	knh_write_char(ctx, cwb->w, "Exception!!");
 	if(IS_NOTNULL(sfp[1].o)) {
 		knh_write_char(ctx, cwb->w, ": ");
-		knh_write(ctx, cwb->w, __tobytes(sfp[1].s));
+		knh_write(ctx, cwb->w, knh_String_tobytes(sfp[1].s));
 	}
 	KNH_SETv(ctx, DP(o)->msg, knh_cwb_newString(ctx, cwb));
 	KNH_SETv(ctx, DP(o)->bag, sfp[2].o);
