@@ -66,6 +66,55 @@ METHOD Object_opInstanceof(Ctx *ctx, knh_sfp_t *sfp)
 	KNH_RETURN_Boolean(ctx, sfp, knh_class_instanceof(ctx, ARG_Object_cid(sfp[0]), p_cid(sfp[1])));
 }
 
+/* ------------------------------------------------------------------------ */
+
+static void knh_stack_utest(Ctx *ctx, knh_sfp_t *sfp, int result, Object *o)
+{
+	if(result) {
+		knh_printf(ctx, KNH_STDERR, "[PASS]\n");
+	}
+	else {
+		knh_printf(ctx, KNH_STDERR, "[FAILED] %O\n", o);
+	}
+}
+
+/* ------------------------------------------------------------------------ */
+//## @Hidden @NullBase method Any Object.opMust(Any it);
+
+static METHOD Object_opMust(Ctx *ctx, knh_sfp_t *sfp)
+{
+	knh_stack_utest(ctx, sfp, knh_Object_compareTo(ctx, sfp[0].o, sfp[1].o) != 0, sfp[1].o);
+	KNH_RETURN(ctx, sfp, sfp[1].o);
+}
+
+/* ------------------------------------------------------------------------ */
+//## @Hidden method Any Boolean.opMust(Any it);
+
+static METHOD Boolean_opMust(Ctx *ctx, knh_sfp_t *sfp)
+{
+	knh_stack_utest(ctx, sfp, sfp[0].bvalue, sfp[1].o);
+	KNH_RETURN(ctx, sfp, sfp[1].o);
+}
+
+/* ------------------------------------------------------------------------ */
+//## @Hidden method Any Exception.opMust(Any it);
+
+static METHOD Exception_opMust(Ctx *ctx, knh_sfp_t *sfp)
+{
+	KNH_ASSERT(IS_bString(sfp[0].s));
+	if(IS_Exception(sfp[1].o)) {
+		knh_expt_t eid =  (IS_Exception(sfp[0].o)) ?
+			DP(sfp[0].e)->eid : knh_geteid(ctx, knh_String_tobytes(sfp[0].s), EXPT_newid);
+		if(knh_expt_isa(ctx, DP(sfp[1].e)->eid, eid)) {
+			knh_stack_utest(ctx, sfp, 1, sfp[1].o);
+			KNH_RETURN(ctx, sfp, KNH_NULL);
+		}
+		knh_throwException(ctx, sfp[1].e, NULL, 0);
+	}
+	knh_stack_utest(ctx, sfp, 0, sfp[1].o);
+	KNH_RETURN(ctx, sfp, sfp[1].o);
+}
+
 /* ======================================================================== */
 /* [Comparator] */
 
@@ -222,7 +271,7 @@ static METHOD Float_opGte(Ctx *ctx, knh_sfp_t *sfp)
 static METHOD String_opHas(Ctx *ctx, knh_sfp_t *sfp)
 {
 	KNH_RETURN_Boolean(ctx, sfp,
-		knh_bytes_indexOf(__tobytes(sfp[0].s), __tobytes(sfp[1].s)) != -1);
+		knh_bytes_indexOf(knh_String_tobytes(sfp[0].s), knh_String_tobytes(sfp[1].s)) != -1);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -233,7 +282,7 @@ static METHOD Pair_opHas(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_Pair_t *t = sfp[0].pair;
 	int res;
-	knh_stack_boxing(ctx, sfp + 1);
+	knh_sfp_boxing(ctx, sfp + 1);
 	res = ((knh_Object_compareTo(ctx, t->first, sfp[1].o) == 0)
 			|| (knh_Object_compareTo(ctx, t->second, sfp[1].o) == 0));
 	KNH_RETURN_Boolean(ctx, sfp, res);
@@ -254,7 +303,7 @@ static METHOD Range_opHas(Ctx *ctx, knh_sfp_t *sfp)
 		KNH_SETv(ctx, sfp[1].o, new_Float(ctx, (knh_float_t)sfp[1].ivalue));
 	}
 	else {
-		knh_stack_boxing(ctx, sfp + 1);
+		knh_sfp_boxing(ctx, sfp + 1);
 	}
 	if(knh_Object_compareTo(ctx, o->start, sfp[1].o) <= 0) {
 		if(knh_Range_isInclusive(o)) {
@@ -278,7 +327,7 @@ static METHOD Range_opHas(Ctx *ctx, knh_sfp_t *sfp)
 static METHOD Array_opHas(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_Array_t *o = (knh_Array_t*)sfp[0].o;
-	knh_stack_boxing(ctx, sfp + 1);
+	knh_sfp_boxing(ctx, sfp + 1);
 	size_t i, res = 0/*NotFound*/;
 	for(i = 0; i < o->size; i++) {
 		if(knh_Object_compareTo(ctx, o->list[i], sfp[1].o) == 0) {
@@ -297,7 +346,7 @@ static METHOD Tuple_opHas(Ctx *ctx, knh_sfp_t *sfp)
 	if(knh_Tuple_isTriple(sfp[0].tuple)) {
 		knh_Tuple_t *t = sfp[0].tuple;
 		int res;
-		knh_stack_boxing(ctx, sfp + 1);
+		knh_sfp_boxing(ctx, sfp + 1);
 		res = ((knh_Object_compareTo(ctx, t->first, sfp[1].o) == 0)
 				|| (knh_Object_compareTo(ctx, t->second, sfp[1].o) == 0)
 				|| (knh_Object_compareTo(ctx, t->third, sfp[1].o) == 0));
@@ -409,7 +458,7 @@ static METHOD Object_opIs(Ctx *ctx, knh_sfp_t *sfp)
 		KNH_RETURN_Boolean(ctx, sfp, IS_NULL(sfp[0].o));
 	}
 	else {
-		TODO();
+		TODO_THROW(ctx);
 		KNH_RETURN_Boolean(ctx, sfp, 0);
 //		knh_sfp_t *lsfp = KNH_LOCAL(ctx);
 //		KNH_LPUSH(ctx, sfp[1].o);
@@ -430,6 +479,7 @@ static METHOD Object_opIs(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD Object_opAs(Ctx *ctx, knh_sfp_t *sfp)
 {
+	TODO_THROW(ctx);
 	KNH_RETURN_Boolean(ctx, sfp, 0);
 //	knh_class_t scid = knh_Object_cid(sfp[0].o);
 //	knh_class_t tcid = (sfp[1].c)->cid;
@@ -447,6 +497,7 @@ static METHOD Object_opAs(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD Object_opInTo(Ctx *ctx, knh_sfp_t *sfp)
 {
+	TODO_THROW(ctx);
 	KNH_RETURN_Boolean(ctx, sfp, 0);
 }
 
@@ -553,17 +604,21 @@ static METHOD Float_opDiv(Ctx *ctx, knh_sfp_t *sfp)
 static METHOD String_opAdd(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
+	knh_sfp_t *esp = KNH_LOCAL(ctx);
 	if(IS_bString(sfp[0].o)) {
-		knh_Bytes_write(ctx, cwb->ba, __tobytes(sfp[0].s));
+		knh_Bytes_write(ctx, cwb->ba, knh_String_tobytes(sfp[0].s));
 	}
 	else {
-		knh_stack_w(ctx, sfp+2, sfp, METHODN__s, cwb->w, KNH_NULL);
+		KNH_SETv(ctx, esp[1].o, sfp[0].o); esp[1].data = sfp[0].data;
+		knh_esp1_format(ctx, METHODN__s, cwb->w, KNH_NULL);
 	}
 	if(IS_bString(sfp[1].o)) {
-		knh_Bytes_write(ctx, cwb->ba, __tobytes(sfp[1].s));
+		knh_Bytes_write(ctx, cwb->ba, knh_String_tobytes(sfp[1].s));
 	}
 	else {
-		knh_stack_w(ctx, sfp+2, sfp+1, METHODN__s, cwb->w, KNH_NULL);
+		KNH_ASSERT(esp == ctx->esp);
+		KNH_SETv(ctx, esp[1].o, sfp[1].o); esp[1].data = sfp[1].data;
+		knh_esp1_format(ctx, METHODN__s, cwb->w, KNH_NULL);
 	}
 	KNH_RETURN(ctx, sfp, knh_cwb_newString(ctx, cwb));
 }
@@ -573,8 +628,8 @@ static METHOD String_opAdd(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD String_opSub(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_bytes_t base = __tobytes(sfp[0].s);
-	knh_bytes_t t = __tobytes(sfp[1].s);
+	knh_bytes_t base = knh_String_tobytes(sfp[0].s);
+	knh_bytes_t t = knh_String_tobytes(sfp[1].s);
 	knh_uchar_t c = t.buf[0];
 	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
 	size_t i;
@@ -603,25 +658,11 @@ static METHOD String_opSub(Ctx *ctx, knh_sfp_t *sfp)
 /* ------------------------------------------------------------------------ */
 /* [Any] */
 //## method Any Any.opAdd(Any value);
-//## method Any Any.opSub(Any value);
-//## method Any Any.opMul(Any value);
-//## method Any Any.opDiv(Any value);
-//## method Any Any.opMod(Any value);
 
 METHOD Any_opAdd(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_Method_t *mtd = knh_lookupMethod(ctx, knh_Object_cid(sfp[0].o), DP(sfp[-1].mtd)->mn);
-	KNH_SETv(ctx, sfp[-1].mtd, mtd);
-	knh_stack_typecheck(ctx, sfp, mtd, NULL);
-	(sfp[-1].mtd)->fcall_1(ctx, sfp);
-#ifdef KNH_USING_UNBOXFIELD
-	{
-	  knh_type_t rtype = knh_Method_rztype(mtd);
-	  if (IS_ubxtype(rtype)) {
-	    KLR_BOX(ctx, -1, CLASS_type(rtype));
-	  }
-	}
-#endif/*KNU_USING_UNBOXFIED*/
+	TODO_THROW(ctx);
+	KNH_RETURN(ctx, sfp, KNH_NULL);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -629,18 +670,44 @@ METHOD Any_opAdd(Ctx *ctx, knh_sfp_t *sfp)
 
 METHOD Any_opNeg(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_Method_t *mtd = knh_lookupMethod(ctx, knh_Object_cid(sfp[0].o), DP(sfp[-1].mtd)->mn);
-	KNH_SETv(ctx, sfp[-1].mtd, mtd);
-	knh_stack_typecheck(ctx, sfp, mtd, NULL);
-	(sfp[-1].mtd)->fcall_1(ctx, sfp);
-#ifdef KNH_USING_UNBOXFIELD
-	{
-	  knh_type_t rtype = knh_Method_rztype(mtd);
-	  if (IS_ubxtype(rtype)) {
-	    KLR_BOX(ctx, -1, CLASS_type(rtype));
-	  }
-	}
-#endif/*KNU_USING_UNBOXFIED*/
+	TODO_THROW(ctx);
+	KNH_RETURN(ctx, sfp, KNH_NULL);
+}
+
+/* ------------------------------------------------------------------------ */
+//## method Any Any.opSub(Any value);
+
+METHOD Any_opSub(Ctx *ctx, knh_sfp_t *sfp)
+{
+	TODO_THROW(ctx);
+	KNH_RETURN(ctx, sfp, KNH_NULL);
+}
+
+/* ------------------------------------------------------------------------ */
+//## method Any Any.opMul(Any value);
+
+METHOD Any_opMul(Ctx *ctx, knh_sfp_t *sfp)
+{
+	TODO_THROW(ctx);
+	KNH_RETURN(ctx, sfp, KNH_NULL);
+}
+
+/* ------------------------------------------------------------------------ */
+//## method Any Any.opDiv(Any value);
+
+METHOD Any_opDiv(Ctx *ctx, knh_sfp_t *sfp)
+{
+	TODO_THROW(ctx);
+	KNH_RETURN(ctx, sfp, KNH_NULL);
+}
+
+/* ------------------------------------------------------------------------ */
+//## method Any Any.opMod(Any value);
+
+METHOD Any_opMod(Ctx *ctx, knh_sfp_t *sfp)
+{
+	TODO_THROW(ctx);
+	KNH_RETURN(ctx, sfp, KNH_NULL);
 }
 
 /* ======================================================================== */
@@ -738,7 +805,11 @@ static METHOD Int_opPrev(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD Float_opNext(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_RETURN_Float(ctx, sfp, (p_float(sfp[0]))+KNH_FLOAT_ONE);
+  #ifdef KNH_USING_NOFLOAT
+	KNH_RETURN_Float(ctx, sfp, (p_float(sfp[0]))+1);
+  #else
+	KNH_RETURN_Float(ctx, sfp, (p_float(sfp[0]))+1.0);
+  #endif
 }
 
 /* ------------------------------------------------------------------------ */
@@ -746,7 +817,11 @@ static METHOD Float_opNext(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD Float_opPrev(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_RETURN_Float(ctx, sfp, (p_float(sfp[0]))-KNH_FLOAT_ONE);
+#ifdef KNH_USING_NOFLOAT
+	KNH_RETURN_Float(ctx, sfp, (p_float(sfp[0])) - 1);
+#else
+	KNH_RETURN_Float(ctx, sfp, (p_float(sfp[0]))-1.0);
+#endif
 }
 
 /* ======================================================================== */
@@ -791,13 +866,9 @@ static METHOD Bytes_getSize(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD String_getSize(Ctx *ctx, knh_sfp_t *sfp)
 {
-	size_t size = 0;
-	if (IS_bString(sfp[0].s)) {
-		if(knh_String_isAscii(sfp[0].s)) {
-			size =  (sfp[0].s)->size;
-		} else {
-			size = knh_bytes_mlen(__tobytes(sfp[0].s));
-		}
+	size_t size = IS_bString(sfp[0].s) ? (sfp[0].s)->size : 0;
+	if(!knh_String_isAscii(sfp[0].s)) {
+		size = knh_bytes_mlen(knh_String_tobytes(sfp[0].s));
 	}
 	KNH_RETURN_Int(ctx, sfp, size);
 }
@@ -897,7 +968,7 @@ static METHOD Bytes_setAll(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD String_get(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_bytes_t base = __tobytes(sfp[0].s);
+	knh_bytes_t base = knh_String_tobytes(sfp[0].s);
 	knh_String_t *s;
 	if(knh_String_isAscii(sfp[0].s)) {
 		size_t n = knh_array_index(ctx, p_int(sfp[1]), knh_String_strlen(sfp[0].s));
@@ -970,7 +1041,7 @@ static METHOD Array_set(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_Array_t *o = (knh_Array_t*)sfp[0].o;
 	size_t n2 = knh_array_index(ctx, p_int(sfp[1]), o->size);
-	knh_stack_boxing(ctx, sfp + 2);
+	knh_sfp_boxing(ctx, sfp + 2);
 	KNH_SETv(ctx, o->list[n2], sfp[2].o);
 	KNH_RETURN_void(ctx, sfp);
 }
@@ -1004,7 +1075,7 @@ static METHOD Array_setAll(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_Array_t *o = (knh_Array_t*)sfp[0].o;
 	size_t i;
-	knh_stack_boxing(ctx, sfp + 1);
+	knh_sfp_boxing(ctx, sfp + 1);
 	for(i = 0; i < o->size; i++) {
 		KNH_SETv(ctx, o->list[i], sfp[1].o);
 	}
@@ -1109,7 +1180,7 @@ static METHOD Bytes_opRangeTo(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD String_substring(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_bytes_t base = __tobytes(sfp[0].s);
+	knh_bytes_t base = knh_String_tobytes(sfp[0].s);
 	knh_bytes_t sub;
 	if(knh_String_isAscii(sfp[0].s)) {
 		size_t offset = IS_NULL(sfp[1].o) ? 0 : knh_array_index(ctx, sfp[1].ivalue, base.len);
@@ -1266,15 +1337,15 @@ int knh_fbytescmp_1(knh_bytes_t t1, knh_bytes_t t2)
 static
 knh_DictMap_t *knh_DictMap_subset(Ctx *ctx, knh_DictMap_t *d, knh_String_t* ss, knh_String_t *es, int isInclusive)
 {
-	knh_bytes_t s = IS_bString(ss) ? __tobytes(ss) : STEXT("");
-	knh_bytes_t e = IS_bString(es) ? __tobytes(es) : STEXT("");
+	knh_bytes_t s = IS_bString(ss) ? knh_String_tobytes(ss) : STEXT("");
+	knh_bytes_t e = IS_bString(es) ? knh_String_tobytes(es) : STEXT("");
 	size_t i, dsize = d->size;
 	knh_DictMap_t *newd = new_DictMap(ctx, knh_Object_p1(d), 0);
 	int zero = (isInclusive) ?  0 : 1;
 	knh_fbytescmp f1 = (s.len > 0) ? d->fcmp : knh_fbytescmp_1;
 	knh_fbytescmp f2 = (e.len > 0) ? d->fcmp : knh_fbytescmp_1;
 	for(i = 0; i < dsize; i++) {
-		knh_bytes_t key = __tobytes(knh_DictMap_keyAt(d, i));
+		knh_bytes_t key = knh_String_tobytes(knh_DictMap_keyAt(d, i));
 		if(f1(key, s) <= 0 && f2(e, key) >= zero) {
 			knh_DictMap_append(ctx, newd, knh_DictMap_keyAt(d, i), knh_DictMap_valueAt(d, i));
 		}
