@@ -41,22 +41,11 @@ extern "C" {
 
 /* ------------------------------------------------------------------------ */
 //## method InputStream InputStream.new(String! urn, String? mode);
-	
+
 static METHOD InputStream_new(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_InputStream_t *in = (knh_InputStream_t*)sfp[0].o;
-	in = (knh_InputStream_t*) knh_InputStream_open(ctx, in, sfp[1].s, sfp[2].s);
-#if defined(KNH_USING_WINDOWS)
-	knh_String_t *enc;
-	if(knh_InputStream_checkUTF8(ctx, in)) {
-		enc = new_String(ctx, STEXT("UTF-8"), NULL);
-	}
-	else {
-		enc = new_String(ctx, STEXT("CP932"), NULL);
-	}
-	knh_InputStream_setEncoding(ctx, in, enc);
-#endif
-	KNH_RETURN(ctx, sfp, UP(in));
+	knh_InputStream_t *o = (knh_InputStream_t*)sfp[0].o;
+	KNH_RETURN(ctx, sfp, knh_InputStream_open(ctx, o, sfp[1].s, sfp[2].s));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -231,7 +220,7 @@ static METHOD OutputStream_isClosed(Ctx *ctx, knh_sfp_t *sfp)
 static METHOD OutputStream_print(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_OutputStream_t *out = (knh_OutputStream_t*)sfp[0].o;
-	knh_sfp_t *v = sfp + 1;
+	knh_sfp_t *v = sfp + 1, *esp1 = ctx->esp + 1;
 	int ac = knh_stack_argc(ctx, v);
 	size_t i;
 	for(i = 0; i < ac; i++) {
@@ -246,13 +235,19 @@ static METHOD OutputStream_print(Ctx *ctx, knh_sfp_t *sfp)
 				DP(out)->indent--;
 			}
 			else {
-				knh_OutputStream_print_(ctx, out, __tobytes(v[i].s), 0);
+				knh_OutputStream_print_(ctx, out, knh_String_tobytes(v[i].s), 0);
 			}
 		}
 		else {
-			knh_stack_w(ctx, v + ac, v+i, METHODN__s, out, KNH_NULL);
+			KNH_SETv(ctx, esp1[0].o, v[i].o);
+			esp1[0].data = v[i].data;
+			KNH_ASSERT(ctx->esp + 1 == esp1);
+			knh_esp1_format(ctx, METHODN__s, out, KNH_NULL);
 		}
 	}
+	if(knh_OutputStream_isAutoFlush(out)) {
+		knh_OutputStream_flush(ctx, out);
+	}
 	KNH_RETURN_void(ctx, sfp);
 }
 
@@ -261,25 +256,59 @@ static METHOD OutputStream_print(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD OutputStream_println(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_SETv(ctx, ctx->esp[0].o, TS_EOL);
-	KNH_SETESP(ctx, (ctx->esp+1));
-	OutputStream_print(ctx, sfp);
+	knh_OutputStream_t *out = (knh_OutputStream_t*)sfp[0].o;
+	knh_sfp_t *v = sfp + 1, *esp1 = ctx->esp + 1;
+	int ac = knh_stack_argc(ctx, v);
+	size_t i;
+	for(i = 0; i < ac; i++) {
+		if(IS_bString(v[i].o)) {
+			if(v[i].s == TS_EOL) {
+				knh_write_EOL(ctx, out);
+			}
+			else if(v[i].s == TS_BEGIN) {
+				DP(out)->indent++;
+			}
+			else if(v[i].s == TS_END) {
+				DP(out)->indent--;
+			}
+			else {
+				knh_OutputStream_print_(ctx, out, knh_String_tobytes(v[i].s), 0);
+			}
+		}
+		else {
+			KNH_SETv(ctx, esp1[0].o, v[i].o);
+			esp1[0].data = v[i].data;
+			DBG2_ASSERT(ctx->esp + 1 == esp1);
+			knh_esp1_format(ctx, METHODN__s, out, KNH_NULL);
+		}
+	}
+	knh_write_EOL(ctx, out);
+	if(knh_OutputStream_isAutoFlush(out)) {
+		knh_OutputStream_flush(ctx, out);
+	}
+	KNH_RETURN_void(ctx, sfp);
 }
 
 /* ------------------------------------------------------------------------ */
-//## method void OutputStream.writeData(Any? data, ...);
+//## method void OutputStream.writeData(Any data, ...);
 
 static METHOD OutputStream_writeData(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_OutputStream_t *out = (knh_OutputStream_t*)sfp[0].o;
-	knh_sfp_t *v = sfp + 1;
+	knh_sfp_t *v = sfp + 1, *esp1 = ctx->esp + 1;
 	int i, ac = knh_stack_argc(ctx, v);
 	knh_intptr_t indent = DP(out)->indent;
 	for(i = 0; i < ac; i++) {
+		KNH_SETv(ctx, esp1[0].o, v[i].o);
+		esp1[0].data = v[i].data;
+		DBG2_ASSERT(ctx->esp + 1 == esp1);
 		DP(out)->indent = 0;
-		knh_stack_w(ctx, v + ac, v+i, METHODN__data, out, KNH_NULL);
-		knh_putc(ctx, out, ';');
-		knh_write_EOL(ctx, out);
+		knh_esp1_format(ctx, METHODN__data, out, KNH_NULL);
+	}
+	knh_putc(ctx, out, ';');
+	knh_write_EOL(ctx, out);
+	if(knh_OutputStream_isAutoFlush(out)) {
+		knh_OutputStream_flush(ctx, out);
 	}
 	DP(out)->indent = indent;
 	KNH_RETURN_void(ctx, sfp);
@@ -338,7 +367,7 @@ static METHOD OutputStream_setEncoding(Ctx *ctx, knh_sfp_t *sfp)
 METHOD Socket_new(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_Socket_t *so = (knh_Socket_t*)sfp[0].o;
-	knh_bytes_t urn = __tobytes(sfp[1].s);
+	knh_bytes_t urn = knh_String_tobytes(sfp[1].s);
 	char *ip_or_host = NULL;
 	if(knh_bytes_startsWith(urn, STEXT("socket:"))) {
 		ip_or_host = (char*)(knh_bytes_last(urn, 7)).buf;
