@@ -29,7 +29,6 @@
 
 #include"commons.h"
 #include"../../include/konoha/konoha_stmt_.h"
-#include"../../include/konoha/konoha_code_.h"
 
 /* ************************************************************************ */
 
@@ -138,15 +137,16 @@ typedef struct {
 /* ------------------------------------------------------------------------ */
 
 static
-void knh_ClassField_initField(Ctx *ctx, knh_ClassField_t *cs, knh_class_t self_cid, Object **v)
+void knh_ClassStruct_initField(Ctx *ctx, knh_ClassStruct_t *cs, knh_class_t self_cid, Object **v)
 {
 	size_t i;
 	knh_cfield_t *cf = cs->fields;
 	for(i = 0; i < cs->fsize; i++) {
 		knh_type_t type = cf[i].type;
 		Object *value = cf[i].value;
-		if(KNH_FLAG_IS(cf[i].flag, FLAG_ClassField_Property)) {
-			value = knh_getProperty(ctx, __tobytes((knh_String_t*)value));
+		if(KNH_FLAG_IS(cf[i].flag, FLAG_ClassStruct_Property)) {
+			value = knh_Context_getProperty(ctx, (knh_Context_t*)ctx,
+				knh_String_tobytes((knh_String_t*)value));
 			DBG2_P("type=%s%s object=%s", TYPEQN(cf[i].type), CLASSNo(value));
 		}
 #ifdef KNH_USING_UNBOXFIELD
@@ -160,7 +160,11 @@ void knh_ClassField_initField(Ctx *ctx, knh_ClassField_t *cs, knh_class_t self_c
 		}
 		else if(IS_ubxfloat(type)) {
 			knh_float_t *data = (knh_float_t*)(v + i);
-			data[0] = value == NULL ? KNH_FLOAT_ZERO : tofloat(value);
+#ifdef KNH_USING_NOFLOAT
+			data[0] = value == NULL ? 0 : tofloat(value);
+#else
+			data[0] = value == NULL ? 0.0 : tofloat(value);
+#endif
 			continue;
 		}
 		else if(IS_ubxboolean(type)) {
@@ -190,11 +194,11 @@ void knh_ObjectField_init(Ctx *ctx, knh_ObjectField_t *of, int init)
 		Object **v = (Object**)of->fields;
 		size_t offset;
 		while((offset = ClassTable(cid).offset) != 0) {
-			knh_ClassField_initField(ctx, ClassTable(cid).cstruct, of->h.cid, v + offset);
+			knh_ClassStruct_initField(ctx, ClassTable(cid).cstruct, of->h.cid, v + offset);
 			cid = ClassTable(cid).supcid;
-			DBG2_ASSERT_cid(cid);
+			KNH_ASSERT_cid(cid);
 		}
-		knh_ClassField_initField(ctx, ClassTable(cid).cstruct, of->h.cid, v + offset);
+		knh_ClassStruct_initField(ctx, ClassTable(cid).cstruct, of->h.cid, v + offset);
 		of->fields = v;
 		of->bsize = ctx->share->ClassTable[of->h.cid].bsize;
 	}
@@ -210,7 +214,7 @@ void knh_ObjectField_traverse(Ctx *ctx, knh_ObjectField_t *of, knh_ftraverse ftr
 {
 	knh_class_t cid = knh_Object_cid(of);
 	while(cid != CLASS_Object) {
-		knh_ClassField_t *cs = ClassTable(cid).cstruct;
+		knh_ClassStruct_t *cs = ClassTable(cid).cstruct;
 		size_t i, offset = ClassTable(cid).offset;
 		for(i = 0; i < cs->fsize; i++) {
 			knh_type_t type = cs->fields[i].type;
@@ -405,7 +409,7 @@ static int knh_Float_compareTo(Ctx *ctx, knh_Float_t *o, knh_Float_t *o2)
 
 static void knh_String_init(Ctx *ctx, knh_String_t *s, int init)
 {
-	s->str = NULL;
+	s->str = (knh_uchar_t*)""; // for safety
 	s->size = 0;
 	s->orign = NULL;
 }
@@ -416,7 +420,7 @@ static void knh_String_traverse(Ctx *ctx, knh_String_t *s, knh_ftraverse ftr)
 		ftr(ctx, UP(s->orign));
 	}
 	else if(!knh_String_isTextSgm(s)) {
-		if(IS_SWEEP(ftr) && s->str) {
+		if(IS_SWEEP(ftr)) {
 			KNH_FREE(ctx, s->str, KNH_SIZE(s->size + 1));
 			s->str = NULL;
 		}
@@ -447,7 +451,7 @@ static int knh_String_compareTo(Ctx *ctx, knh_String_t *o, knh_String_t *o2)
 	else {
 		if(o->h.cid == o2->h.cid) {
 			knh_ClassSpec_t *u = knh_getClassSpec(ctx, o->h.cid);
-			return DP(u)->fscmp(u, __tobytes(o), __tobytes(o2));
+			return DP(u)->fscmp(u, knh_String_tobytes(o), knh_String_tobytes(o2));
 		}
 		return (int)(o - o2);
 	}
@@ -572,7 +576,7 @@ static MAPPER Range_Iterator(Ctx *ctx, knh_sfp_t *sfp);
 
 static void knh_Range_newClass(Ctx *ctx, knh_class_t cid)
 {
-	DBG2_ASSERT_cid(cid);
+	KNH_ASSERT_cid(cid);
 	knh_class_t p1 = ClassTable(cid).p1;
 	knh_class_t icid = knh_class_Iterator(ctx, p1);
 	DBG2_P("*** %s, %s", CLASSN(p1), CLASSN(icid));
@@ -620,7 +624,7 @@ static void knh_Array_traverse(Ctx *ctx, knh_Array_t *a, knh_ftraverse ftr)
 
 static void knh_Array_newClass(Ctx *ctx, knh_class_t cid)
 {
-	DBG2_ASSERT_cid(cid);
+	KNH_ASSERT_cid(cid);
 	knh_class_t p1 = ClassTable(cid).p1;
 	knh_class_t icid = knh_class_Iterator(ctx, p1);
 	DBG2_P("*** %s, %s", CLASSN(p1), CLASSN(icid));
@@ -757,7 +761,7 @@ static MAPPER Iterator_Iterator(Ctx *ctx, knh_sfp_t *sfp)
 static void knh_Iterator_newClass(Ctx *ctx, knh_class_t cid)
 {
 	if(cid != CLASS_Iterator) {
-		DBG2_ASSERT_cid(cid);
+		KNH_ASSERT_cid(cid);
 		DBG2_P("*** %s", CLASSN(cid));
 		knh_addMapperFunc(ctx, FLAG_Mapper_Iteration, CLASS_Iterator, cid, Iterator_Iterator, KNH_NULL);
 	}
@@ -990,23 +994,23 @@ static void knh_Class_newClass(Ctx *ctx, knh_class_t cid)
 static knh_String_t *knh_Class_getkey(Ctx *ctx,knh_sfp_t *lsfp)
 {
 	knh_Class_t *c = (knh_Class_t*)lsfp[0].o;
-	DBG2_ASSERT_cid(c->cid);
+	KNH_ASSERT_cid(c->cid);
 	return ClassTable(c->cid).lname;
 }
 
 /* ======================================================================== */
-/* ClassField */
+/* ClassStruct */
 
-#define knh_ClassField_init_ NULL
-#define knh_ClassField_copy NULL
-#define knh_ClassField_traverse_ NULL
-#define knh_ClassField_compareTo NULL
-#define knh_ClassField_hashCode NULL
-#define knh_ClassField_newClass NULL
-#define knh_ClassField_getkey NULL
+#define knh_ClassStruct_init_ NULL
+#define knh_ClassStruct_copy NULL
+#define knh_ClassStruct_traverse_ NULL
+#define knh_ClassStruct_compareTo NULL
+#define knh_ClassStruct_hashCode NULL
+#define knh_ClassStruct_newClass NULL
+#define knh_ClassStruct_getkey NULL
 
 static
-void knh_ClassField_init(Ctx *ctx, knh_ClassField_t *b, int init)
+void knh_ClassStruct_init(Ctx *ctx, knh_ClassStruct_t *b, int init)
 {
 	b->fsize = init;
 	if(b->fsize == 0) {
@@ -1024,7 +1028,7 @@ void knh_ClassField_init(Ctx *ctx, knh_ClassField_t *b, int init)
 	KNH_INITv(b->methods, KNH_NULL);
 }
 
-static void knh_ClassField_traverse(Ctx *ctx, knh_ClassField_t *b, knh_ftraverse ftr)
+static void knh_ClassStruct_traverse(Ctx *ctx, knh_ClassStruct_t *b, knh_ftraverse ftr)
 {
 	ftr(ctx, UP(b->methods));
 	if(b->fields != NULL) {
@@ -1124,8 +1128,8 @@ static void knh_Method_init(Ctx *ctx, knh_Method_t *mtd, int init)
 	KNH_INITv(b->mf, knh_findMethodField0(ctx, TYPE_void));
 	b->code  = NULL;
 	b->uri  = 0;  b->domain = 0;
-//	b->prof_count = 0;
-//	b->prof_time = 0;
+	b->prof_count = 0;
+	b->prof_time = 0;
 }
 
 static void knh_Method_traverse(Ctx *ctx, knh_Method_t *mtd, knh_ftraverse ftr)
@@ -1256,40 +1260,6 @@ static void knh_Closure_traverse(Ctx *ctx, knh_Closure_t *cc, knh_ftraverse ftr)
 	}
 }
 
-/* ======================================================================== */
-/* Thunk */
-
-#define knh_Thunk_init_ NULL
-#define knh_Thunk_copy NULL
-#define knh_Thunk_traverse_ NULL
-#define knh_Thunk_compareTo NULL
-#define knh_Thunk_hashCode NULL
-#define knh_Thunk_newClass NULL
-#define knh_Thunk_getkey NULL
-
-static void knh_Thunk_init(Ctx *ctx, knh_Thunk_t *thk, int init)
-{
-	thk->envsfp = NULL;
-	thk->envsize = 0;
-	knh_Thunk_setEvaluated(thk, 0);
-}
-
-static void knh_Thunk_traverse(Ctx *ctx, knh_Thunk_t *thk, knh_ftraverse ftr)
-{
-	if(thk->envsize > 0) {
-		size_t i;
-		for(i = 0; i < (thk)->envsize; i++) {
-			ftr(ctx, (thk)->envsfp[i].o);
-		}
-		if(IS_SWEEP(ftr)) {
-			KNH_FREE(ctx, (thk)->envsfp, (sizeof(knh_sfp_t) * (thk)->envsize));
-			(thk)->envsfp = NULL;
-			(thk)->envsize = 0;
-			knh_Thunk_setEvaluated(thk, 0);
-		}
-	}
-}
-
 /* ======================================================================== */
 /* AffineConv */
 
@@ -1303,8 +1273,13 @@ static void knh_Thunk_traverse(Ctx *ctx, knh_Thunk_t *thk, knh_ftraverse ftr)
 
 static void knh_AffineConv_init(Ctx *ctx, knh_AffineConv_t *o, int init)
 {
-	o->scale = KNH_FLOAT_ONE;
-	o->shift = KNH_FLOAT_ZERO;
+#ifndef KNH_USING_NOFLOAT
+	o->scale = 1.0;
+	o->shift = 0.0;
+#else
+	o->scale = 1;
+	o->shift = 0;
+#endif
 }
 
 /* ======================================================================== */
@@ -1407,7 +1382,11 @@ static
 int knh_ffcmp__default(knh_ClassSpec_t *u, knh_float_t v1, knh_float_t v2)
 {
 	knh_float_t delta = v1 - v2;
-	if(delta == KNH_FLOAT_ZERO) return 0;
+#ifndef KNH_USING_NOFLOAT
+	if(delta == 0.0) return 0;
+#else
+	if(delta == 0) return 0;
+#endif
 	return delta < 0 ? -1 : 1;
 }
 
@@ -1981,7 +1960,7 @@ static void knh_System_newClass(Ctx *ctx, knh_class_t cid)
 void knh_Context_init(Ctx *ctx, knh_Context_t *o, int init)
 {
 	/* Context can be only instatiated in new_Context() */
-	KNH_THROW__T(ctx, "Abort!!");
+	KNH_THROWs(ctx, "Abort!!");
 }
 
 static Object *knh_Context_fdefault(Ctx *ctx, knh_class_t cid)
@@ -2035,7 +2014,7 @@ static void knh_Stmt_init(Ctx *ctx, knh_Stmt_t *stmt, int init)
 {
 	knh_Stmt_struct *b = DP(stmt);
 	b->type = TYPE_var;
-	b->esp = 0;
+	b->used_stack = 0;
 	b->size = 0;
 	b->capacity = init;
 	if(b->capacity > 0) {
@@ -2081,118 +2060,113 @@ Term *knh_Stmt_done(Ctx *ctx, knh_Stmt_t *o)
 {
 	SP(o)->stt = STT_DONE;
 	if(DP(o)->terms != NULL) {
-#ifdef KNH_USING_RCGC
 		knh_Stmt_terms_traverse(ctx, DP(o), knh_Object_sweep);
-#endif
 	}
 	KNH_SETv(ctx, DP(o)->metaDictMap, KNH_NULL);
 	return (Term*)(o);
 }
 
 /* ======================================================================== */
-/* Gamma */
+/* Asm */
 
-#define knh_Gamma_init_ NULL
-#define knh_Gamma_copy NULL
-#define knh_Gamma_traverse_ NULL
-#define knh_Gamma_compareTo NULL
-#define knh_Gamma_hashCode NULL
-#define knh_Gamma_newClass NULL
-#define knh_Gamma_getkey NULL
+#define knh_Asm_init_ NULL
+#define knh_Asm_copy NULL
+#define knh_Asm_traverse_ NULL
+#define knh_Asm_compareTo NULL
+#define knh_Asm_hashCode NULL
+#define knh_Asm_newClass NULL
+#define knh_Asm_getkey NULL
 
-static void knh_Gamma_init(Ctx *ctx, knh_Gamma_t *kc, int init)
+static void knh_Asm_init(Ctx *ctx, knh_Asm_t *abr, int init)
 {
-	knh_Gamma_struct *b = DP(kc);
+	knh_Asm_struct *b = DP(abr);
 	b->flag = 0;
 	b->this_cid = CLASS_Object;
+
 	KNH_INITv(b->ns, ctx->share->mainns);
-	DBG2_ASSERT(IS_NameSpace(b->ns));
 	KNH_INITv(b->mtd,   KNH_NULL);
+	b->level = 0;
 
 	knh_intptr_t i;
-	b->gamma = (knh_cfield_t*)KNH_MALLOC(ctx, K_GAMMASIZE * sizeof(knh_cfield_t));
-	knh_bzero(b->gamma, K_GAMMASIZE * sizeof(knh_cfield_t));
-	for(i = 0; i < K_GAMMASIZE; i++) {
+	b->gamma = (knh_cfield_t*)KNH_MALLOC(ctx, KONOHA_LOCALSIZE * sizeof(knh_cfield_t));
+	knh_bzero(b->gamma, KONOHA_LOCALSIZE * sizeof(knh_cfield_t));
+	b->gamma_size = 0;
+	for(i = 0; i < KONOHA_LOCALSIZE; i++) {
 		b->gamma[i].flag  = 0;
 		b->gamma[i].type  = TYPE_void;
 		b->gamma[i].fn    = FIELDN_NONAME;
 		b->gamma[i].value = NULL;
 	}
-	b->psize = 0;
+	b->stack = 0;
+	b->globalidx = -1;
 
-	KNH_INITv(b->lstacks, new_Array0(ctx, 0));
-	KNH_INITv(b->insts, new_Array0(ctx, 256));
-	KNH_INITv(b->decls, new_Array0(ctx, 0));
-	KNH_INITv(b->untypes, new_Array0(ctx, 0));
+	b->xflag = 0;
+	b->xrtype = 0;
+	b->xstack_size = 0;
+	KNH_INITv(b->xmtd, KNH_NULL);
+
+	b->regs = (knh_asmreg_t*)KNH_MALLOC(ctx, KNH_ASM_REGMAX * sizeof(knh_asmreg_t));
+	knh_bzero(b->regs, KNH_ASM_REGMAX * sizeof(knh_asmreg_t));
+	b->regs_size = 0;
+
+	b->labels = NULL;
+	b->labelmax = 0;
+	b->labelcapacity = 0;
+	KNH_INITv(b->name2labelIdDictSet, new_DictSet(ctx, 256));
+
+	KNH_INITv(b->lstacks, new_Array(ctx, CLASS_String, 8));
 	KNH_INITv(b->finallyStmt, KNH_NULL);
 
+	b->uri = 0;
+	b->line = 0;
+	KNH_INITv(b->elf, new_Bytes(ctx, 4096));
+	KNH_INITv(b->dwarf, new_Bytes(ctx, 1024));
+
 	b->dlhdr = NULL;
 	KNH_INITv(b->symbolDictMap, new_DictMap0(ctx, 256));
 	KNH_INITv(b->constPools, KNH_NULL);
+	KNH_INITv(b->exportsMethods, KNH_NULL);
 }
 
-static void knh_Gamma_traverse(Ctx *ctx, knh_Gamma_t *kc, knh_ftraverse ftr)
+static void knh_Asm_traverse(Ctx *ctx, knh_Asm_t *abr, knh_ftraverse ftr)
 {
+	knh_Asm_struct *b = DP(abr);
 	size_t i;
-	knh_Gamma_struct *b = DP(kc);
-	for(i = 0; i < K_GAMMASIZE; i++) {
+	for(i = 0; i < KONOHA_LOCALSIZE; i++) {
 		if(b->gamma[i].value != NULL) {
 			ftr(ctx, b->gamma[i].value);
 		}
 	}
-	if(IS_SWEEP(ftr)) {
-		KNH_FREE(ctx, b->gamma, K_GAMMASIZE * sizeof(knh_cfield_t));
-		b->dlhdr = NULL;
+	for(i = 0; i < b->labelcapacity; i++) {
+		ftr(ctx, UP(b->labels[i].tklabel));
 	}
+	if(IS_SWEEP(ftr)) {
+		KNH_FREE(ctx, b->gamma, KONOHA_LOCALSIZE * sizeof(knh_cfield_t));
+		if(b->labels != NULL) {
+			KNH_FREE(ctx, b->labels, b->labelcapacity * sizeof(knh_labeltbl_t));
+		}
+		KNH_FREE(ctx, b->regs, KNH_ASM_REGMAX * sizeof(knh_asmreg_t));
+		b->regs_size = 0;
+	}
+
 	ftr(ctx, UP(b->ns));
 	ftr(ctx, UP(b->mtd));
+	ftr(ctx, UP(b->xmtd));
+	ftr(ctx, UP(b->elf));
+	ftr(ctx, UP(b->dwarf));
+
+	ftr(ctx, UP(b->name2labelIdDictSet));
 	ftr(ctx, UP(b->lstacks));
-	ftr(ctx, UP(b->insts));
-	ftr(ctx, UP(b->decls));
-	ftr(ctx, UP(b->untypes));
 	ftr(ctx, UP(b->finallyStmt));
+
+	if(IS_SWEEP(ftr) && b->dlhdr != NULL) {
+		b->dlhdr = NULL;
+	}
 	ftr(ctx, UP(b->symbolDictMap));
+	ftr(ctx, UP(b->exportsMethods));
 	ftr(ctx, UP(b->constPools));
 }
 
-/* ======================================================================== */
-/* KLRInst */
-
-#define knh_KLRInst_init_ NULL
-#define knh_KLRInst_copy NULL
-#define knh_KLRInst_traverse_ NULL
-#define knh_KLRInst_compareTo NULL
-#define knh_KLRInst_hashCode NULL
-#define knh_KLRInst_newClass NULL
-#define knh_KLRInst_getkey NULL
-
-static void knh_KLRInst_init(Ctx *ctx, knh_KLRInst_t *inst, int opcode)
-{
-	if(opcode < OPCODE_NOP) {
-		size_t size = knh_opcode_size(opcode);
-		inst->op = (knh_inst_t*)KNH_MALLOC(ctx, size);
-		knh_bzero(inst->op, size);
-		inst->op->opcode = opcode;
-	}
-	else {
-		inst->op = NULL;
-	}
-	inst->opcode = opcode;
-	inst->line   = 0;
-	inst->code_pos = NULL;
-}
-
-static void knh_KLRInst_traverse(Ctx *ctx, knh_KLRInst_t *inst, knh_ftraverse ftr)
-{
-	if(inst->opcode <= OPCODE_NOP) {
-		knh_opcode_traverse(ctx, inst->op, ftr);
-		if(IS_SWEEP(ftr)) {
-			KNH_FREE(ctx, inst->op, knh_opcode_size(inst->opcode));
-			inst->op = NULL;
-		}
-	}
-}
-
 /* ======================================================================== */
 /* KLRCode */
 
@@ -2204,30 +2178,33 @@ static void knh_KLRInst_traverse(Ctx *ctx, knh_KLRInst_t *inst, knh_ftraverse ft
 #define knh_KLRCode_newClass NULL
 #define knh_KLRCode_getkey NULL
 
+void knh_code_traverse(Ctx *ctx, knh_code_t *pc, knh_ftraverse ftr);
+
 static void knh_KLRCode_init(Ctx *ctx, knh_KLRCode_t *o, int init)
 {
 	knh_KLRCode_struct *b = DP(o);
 	b->size = 0;
 	b->code = (knh_code_t*)"";
 	b->uri = 0;
-	b->dwarf_size  = 0;
-	b->dwarf2_size = 0;
+//	b->nsid = 0;
+	b->dwarf = NULL;
+	b->dsize = 0;
 }
 
 static void knh_KLRCode_traverse(Ctx *ctx, knh_KLRCode_t *o, knh_ftraverse ftr)
 {
 	knh_KLRCode_struct *b = DP(o);
-	knh_code_t *pc = DP(o)->code;
-	while(KNH_OPCODE(pc) != OPCODE_HALT) {
-		knh_opcode_traverse(ctx, (knh_inst_t*)pc, ftr);
-		pc += knh_opcode_size(KNH_OPCODE(pc));
-	}
+	knh_code_traverse(ctx, b->code, ftr);
 	if(IS_SWEEP(ftr)) {
 		KNH_FREE(ctx, b->code, KNH_SIZE(b->size));
+		if(b->dwarf != NULL) {
+			KNH_FREE(ctx, b->dwarf, b->dsize * sizeof(knh_dwarf_t));
+		}
 	}
 }
 
 /* ======================================================================== */
+/* [commons] */
 
 #ifdef __cplusplus
 }
@@ -2412,7 +2389,7 @@ static void knh_loadClassData(Ctx *ctx, knh_ClassData_t *data)
 			t->bsize  = t->size / sizeof(knh_Object_t*);
 			DBG2_ASSERT(t->cstruct == NULL);
 			if(data->method_size + data->formatter_size > 0) {
-				KNH_INITv(t->cstruct, new_ClassField0(ctx, 0, data->method_size + data->formatter_size));
+				KNH_INITv(t->cstruct, new_ClassStruct0(ctx, 0, data->method_size + data->formatter_size));
 			}
 			else {
 				//DBG2_P("no method %s, %d,%d", data->name, data->method_size, data->formatter_size);
@@ -2447,7 +2424,7 @@ void knh_loadFieldNameData(Ctx *ctx, knh_FieldNameData_t *data)
 		knh_String_t *name = T__(data->name);
 #if defined(KNH_DBGMODE2)
 		knh_fieldn_t fn = (knh_fieldn_t)knh_DictIdx_add__fast(ctx, DP(ctx->sys)->FieldNameDictIdx, name);
-			//DEBUG("'%s' %d %d", __tochar(name), fn, fn2);
+			//DEBUG("'%s' %d %d", knh_String_tochar(name), fn, fn2);
 		KNH_ASSERT(fn == data->fn);
 #else
 		knh_DictIdx_add__fast(ctx, DP(ctx->sys)->FieldNameDictIdx, name);
@@ -2502,7 +2479,7 @@ static
 void knh_loadMethodData(Ctx *ctx, knh_MethodData_t *data, knh_MethodField_t **pools)
 {
 	while(data->func != NULL) {
-		DBG2_ASSERT_cid(data->cid);
+		KNH_ASSERT_cid(data->cid);
 		knh_Method_t *mtd = new_Method(ctx, data->flag, data->cid, data->mn, data->func);
 		DP(mtd)->code  = data->optfunc;
 		DP(mtd)->delta = data->delta;
@@ -2514,7 +2491,7 @@ void knh_loadMethodData(Ctx *ctx, knh_MethodData_t *data, knh_MethodField_t **po
 			DP(mtd)->flag = DP(mtd)->flag | FLAG_Method_Static;
 		}
 		{
-			knh_ClassField_t *cs = ClassTable(data->cid).cstruct;
+			knh_ClassStruct_t *cs = ClassTable(data->cid).cstruct;
 			knh_Array_add(ctx, cs->methods, UP(mtd));
 		}
 		data++;
