@@ -28,7 +28,7 @@
 /* ************************************************************************ */
 
 #include"commons.h"
-#if !defined(KONOHA_ON_LKM) && !defined(KNH_USING_BTRON)
+#if !defined(KONOHA_ON_LKM) && !defined(KNH_USING_BTRON) && !defined(KNH_USING_WINDOWS)
 #include <dirent.h>
 #endif
 
@@ -41,36 +41,76 @@ extern "C" {
 #ifdef KNH_CC_METHODAPI
 
 /* ======================================================================== */
-/* [method] */
+/* [System] */
 
 /* ------------------------------------------------------------------------ */
-/* @method[STATIC] InputStream! System.getIn() */
+//## @Static method InputStream! System.getIn();
 
-static METHOD knh__System_getIn(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD System_getIn(Ctx *ctx, knh_sfp_t *sfp)
 {
 	KNH_RETURN(ctx, sfp, DP(ctx->sys)->in);
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method[STATIC] OutputStream! System.getOut() */
+//## @Static method OutputStream! System.getOut();
 
-static METHOD knh__System_getOut(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD System_getOut(Ctx *ctx, knh_sfp_t *sfp)
 {
 	KNH_RETURN(ctx, sfp, DP(ctx->sys)->out);
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method[STATIC] OutputStream! System.getErr() */
+//## @Static method OutputStream! System.getErr();
 
-static METHOD knh__System_getErr(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD System_getErr(Ctx *ctx, knh_sfp_t *sfp)
 {
 	KNH_RETURN(ctx, sfp, DP(ctx->sys)->err);
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method[STATIC] Boolean! System.hasLib(String! lib, String func) */
+//## @Static method Any? System.getProperty(String! key);
 
-METHOD knh__System_hasLib(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD System_getProperty(Ctx *ctx, knh_sfp_t *sfp)
+{
+	KNH_RETURN(ctx, sfp,
+		knh_System_getProperty(ctx,(knh_System_t*)sfp[0].o, knh_String_tobytes(sfp[1].s)));
+}
+
+/* ------------------------------------------------------------------------ */
+//## @Static method void System.setProperty(String! key, Any? value);
+
+static METHOD System_setProperty(Ctx *ctx, knh_sfp_t *sfp)
+{
+	knh_System_setProperty(ctx, (knh_System_t*)sfp[0].o, sfp[1].s, sfp[2].o);
+	KNH_RETURN_void(ctx, sfp);
+}
+
+/* ------------------------------------------------------------------------ */
+//## @Static method void System.gc();
+
+static METHOD System_gc(Ctx *ctx, knh_sfp_t *sfp)
+{
+	knh_System_gc(ctx);
+}
+
+/* ------------------------------------------------------------------------ */
+//## @static method Int Script.system(String cmd);
+//## @static method Int Script.sudo(String cmd);
+
+static METHOD Script_system(Ctx *ctx, knh_sfp_t *sfp)
+{
+    KNH_SECURE(ctx,sfp);
+    int ret = system(String_to(char*, sfp[1]));
+    if(ret  == -1) {
+        KNH_PERRNO(ctx, NULL, "OS!!", "system", knh_Context_isStrict(ctx));
+    }
+    KNH_RETURN_Int(ctx, sfp,ret);
+}
+
+/* ------------------------------------------------------------------------ */
+//## @Static method Boolean! System.hasLib(String! lib, String? func);
+
+static METHOD System_hasLib(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
 	knh_bytes_t libname = knh_String_tobytes(sfp[1].s);
@@ -96,21 +136,11 @@ METHOD knh__System_hasLib(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method[STATIC] void System.setRandomSeed(Int seed) */
+//## @Static @Audit method void System.exit(Int status);
 
-METHOD knh__System_setRandomSeed(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD System_exit(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_uint_t seed = IS_NULL(sfp[1].o) ? 0 : p_uinteger(sfp[1]);
-	knh_srand(seed);
-	KNH_RETURN_void(ctx, sfp);
-}
-
-/* ------------------------------------------------------------------------ */
-/* @method[STATIC] void System.exit(Int status) @Tricky */
-
-static METHOD knh__System_exit(Ctx *ctx, knh_sfp_t *sfp)
-{
-#ifdef KNH_USING_STDC
+#if defined(KNH_USING_STDC)
 	int status = IS_NULL(sfp[1].o) ? 0 : p_int(sfp[1]);
 	KNH_SECURE(ctx, sfp);
 	KNH_NOTICE(ctx, "exiting by a user");
@@ -120,18 +150,19 @@ static METHOD knh__System_exit(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method String[] System.listDir(String dirname) */
-METHOD knh__System_listDir(Ctx *ctx, knh_sfp_t *sfp)
+//## method String[] System.listDir(String? path);
+
+static METHOD System_listDir(Ctx *ctx, knh_sfp_t *sfp)
 {
-#if defined(KONOHA_USING_POSIX)
-	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf)
-	Array *a = new_Array(ctx, CLASS_String, 0);
+#if defined(KNH_USING_POSIX)
+	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
+	knh_Array_t *a = new_Array(ctx, CLASS_String, 0);
 	knh_bytes_t t = (IS_NULL(sfp[1].s)) ? STEXT(".") : knh_String_tobytes(sfp[1].s);
 	knh_cwb_write(ctx, cwb, t);
 	knh_cwb_ospath(ctx, cwb);
-	Array *a = new_Array(ctx, CLASS_String, 0);
 	KNH_SETv(ctx, sfp[2].o, a);
 	DIR *dirptr;
+	char *dirname = knh_cwb_tochar(ctx, cwb);
 	if ((dirptr = opendir(dirname)) == NULL) {
 		KNH_PERRNO(ctx, cwb, "OS!!", "opendir", knh_Context_isStrict(ctx));
 	} else {
@@ -149,9 +180,9 @@ METHOD knh__System_listDir(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method Boolean! System.hasDir(String! path) */
+//## method Boolean! System.hasDir(String! path);
 
-static METHOD knh__System_hasDir(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD System_hasDir(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_cwb_t cwbbuf, *cwb = knh_cwb_openinit(ctx, &cwbbuf, knh_String_tobytes(sfp[1].s));
 	knh_cwb_ospath(ctx, cwb);
@@ -161,9 +192,9 @@ static METHOD knh__System_hasDir(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method Boolean! System.hasFile(String! path) */
+//## method Boolean! System.hasFile(String! path);
 
-static METHOD knh__System_hasFile(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD System_hasFile(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_cwb_t cwbbuf, *cwb = knh_cwb_openinit(ctx, &cwbbuf, knh_String_tobytes(sfp[1].s));
 	knh_cwb_ospath(ctx, cwb);
@@ -173,9 +204,9 @@ static METHOD knh__System_hasFile(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method Boolean! System.mkdir(String! filename) @Tricky */
+//## @Audit method Boolean! System.mkdir(String! path);
 
-static METHOD knh__System_mkdir(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD System_mkdir(Ctx *ctx, knh_sfp_t *sfp)
 {
 	KNH_SECURE(ctx, sfp);
 	KNH_RETURN_Boolean(ctx, sfp,
@@ -184,9 +215,9 @@ static METHOD knh__System_mkdir(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method Boolean! System.unlink(String! filename) @Tricky */
+//## @Audit method Boolean! System.unlink(String! path);
 
-static METHOD knh__System_unlink(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD System_unlink(Ctx *ctx, knh_sfp_t *sfp)
 {
 	KNH_SECURE(ctx, sfp);
 	KNH_RETURN_Boolean(ctx, sfp,
@@ -195,13 +226,14 @@ static METHOD knh__System_unlink(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method Boolean! System.rename(String oldname, String newname) @Tricky */
+//## @Audit method Boolean! System.rename(String path, String newpath);
 
-static METHOD knh__System_rename(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD System_rename(Ctx *ctx, knh_sfp_t *sfp)
 {
 	KNH_SECURE(ctx, sfp);
 	KNH_RETURN_Boolean(ctx, sfp,
-		knh_rename(ctx, knh_String_tobytes(sfp[1].s), knh_String_tobytes(sfp[2].s), knh_Context_isStrict(ctx))
+		knh_rename(ctx, knh_String_tobytes(sfp[1].s),
+			knh_String_tobytes(sfp[2].s), knh_Context_isStrict(ctx))
 	);
 }
 
@@ -209,41 +241,50 @@ static METHOD knh__System_rename(Ctx *ctx, knh_sfp_t *sfp)
 /* [Context] */
 
 /* ------------------------------------------------------------------------ */
-/* @method Any Context.getProperty(String! key) */
+//## @Static method Any Context.getProperty(String! key);
 
-Object* knh_Context_getProperty(Ctx *ctx, Context *b, knh_bytes_t key)
+static METHOD Context_getProperty(Ctx *ctx, knh_sfp_t *sfp)
 {
-	Object *v = knh_DictMap_get__b(ctx,  b->props, key);
-	if(IS_NULL(v)) {
-		return knh_System_getProperty(ctx, ctx->sys, key);
-	}
-	return v;
+	KNH_RETURN(ctx, sfp,
+		knh_Context_getProperty(ctx, (knh_Context_t*)sfp[0].o,
+				knh_String_tobytes(sfp[1].s)));
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void Context.setProperty(String! key, Any value) */
+//## @Static method void Context.setProperty(String! key, Any? value);
 
-void knh_Context_setProperty(Ctx *ctx, Context *b, String *key, Any *value)
+static METHOD Context_setProperty(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_DictMap_set(ctx, b->props, key, value);
+	knh_Context_setProperty(ctx, (knh_Context_t*)sfp[0].o, sfp[1].s, sfp[2].o);
+	KNH_RETURN_void(ctx, sfp);
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method String[]! Context.listProperties(String key) */
+//## @Static method void Context.setEncoding(String? enc);
 
-static METHOD knh__Context_listProperties(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD Context_setEncoding(Ctx *ctx, knh_sfp_t *sfp)
 {
-	DictSet *ds = new_DictSet(ctx, 64);
+	knh_Context_setEncoding(ctx, (knh_Context_t*)sfp[0].o, sfp[1].s);
+	KNH_RETURN_void(ctx, sfp);
+}
+
+/* ------------------------------------------------------------------------ */
+//## @Hidden method String[] Context.listProperties(String key);
+
+static METHOD Context_listProperties(Ctx *ctx, knh_sfp_t *sfp)
+{
+	knh_DictSet_t *ds = new_DictSet(ctx, 64);
 	KNH_MOV(ctx, sfp[0].o, ds); // TO RCGC
 	if(IS_NULL(sfp[1].s)) {
 		KNH_MOV(ctx, sfp[1].o, TS_EMPTY);
 	}
-	DictMap *map = DP(ctx->sys)->props;
+	knh_bytes_t prefix = knh_String_tobytes(sfp[1].s);
+	knh_DictMap_t *map = DP(ctx->sys)->props;
 	size_t i;
 	for(i = 0; i < map->size; i++) {
 		if(IS_NOTNULL(knh_DictMap_valueAt(map, i))) {
-			String *key = knh_DictMap_keyAt(map, i);
-			if(knh_String_startsWith(key, knh_String_tobytes(sfp[1].s))) {
+			knh_String_t *key = knh_DictMap_keyAt(map, i);
+			if(knh_bytes_matchWildCard(knh_String_tobytes(key), prefix)) {
 				knh_DictSet_add(ctx, ds, key);
 			}
 		}
@@ -251,61 +292,57 @@ static METHOD knh__Context_listProperties(Ctx *ctx, knh_sfp_t *sfp)
 	map = ctx->props;
 	for(i = 0; i < map->size; i++) {
 		if(IS_NOTNULL(knh_DictMap_valueAt(map, i))) {
-			String *key = knh_DictMap_keyAt(map, i);
-			if(knh_String_startsWith(key, knh_String_tobytes(sfp[1].s))) {
+			knh_String_t *key = knh_DictMap_keyAt(map, i);
+			if(knh_bytes_matchWildCard(knh_String_tobytes(key), prefix)) {
 				knh_DictSet_add(ctx, ds, key);
 			}
 		}
 	}
-	Array *a = new_Array(ctx, CLASS_String, ds->size);
+	knh_Array_t *a = new_Array(ctx, CLASS_String, ds->size);
 	for(i = 0; i < ds->size; i++) {
 		knh_Array_add(ctx, a, UP(knh_DictSet_keyAt(ds, i)));
 	}
 	KNH_RETURN(ctx, sfp, a);
 }
 
-
 /* ------------------------------------------------------------------------ */
-/* @method void Context.setIn(InputStream in) */
+//## @Static method void Context.setIn(InputStream? in);
 
-static METHOD knh__Context_setIn(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD Context_setIn(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_ASSERT(ctx == (Ctx*)sfp[0].o);
 	if(IS_NULL(sfp[1].o)) {
-		KNH_SETv(ctx, ((Context*)ctx)->in, knh_getClassDefaultValue(ctx, CLASS_InputStream));
+		KNH_SETv(ctx, ((knh_Context_t*)ctx)->in, DP(ctx->sys)->in);
 	}
 	else {
-		KNH_SETv(ctx, ((Context*)ctx)->in, sfp[1].o);
+		KNH_SETv(ctx, ((knh_Context_t*)ctx)->in, sfp[1].o);
 	}
 	KNH_RETURN_void(ctx, sfp);
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void Context.setOut(OutputStream in) */
+//## @Static method void Context.setOut(OutputStream? out);
 
-static METHOD knh__Context_setOut(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD Context_setOut(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_ASSERT(ctx == (Ctx*)sfp[0].o);
 	if(IS_NULL(sfp[1].o)) {
-		KNH_SETv(ctx, ((Context*)ctx)->out, knh_getClassDefaultValue(ctx, CLASS_OutputStream));
+		KNH_SETv(ctx, ((knh_Context_t*)ctx)->out, DP(ctx->sys)->out);
 	}
 	else {
-		KNH_SETv(ctx, ((Context*)ctx)->out, sfp[1].o);
+		KNH_SETv(ctx, ((knh_Context_t*)ctx)->out, sfp[1].o);
 	}
 	KNH_RETURN_void(ctx, sfp);
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void Context.setErr(OutputStream out) */
+//## @Static method void Context.setErr(OutputStream? out);
 
-static METHOD knh__Context_setErr(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD Context_setErr(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_ASSERT(ctx == (Ctx*)sfp[0].o);
 	if(IS_NULL(sfp[1].o)) {
-		KNH_SETv(ctx, ((Context*)ctx)->err, knh_getClassDefaultValue(ctx, CLASS_OutputStream));
+		KNH_SETv(ctx, ((knh_Context_t*)ctx)->err, DP(ctx->sys)->err);
 	}
 	else {
-		KNH_SETv(ctx, ((Context*)ctx)->err, sfp[1].o);
+		KNH_SETv(ctx, ((knh_Context_t*)ctx)->err, sfp[1].o);
 	}
 	KNH_RETURN_void(ctx, sfp);
 }
@@ -313,10 +350,9 @@ static METHOD knh__Context_setErr(Ctx *ctx, knh_sfp_t *sfp)
 /* ======================================================================== */
 /* [movabletext] */
 
-/* @method void Context.%dump(OutputStream w, String m) */
+//## method void Context.%dump(OutputStream w, String m);
 
-static
-void knh_Context__dump(Ctx *ctx, Context *b, OutputStream *w, String *m)
+static void knh_Context__dump(Ctx *ctx, knh_Context_t *b, knh_OutputStream_t *w, knh_String_t *m)
 {
 //	knh_int_t i = 0;
 //	for(i = b->esp - b->stack; i >= 0; i--) {
@@ -325,161 +361,23 @@ void knh_Context__dump(Ctx *ctx, Context *b, OutputStream *w, String *m)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method Exception! Exception.new(String msg, Any bag) */
 
-static METHOD knh__Exception_new(Ctx *ctx, knh_sfp_t *sfp)
-{
-	Exception *o = sfp[0].e;
-	DP(o)->eid  = EXPT_Exception;
-	DP(o)->flag = ctx->share->ExptTable[EXPT_Exception].flag;
 
-	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
-	knh_write_char(ctx, cwb->w, "Exception!!");
-	if(IS_NOTNULL(sfp[1].o)) {
-		knh_write_char(ctx, cwb->w, ": ");
-		knh_write(ctx, cwb->w, knh_String_tobytes(sfp[1].s));
-	}
-	KNH_SETv(ctx, DP(o)->msg, new_String__cwb(ctx, cwb));
-	KNH_SETv(ctx, DP(o)->bag, sfp[2].o);
-	KNH_RETURN(ctx, sfp, o);
-}
 
 /* ------------------------------------------------------------------------ */
-/* @method Boolean! Exception.opIsa(String! m) */
+//## method Boolean! Exception.opIsa(String msg);
 
-static METHOD knh__Exception_opIsa(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD Exception_opIsa(Ctx *ctx, knh_sfp_t *sfp)
 {
 	KNH_RETURN_Boolean(ctx, sfp, knh_Exception_isa(ctx, sfp[0].e, sfp[1].s));
 }
 
-
 /* ------------------------------------------------------------------------ */
-/* @method void Exception.%s(OutputStream w, String m) */
+//## @Hidden @Const method void NameSpace.setConst(String! name, Any value);
 
-static
-void knh_Exception__s(Ctx *ctx, Exception *o, OutputStream *w, String *m)
+static METHOD NameSpace_setConst(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_ASSERT(IS_Exception(o));
-	knh_write_char(ctx, w, EXPTN(DP(o)->eid));
-	knh_write_char(ctx, w, "!!");
-}
-
-/* ------------------------------------------------------------------------ */
-/* @method void Exception.%k(OutputStream w, String m) */
-
-static
-void knh_Exception__k(Ctx *ctx, Exception *o, OutputStream *w, String *m)
-{
-	KNH_ASSERT(IS_Exception(o));
-	knh_putc(ctx, w, '\'');
-	knh_write(ctx, w, knh_String_tobytes(DP(o)->msg));
-	knh_putc(ctx, w, '\'');
-}
-
-/* ------------------------------------------------------------------------ */
-/* @method void Exception.%dump(OutputStream w, String m) */
-
-static
-void knh_Exception__dump(Ctx *ctx, Exception *o, OutputStream *w, String *m)
-{
-	DBG2_ASSERT(IS_Exception(o));
-	knh_write_EOL(ctx, w);
-	if(DP(o)->line != 0) {
-		knh_printf(ctx, w, "[%s:%d] ", DP(o)->file, DP(o)->line);
-	}
-	knh_write(ctx, w, knh_String_tobytes(DP(o)->msg));
-	knh_print(ctx, w, STEXT(" Exception"));
-	if(IS_bArray(DP(o)->traces)) {
-		size_t i, size = knh_Array_size(DP(o)->traces), c = 0;
-		knh_bytes_t prev = STEXT("?");
-		for(i = 0; i < size; i++) {
-			String *s = (String*)knh_Array_n(DP(o)->traces, i);
-			if(knh_String_startsWith(s, prev)) {
-				c++;
-				continue;
-			}
-			if(c > 0) {
-				knh_printf(ctx, w, "\n    ** called %d times recursively **", c);
-				c = 0;
-			}
-			knh_printf(ctx, w, "\n  at %s", knh_String_tochar(s));
-			prev = knh_String_tobytes(s);
-			prev = knh_bytes_first(prev, knh_bytes_rindex(prev, '('));
-		}
-	}
-	knh_write_EOL(ctx, w);
-}
-
-/* ------------------------------------------------------------------------ */
-
-#else /*KNH_CC_METHODAPI*/
-
-/* ------------------------------------------------------------------------ */
-/* @method void System.%k(OutputStream w, String m) */
-
-void knh_System__k(Ctx *ctx, System *o, OutputStream *w, String *m)
-{
-	knh_printf(ctx, w, "Konoha %s(%s) %s (#%d, %s %s)",
-		KONOHA_VERSION, KONOHA_XCODE, KONOHA_LICENSE,
-		((size_t)KONOHA_BUILDID), __DATE__, __TIME__);
-}
-
-/* ------------------------------------------------------------------------ */
-/* @method void System.%dump(OutputStream w, String m) */
-
-void knh_System__dump(Ctx *ctx, System *o, OutputStream *w, String *m)
-{
-	knh_System__k(ctx, o, w, m);
-	knh_write_EOL(ctx, w);
-	knh_printf(ctx, w, "[%s] on %s (%d, %s)\n", KONOHA_CC_VERSION, KONOHA_PLATFORM, (knh_intptr_t)(sizeof(void*) * 8), konoha_encoding());
-	knh_write_char(ctx, w, "Options:");
-
-#ifdef KNH_FASTMODE
-	knh_write_char(ctx, w, " fastmode");
-#endif
-
-#ifdef KNH_USING_ICONV
-	knh_write_char(ctx, w, " iconv");
-#endif
-#ifdef KNH_USING_INT32
-	knh_write_char(ctx, w, " int32");
-#endif
-#ifdef KNH_USING_RCGC
-	knh_write_char(ctx, w, " refc");
-#endif
-	knh_write_USING_SQLITE3(ctx, w);
-#ifdef KNH_USING_THREAD
-	knh_write_char(ctx, w, " thread");
-#endif
-	knh_write_USING_REGEX(ctx, w);
-	knh_printf(ctx, w, " used_memory:%d kb", (knh_intptr_t)(ctx->stat->usedMemorySize / 1024));
-	knh_write_EOL(ctx, w);
-}
-
-/* ------------------------------------------------------------------------ */
-/* @method void Context.setEncoding(String enc) */
-
-void knh_Context_setEncoding(Ctx *ctx, knh_Context_t *o, String *enc)
-{
-	if(IS_NULL(enc)) {
-		enc = KNH_ENC;
-	}
-	KNH_SETv(ctx, o->enc, enc);
-	if(knh_bytes_strcasecmp(knh_String_tobytes(enc), STEXT(KONOHA_ENCODING))==0) {
-		KNH_SETv(ctx, DP(o->in)->bconv, KNH_NULL);
-		KNH_SETv(ctx, DP(o->out)->bconv, KNH_NULL);
-		KNH_SETv(ctx, DP(o->err)->bconv, KNH_NULL);
-	}
-	else {
-		BytesConv *bin = new_BytesConv__in(ctx, knh_String_tochar(enc));
-		BytesConv *bout = new_BytesConv__out(ctx, knh_String_tochar(enc));
-		if(IS_NULL(bin) || IS_NULL(bout)) {
-			KNH_WARNING(ctx, "unsupported character encoding: %s", knh_String_tochar(enc));
-		}
-		KNH_SETv(ctx, DP(o->in)->bconv, bin);
-		KNH_SETv(ctx, DP(o->out)->bconv, bout);
-		KNH_SETv(ctx, DP(o->err)->bconv, bout);
-	}
+	knh_NameSpace_addConst(ctx, sfp[0].ns, sfp[1].s, sfp[2].o);
 }
 
 /* ------------------------------------------------------------------------ */
