@@ -28,7 +28,7 @@
 /* ************************************************************************ */
 
 #include"commons.h"
-#include"../../include/konoha/gen/konohac_klr_.h"
+#include"../../include/konoha/konoha_code_.h"
 
 size_t knh_code_opsize(int opcode);
 
@@ -39,29 +39,6 @@ extern "C" {
 #endif
 
 /* ======================================================================== */
-/* [constructors] */
-
-///* ------------------------------------------------------------------------ */
-///* @ method This! Closure.new(Any! base, Method! mtd) */
-//
-//METHOD knh__Closure_new(Ctx *ctx, knh_sfp_t *sfp)
-//{
-//	KNH_INITv((sfp[0].cc)->base, sfp[1].o);
-//	KNH_INITv((sfp[0].cc)->mtd, sfp[2].mtd);
-//	(sfp[0].cc)->envsfp = NULL;
-//	KNH_RETURN(ctx, sfp, sfp[0].o);
-//}
-
-/* ------------------------------------------------------------------------ */
-/* @method Any0 Closure.invoke(Any1 x, Any2 y, Any3 z) @VarArgs */
-
-METHOD knh__Closure_invoke(Ctx *ctx, knh_sfp_t *sfp)
-{
-	//DBG2_ASSERT(IS_bClosure(sfp[0].cc));
-	KNH_INVOKE(ctx, sfp);
-}
-
-/* ------------------------------------------------------------------------ */
 
 int
 knh_stack_vpush(Ctx *ctx, knh_sfp_t *sfp, const char *fmt, va_list args)
@@ -137,7 +114,7 @@ KNHAPI(void) knh_putsfp(Ctx *ctx, knh_sfp_t *lsfp, int n, Object *obj)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_Closure_invokesfp(Ctx *ctx, Closure *cc, knh_sfp_t *lsfp, int argc)
+KNHAPI(void) knh_Closure_invokesfp(Ctx *ctx, knh_Closure_t *cc, knh_sfp_t *lsfp, int argc)
 {
 	KNH_MOV(ctx, lsfp[1].o, (cc)->base);
 	KNH_SCALL(ctx, lsfp, 0, (cc)->mtd, argc);
@@ -146,15 +123,10 @@ KNHAPI(void) knh_Closure_invokesfp(Ctx *ctx, Closure *cc, knh_sfp_t *lsfp, int a
 
 /* ------------------------------------------------------------------------ */
 
-//#define _knh_Closure_binvokef(ctx, c, fmt, ...)  ((knh_Closure_invokef(ctx, c, fmt, ## __VA_ARGS__))->bvalue)
-//#define _knh_Closure_iinvokef(ctx, c, fmt, ...)  ((knh_Closure_invokef(ctx, c, fmt, ## __VA_ARGS__))->ivalue)
-//#define _knh_Closure_finvokef(ctx, c, fmt, ...)  ((knh_Closure_invokef(ctx, c, fmt, ## __VA_ARGS__))->fvalue)
-//#define _knh_Closure_sinvokef(ctx, c, fmt, ...)  knh_String_tochar((knh_Closure_invokef(ctx, c, fmt, ## __VA_ARGS__))->s)
-
 // knh_Closure_finvoke(ctx, c, "oo", sfp[1].o, sfp[2].o)
 // knh_Closure_finvoke(ctx, c, "ii", 1, 2)
 
-KNHAPI(knh_sfp_t*) knh_Closure_invokef(Ctx *ctx, Closure *c, const char *fmt, ...)
+KNHAPI(knh_sfp_t*) knh_Closure_invokef(Ctx *ctx, knh_Closure_t *c, const char *fmt, ...)
 {
 	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
 	int n;
@@ -168,7 +140,7 @@ KNHAPI(knh_sfp_t*) knh_Closure_invokef(Ctx *ctx, Closure *c, const char *fmt, ..
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_Closure_invoke(Ctx *ctx, Closure *c, const char *fmt, ...)
+KNHAPI(void) knh_Closure_invoke(Ctx *ctx, knh_Closure_t *c, const char *fmt, ...)
 {
 	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
 	int n;
@@ -190,39 +162,53 @@ METHOD knh_fmethod_closureDEFAULT(Ctx *ctx, knh_sfp_t *sfp)
 	if(rtype == TYPE_void) {
 		KNH_RETURN_void(ctx, sfp);
 	}
-	KNH_RETURN(ctx, sfp, knh_getDefaultValue(ctx, rtype));
+	KNH_RETURN(ctx, sfp, KNH_DEF(ctx, CLASS_type(rtype)));
 }
 
 /* ------------------------------------------------------------------------ */
 
-static
-Closure* new_ClosureDEFAULT(Ctx *ctx, knh_type_t rtype, knh_class_t cid)
+knh_Object_t *knh_fdefault__NEWCLOSURE(Ctx *ctx, knh_class_t cid)
 {
-	Closure *cc = (Closure*)new_Object_init(ctx, FLAG_Closure, cid, 0);
-	Method *mtd = new_Method(ctx, 0, cid, METHODN_lambda, knh_fmethod_closureDEFAULT);
+	knh_ClassTable_t *t = pClassTable(cid);
+	knh_Closure_t *cc = (knh_Closure_t*)new_Object_init(ctx, FLAG_Closure, cid, 0);
+	knh_Method_t *mtd = new_Method(ctx, 0, cid, METHODN_LAMBDA, knh_fmethod_closureDEFAULT);
 	KNH_INITv((cc)->mtd, mtd);
-	KNH_SETv(ctx, DP(mtd)->mf, knh_findMethodField0(ctx, rtype));
+	KNH_SETv(ctx, DP(mtd)->mf, knh_findMethodField0(ctx, t->r0));
 	knh_Method_setVarArgs(mtd, 1);
 	KNH_INITv((cc)->base, KNH_NULL);
 	(cc)->envsfp = NULL;
-	return cc;
+
+	DBG2_P("create new default value of %s", CLASSN(cid));
+	KNH_ASSERT(t->cspec == NULL);
+	KNH_INITv(t->cspec, cc);
+	t->fdefault = knh_fdefault__CONST;
+	return t->cspec;
 }
 
 /* ------------------------------------------------------------------------ */
 
 knh_class_t
-knh_addClosureClass(Ctx *ctx, knh_class_t cid, String *name, knh_type_t r0, knh_type_t p1, knh_type_t p2, knh_type_t p3)
+knh_addClosureClass(Ctx *ctx, knh_class_t cid, knh_String_t *name, knh_type_t r0, knh_type_t p1, knh_type_t p2, knh_type_t p3)
 {
+	knh_ClassTable_t *t = NULL;
+	knh_flag_t mask = 0;
 	if(cid == CLASS_newid) {
 		cid = knh_ClassTable_newId(ctx);
 	} else {
-		//KNH_ASSERT(cid + 1 == ctx->share->ClassTableSize);
 		((knh_SharedData_t*)ctx->share)->ClassTableSize = cid;
 	}
-	knh_ClassTable_t *t = pClassTable(cid);
-	KNH_ASSERT(ClassTable(cid).class_cid == NULL);
+	/* knh_ClassTable_t */ t = pClassTable(cid);
+	KNH_ASSERT(ClassTable(cid).sname == NULL);
+	if(knh_class_isTypeVariable(CLASS_type(r0)) ||
+			knh_class_isTypeVariable(CLASS_type(p1)) ||
+			knh_class_isTypeVariable(CLASS_type(p2)) ||
+			knh_class_isTypeVariable(CLASS_type(p3))) {
+		mask = FLAG_Class_TypeVariable;
+		DBG2_P("TypeVarable: %s", knh_String_tochar(name));
+	}
 
-	t->cflag  = ClassTable(CLASS_Closure).cflag;
+	knh_setClassName(ctx, cid, name);
+	t->cflag  = ClassTable(CLASS_Closure).cflag | mask;
 	t->oflag  = ClassTable(CLASS_Closure).oflag;
 	t->sid    = ClassTable(CLASS_Closure).sid;
 
@@ -233,50 +219,61 @@ knh_addClosureClass(Ctx *ctx, knh_class_t cid, String *name, knh_type_t r0, knh_
 	t->size = ClassTable(CLASS_Closure).size;
 	t->bsize  = ClassTable(CLASS_Closure).bsize;
 
-	knh_setClassName(ctx, cid, name);
 	KNH_INITv(t->cstruct, ClassTable(CLASS_Closure).cstruct);
 	KNH_INITv(t->cmap, ClassTable(CLASS_Closure).cmap);
 	t->r0 = r0;
 	t->p1 = p1;
 	t->p2 = p2;
 	t->p3 = p3;
-	knh_setClassDefaultValue(ctx, cid, UP(new_ClosureDEFAULT(ctx, r0, cid)), NULL);
+	StructTable(CLASS_Closure).fnewClass(ctx, cid);
 	return cid;
 }
 
+/* ------------------------------------------------------------------------ */
+
+void knh_Closure_newClass(Ctx *ctx, knh_class_t cid)
+{
+	knh_setClassDefaultValue(ctx, cid, NULL, knh_fdefault__NEWCLOSURE);
+}
+
+
 /* ------------------------------------------------------------------------ */
 
 knh_class_t knh_class_Closure(Ctx *ctx, knh_type_t r0, knh_type_t p1, knh_type_t p2, knh_type_t p3)
 {
-	char buf[CLASSNAME_BUFSIZ*2];
-	char tb0[CLASSNAME_BUFSIZ], tb1[CLASSNAME_BUFSIZ];
-	knh_format_type(ctx, tb0, sizeof(tb0), r0);
-	knh_format_type(ctx, tb1, sizeof(tb1), p1);
+	knh_class_t cid = CLASS_unknown;
+	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
+	if(r0 == CLASS_unknown || p1 == CLASS_unknown || p2 == CLASS_unknown|| p3 == CLASS_unknown) {
+		return cid;
+	}
+	knh_write_ltype(ctx, cwb->w, r0);
+	knh_putc(ctx, cwb->w, '(');
+	knh_write_ltype(ctx, cwb->w, p1);
 	if(p2 == TYPE_void) {
-		knh_snprintf(buf, sizeof(buf), "%s(%s)", tb0, tb1);
+		goto L_CLOSE;
 	}
-	else if(p3 == TYPE_void) {
-		char tb2[CLASSNAME_BUFSIZ];
-		knh_format_type(ctx, tb2, sizeof(tb2), p2);
-		knh_snprintf(buf, sizeof(buf), "%s(%s,%s)", tb0, tb1, tb2);
+	knh_putc(ctx, cwb->w, ',');
+	knh_write_ltype(ctx, cwb->w, p2);
+	if(p3 == TYPE_void) {
+		goto L_CLOSE;
 	}
-	else {
-		char tb2[CLASSNAME_BUFSIZ], tb3[CLASSNAME_BUFSIZ];
-		knh_format_type(ctx, tb2, sizeof(tb2), p2);
-		knh_format_type(ctx, tb3, sizeof(tb3), p3);
-		knh_snprintf(buf, sizeof(buf), "%s(%s,%s,%s)", tb0, tb1, tb2, tb3);
-	}
-	knh_class_t cid = knh_getcid(ctx, B(buf));
+	knh_putc(ctx, cwb->w, ',');
+	knh_write_ltype(ctx, cwb->w, p3);
+	L_CLOSE:;
+	knh_putc(ctx, cwb->w, ')');
+
+	/* knh_class_t*/ cid = knh_getcid(ctx, knh_cwb_tobytes(cwb));
 	if(cid == CLASS_unknown) {
-		DBG2_P("*** Generating %s ***", buf);
-		cid = knh_addClosureClass(ctx, CLASS_newid, new_String(ctx, B(buf), NULL), r0, p1, p2, p3);
+		cid = knh_addClosureClass(ctx, CLASS_newid, knh_cwb_newString(ctx, cwb), r0, p1, p2, p3);
+	} else {
+		knh_cwb_close(cwb);
 	}
 	return cid;
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_class_t knh_class_MethodClosure(Ctx *ctx, knh_class_t cid, Method *mtd)
+knh_class_t knh_class_MethodClosure(Ctx *ctx, knh_class_t cid, knh_Method_t *mtd)
 {
 	if(knh_Method_psize(mtd) > 3) return CLASS_Closure;
 	knh_type_t r0 = knh_Method_rtype(ctx, mtd, cid);
@@ -303,28 +300,27 @@ void knh_sfp_copy(Ctx *ctx, knh_sfp_t *dst, knh_sfp_t *src, size_t size)
 	}
 }
 
-/* ------------------------------------------------------------------------ */
-
-Closure* new_Closure(Ctx *ctx, knh_sfp_t *sfp, Method *mtd)
-{
-	knh_class_t cid = knh_class_MethodClosure(ctx, knh_Object_cid(sfp[0].o), mtd);
-	Closure *cc = (Closure*)new_Object_init(ctx, FLAG_Closure, cid, 0);
-	KNH_INITv((cc)->mtd, mtd);
-	(cc)->self = cc;   // TO AVOID CYCLIC REFERENCE
-	(cc)->envsfp = sfp;
-	return cc;
-}
+///* ------------------------------------------------------------------------ */
+//
+//knh_Closure_t* new_Closure(Ctx *ctx, knh_sfp_t *sfp, knh_Method_t *mtd)
+//{
+//	knh_class_t cid = knh_class_MethodClosure(ctx, knh_Object_cid(sfp[0].o), mtd);
+//	knh_Closure_t *cc = (knh_Closure_t*)new_Object_init(ctx, FLAG_Closure, cid, 0);
+//	KNH_INITv((cc)->mtd, mtd);
+//	(cc)->self = cc;   // TO AVOID CYCLIC REFERENCE
+//	(cc)->envsfp = sfp;
+//	return cc;
+//}
 
 /* ------------------------------------------------------------------------ */
 
-void knh_Closure_storeEnv(Ctx *ctx, Closure *cc, knh_sfp_t *sfp)
+void knh_Closure_copyEnv(Ctx *ctx, knh_Closure_t *cc, knh_sfp_t *sfp)
 {
-	size_t stacksize = ctx->esp - (cc)->envsfp;
+	size_t stacksize = ctx->esp - sfp;
 	size_t* hstack = (size_t*)KNH_MALLOC(ctx, (sizeof(knh_sfp_t) * stacksize) + sizeof(size_t));
 	knh_sfp_t *envsfp = (knh_sfp_t*)(&hstack[1]);
-	KNH_ASSERT(cc->envsfp == sfp);
 	hstack[0] = stacksize;
-	knh_sfp_copy(ctx, (cc)->envsfp, envsfp, stacksize);
+	knh_sfp_copy(ctx, sfp, envsfp, stacksize);
 	(cc)->envsfp = envsfp;
 	knh_Closure_setStoredEnv(cc, 1);
 	DBG2_P("STORED %d", (int)stacksize);
@@ -334,7 +330,7 @@ void knh_Closure_storeEnv(Ctx *ctx, Closure *cc, knh_sfp_t *sfp)
 /* [Generator] */
 /* ------------------------------------------------------------------------ */
 
-knh_class_t knh_Method_gencid(Ctx *ctx, Method *mtd, knh_class_t cid)
+knh_class_t knh_Method_gencid(Ctx *ctx, knh_Method_t *mtd, knh_class_t cid)
 {
 	knh_type_t rtype = knh_pmztype_totype(ctx, knh_Method_rztype(mtd), cid);
 	cid = CLASS_type(rtype);
@@ -359,8 +355,8 @@ knh_code_t *knh_KLRCode_yeildingNext(knh_code_t *pc)
 static
 ITRNEXT knh_Generator_fnext(Ctx *ctx, knh_sfp_t *sfp, int n)
 {
-	Iterator *it = sfp[0].it;
-	Closure *cc = (Closure*)DP(it)->source;
+	knh_Iterator_t *it = sfp[0].it;
+	knh_Closure_t *cc = (knh_Closure_t*)DP(it)->source;
 	knh_code_t *pc = knh_KLRCode_yeildingNext(DP(it)->pc);
 	if(pc == NULL) {
 		KNH_ITREND(ctx, sfp, n);
@@ -382,14 +378,14 @@ ITRNEXT knh_Generator_fnext(Ctx *ctx, knh_sfp_t *sfp, int n)
 /* ------------------------------------------------------------------------ */
 
 static
-Iterator* new_Generator(Ctx *ctx, knh_sfp_t *sfp)
+knh_Iterator_t* new_Generator(Ctx *ctx, knh_sfp_t *sfp)
 {
-	Closure *cc = (Closure*)new_Object_init(ctx, FLAG_Closure, CLASS_Closure, 0);
+	knh_Closure_t *cc = (knh_Closure_t*)new_Object_init(ctx, FLAG_Closure, CLASS_Closure, 0);
 	KNH_INITv((cc)->mtd, sfp[-1].mtd);
 	KNH_INITv((cc)->base, sfp[0].o);
 	{
 		knh_class_t cid = knh_Method_gencid(ctx, sfp[-1].mtd, knh_Object_cid(sfp[0].o));
-		Iterator *it = new_Iterator(ctx, cid, UP(cc), knh_Generator_fnext);
+		knh_Iterator_t *it = new_Iterator(ctx, cid, UP(cc), knh_Generator_fnext);
 		knh_code_t *pc = (sfp[-1].mtd)->pc_start;
 		KNH_ASSERT(((klr_setesp_t*)pc)->opcode == OPCODE_SETESP);
 		size_t stacksize = 1 + ((klr_setesp_t*)pc)->a1;
@@ -422,7 +418,7 @@ METHOD knh_fmethod_generator(Ctx *ctx, knh_sfp_t *sfp)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_Method_toGenerator(Method *mtd)
+void knh_Method_toGenerator(knh_Method_t *mtd)
 {
 	mtd->fcall_1 = knh_fmethod_generator;
 }
