@@ -35,12 +35,16 @@
 extern "C" {
 #endif
 
+#ifndef KONOHA_CONTEXT_BUFSIZ
+#define KONOHA_CONTEXT_BUFSIZ  4096
+#endif
+
 /* ======================================================================== */
 /* [Context] */
 
-Object* knh_getProperty(Ctx *ctx, knh_bytes_t key)
+Object* knh_Context_getProperty(Ctx *ctx, knh_Context_t *b, knh_bytes_t key)
 {
-	Object *v = knh_DictMap_get__b(ctx,  (ctx)->props, key);
+	Object *v = knh_DictMap_get__b(ctx,  b->props, key);
 	if(IS_NULL(v)) {
 		v = knh_System_getProperty(ctx, ctx->sys, key);
 	}
@@ -55,7 +59,6 @@ knh_type_t knh_getPropertyType(Ctx *ctx, knh_bytes_t key)
 	Object *v = knh_DictMap_get__b(ctx,  ctx->props, key);
 	if(IS_NULL(v)) {
 		v = knh_System_getProperty(ctx, ctx->sys, key);
-		KNH_SETv(ctx, ctx->esp[0].o, v);
 	}
 	if(IS_NOTNULL(v)) {
 		type = NNTYPE_cid(knh_Object_cid(v));
@@ -65,16 +68,17 @@ knh_type_t knh_getPropertyType(Ctx *ctx, knh_bytes_t key)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_setProperty(Ctx *ctx, knh_String_t *key, Any *value)
+void knh_Context_setProperty(Ctx *ctx, knh_Context_t *b, knh_String_t *key, Any *value)
 {
-	knh_type_t type = knh_getPropertyType(ctx, __tobytes(key));
+	knh_type_t type = knh_getPropertyType(ctx, knh_String_tobytes(key));
 	if(type == TYPE_Any || knh_Object_cid(value) == CLASS_type(type)) {
-		knh_DictMap_set(ctx, ctx->props, key, value);
+		knh_DictMap_set(ctx, b->props, key, value);
 	}
 	else {
 		knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
 		knh_printf(ctx, cwb->w, "PropertyType!!: $%s must be %T", knh_String_text(ctx, key), type);
-		knh_stack_throw(ctx, ctx->esp, knh_cwb_newException(ctx, cwb), NULL, 0);
+		knh_String_t *s = knh_cwb_newString(ctx, cwb);
+		KNH_THROW(ctx, s);
 	}
 }
 
@@ -86,16 +90,16 @@ void knh_Context_setEncoding(Ctx *ctx, knh_Context_t *o, knh_String_t *enc)
 		enc = KNH_ENC;
 	}
 	KNH_SETv(ctx, o->enc, enc);
-	if(knh_bytes_strcasecmp(__tobytes(enc), STEXT(KONOHA_ENCODING))==0) {
+	if(knh_bytes_strcasecmp(knh_String_tobytes(enc), STEXT(KONOHA_ENCODING))==0) {
 		KNH_SETv(ctx, DP(o->in)->bconv, KNH_NULL);
 		KNH_SETv(ctx, DP(o->out)->bconv, KNH_NULL);
 		KNH_SETv(ctx, DP(o->err)->bconv, KNH_NULL);
 	}
 	else {
-		knh_BytesConv_t *bin = new_BytesConv__in(ctx, __tochar(enc));
-		knh_BytesConv_t *bout = new_BytesConv__out(ctx, __tochar(enc));
+		knh_BytesConv_t *bin = new_BytesConv__in(ctx, knh_String_tochar(enc));
+		knh_BytesConv_t *bout = new_BytesConv__out(ctx, knh_String_tochar(enc));
 		if(IS_NULL(bin) || IS_NULL(bout)) {
-			KNH_WARNING(ctx, "unsupported character encoding: %s", __tochar(enc));
+			KNH_WARNING(ctx, "unsupported character encoding: %s", knh_String_tochar(enc));
 		}
 		KNH_SETv(ctx, DP(o->in)->bconv, bin);
 		KNH_SETv(ctx, DP(o->out)->bconv, bout);
@@ -113,7 +117,9 @@ void knh_Context_clearstack(Ctx *ctx)
 	}
 }
 
-/* ------------------------------------------------------------------------ */
+/* ======================================================================== */
+/* [properties] */
+
 
 #ifdef __cplusplus
 }
