@@ -105,7 +105,7 @@ knh_index_t knh_DictIdx_add__fast(Ctx *ctx, knh_DictIdx_t *o, knh_String_t *term
 knh_index_t knh_DictIdx_add(Ctx *ctx, knh_DictIdx_t *o, knh_String_t *term)
 {
 	knh_index_t dictid =
-		(knh_index_t)knh_DictSet_get__b(o->termsDictSet, __tobytes(term));
+		(knh_index_t)knh_DictSet_get__b(o->termsDictSet, knh_String_tobytes(term));
 	if(dictid != 0) {
 		knh_DictSet_set(ctx, o->termsDictSet, term, dictid); /* TO AVOID GC HERE */
 		return dictid_totermid(dictid);
@@ -129,7 +129,7 @@ knh_String_t* knh_DictIdx_get__fast(knh_DictIdx_t *o, knh_intptr_t termid)
 {
 	size_t n = termid_toarrayn(termid);
 	if(unlikely(n >= knh_Array_size(o->terms))) {
-		DBG2_P("outof index %zd >= %zd", n, knh_Array_size(o->terms));
+		DBG2_P("outof index %ld >= %ld", n, knh_Array_size(o->terms));
 		n = 0;
 	}
 	return (knh_String_t*)knh_Array_n(o->terms, n);
