@@ -51,7 +51,6 @@ typedef struct {
 
 /* ------------------------------------------------------------------------ */
 
-static
 size_t knh_dict_capacity(knh_dict_t *a)
 {
 	if(a == NULL) {
@@ -101,7 +100,6 @@ knh_dict_t *knh_dict_malloc(Ctx *ctx, size_t capacity,
 	if(!(capacity > 0)) {
 		capacity = (KONOHA_SMALLPAGESIZE - sizeof(knh_hdict_t)) / sizeof(knh_dict_t);
 	}
-
 	h = (knh_hdict_t*)KNH_MALLOC(ctx, (capacity * sizeof(knh_dict_t)) + sizeof(knh_hdict_t));
 	h->capacity = capacity;
 	h->sorted = 0;
@@ -192,7 +190,7 @@ void knh_dict_sort(knh_dict_t *a, size_t size)
 	if(a != NULL) {
 		knh_hdict_t *h = ((knh_hdict_t*)a) - 1;
 		if(h->fdict_compar != NULL) {
-			knh_sort(a, size, sizeof(knh_dict_t), h->fdict_compar);
+			knh_qsort(a, size, sizeof(knh_dict_t), h->fdict_compar);
 			h->sorted = size;
 		}
 		else {
@@ -253,9 +251,10 @@ knh_index_t knh_dict_index_b(knh_dict_t *a, size_t size, knh_fbytescmp fcmp, knh
 	}
 }
 
+
 /* ------------------------------------------------------------------------ */
 
-void knh_DictMap_sort(DictMap *o)
+void knh_DictMap_sort(knh_DictMap_t *o)
 {
 	if(knh_dict_sorted(o->_list) != o->size) {
 		knh_dict_sort(o->_list, o->size);
@@ -264,7 +263,7 @@ void knh_DictMap_sort(DictMap *o)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictSet_sort(DictSet *o)
+void knh_DictSet_sort(knh_DictSet_t *o)
 {
 	if(knh_dict_sorted(o->_list) != o->size) {
 		knh_dict_sort(o->_list, o->size);
@@ -343,7 +342,7 @@ void knh_dict_tocase(knh_dict_t *a)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictMap_toIgnoreCase(DictMap *o)
+void knh_DictMap_toIgnoreCase(knh_DictMap_t *o)
 {
 	knh_dict_tocase(o->_list);
 	o->fcmp = knh_bytes_strcasecmp;
@@ -352,7 +351,7 @@ void knh_DictMap_toIgnoreCase(DictMap *o)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictSet_toIgnoreCase(DictSet *o)
+void knh_DictSet_toIgnoreCase(knh_DictSet_t *o)
 {
 	knh_dict_tocase(o->_list);
 	o->fcmp = knh_bytes_strcasecmp;
@@ -369,7 +368,7 @@ knh_dict_t *knh_dictmap_malloc(Ctx *ctx, size_t init)
 
 /* ------------------------------------------------------------------------ */
 
-DictMap* new_DictMap0(Ctx *ctx, knh_intptr_t capacity)
+knh_DictMap_t* new_DictMap0(Ctx *ctx, knh_intptr_t capacity)
 {
 	knh_DictMap_t *o = (knh_DictMap_t*)new_hObject(ctx, FLAG_DictMap, CLASS_DictMap, CLASS_DictMap);
 	o->_list = knh_dict_malloc(ctx, capacity, knh_dictmap_finit, knh_dictmap_ftraverse, knh_dict_fcmp);
@@ -380,9 +379,9 @@ DictMap* new_DictMap0(Ctx *ctx, knh_intptr_t capacity)
 
 /* ------------------------------------------------------------------------ */
 
-DictMap* new_DictMap(Ctx *ctx, knh_class_t p1, knh_intptr_t capacity)
+knh_DictMap_t* new_DictMap(Ctx *ctx, knh_class_t p1, knh_intptr_t capacity)
 {
-	knh_class_t cid = knh_class_Generics(ctx, CLASS_DictMap, p1, CLASS_unknown);
+	knh_class_t cid = knh_class_Generics(ctx, CLASS_DictMap, p1, CLASS_Tvoid);
 	knh_DictMap_t *o = (knh_DictMap_t*)new_hObject(ctx, FLAG_DictMap, CLASS_DictMap, cid);
 	o->_list = knh_dict_malloc(ctx, capacity, knh_dictmap_finit, knh_dictmap_ftraverse, knh_dict_fcmp);
 	o->size = 0;
@@ -393,14 +392,14 @@ DictMap* new_DictMap(Ctx *ctx, knh_class_t p1, knh_intptr_t capacity)
 /* ======================================================================== */
 /* [methods] */
 
-size_t knh_DictMap_size(DictMap *o)
+size_t knh_DictMap_size(knh_DictMap_t *o)
 {
 	return o->size;
 }
 
 /* ------------------------------------------------------------------------ */
 
-String* knh_DictMap_keyAt(DictMap *o, size_t n)
+knh_String_t* knh_DictMap_keyAt(knh_DictMap_t *o, size_t n)
 {
 	KNH_ASSERT(IS_bDictMap(o));
 	KNH_ASSERT(n < o->size);
@@ -409,7 +408,7 @@ String* knh_DictMap_keyAt(DictMap *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-Object* knh_DictMap_valueAt(DictMap *o, size_t n)
+Object* knh_DictMap_valueAt(knh_DictMap_t *o, size_t n)
 {
 	KNH_ASSERT(IS_bDictMap(o));
 	KNH_ASSERT(n < o->size);
@@ -418,7 +417,7 @@ Object* knh_DictMap_valueAt(DictMap *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictMap_removeAt(Ctx *ctx, DictMap *o, size_t n)
+void knh_DictMap_removeAt(Ctx *ctx, knh_DictMap_t *o, size_t n)
 {
 	KNH_ASSERT(IS_bDictMap(o));
 	KNH_ASSERT(n < o->size);
@@ -427,7 +426,7 @@ void knh_DictMap_removeAt(Ctx *ctx, DictMap *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-knh_index_t knh_DictMap_index__b(DictMap *o, knh_bytes_t key)
+knh_index_t knh_DictMap_index(knh_DictMap_t *o, knh_bytes_t key)
 {
 	KNH_ASSERT(IS_bDictMap(o));
 	return knh_dict_index_b(o->_list, o->size, o->fcmp, key);
@@ -435,7 +434,45 @@ knh_index_t knh_DictMap_index__b(DictMap *o, knh_bytes_t key)
 
 /* ------------------------------------------------------------------------ */
 
-String *new_String__DictMap(Ctx *ctx, DictMap *o, knh_bytes_t key)
+static
+knh_index_t knh_dict_first(knh_dict_t *a, knh_fbytescmp fcmp, knh_bytes_t key, size_t sp, size_t ep)
+{
+	L_TAIL:;
+	if(ep - sp < UNSORTED_MAXSIZ) {
+		size_t i;
+		for(i = sp; i < ep; i++) {
+			int res = fcmp(knh_String_tobytes(a[i].key), key);
+			if(res >= 0) return i;
+		}
+		return -1;
+	}
+	else {
+		size_t cp = KNH_MID(sp, ep);
+		int i = fcmp(key, knh_String_tobytes(a[cp].key));
+		if(i == 0) {
+			return cp;
+		}
+		else if(i < 0) {
+			ep = cp;
+		}else {
+			sp = cp + 1;
+		}
+		goto L_TAIL;
+	}
+}
+
+/* ------------------------------------------------------------------------ */
+
+knh_index_t knh_DictMap_firstIndex(knh_DictMap_t *o, knh_bytes_t key)
+{
+	KNH_ASSERT(IS_bDictMap(o));
+	knh_DictMap_sort(o);
+	return knh_dict_first(o->_list, o->fcmp, key, 0, o->size);
+}
+
+/* ------------------------------------------------------------------------ */
+
+knh_String_t *new_String__DictMap(Ctx *ctx, knh_DictMap_t *o, knh_bytes_t key)
 {
 	KNH_ASSERT(IS_bDictMap(o));
 	knh_index_t loc = knh_dict_index_b(o->_list, o->size, o->fcmp, key);
@@ -449,7 +486,7 @@ String *new_String__DictMap(Ctx *ctx, DictMap *o, knh_bytes_t key)
 
 /* ------------------------------------------------------------------------ */
 
-Object *knh_DictMap_get__b(Ctx *ctx, DictMap *o, knh_bytes_t key)
+Object *knh_DictMap_get__b(Ctx *ctx, knh_DictMap_t *o, knh_bytes_t key)
 {
 	KNH_ASSERT(IS_bDictMap(o));
 	knh_intptr_t loc = knh_dict_index_b(o->_list, o->size, o->fcmp, key);
@@ -458,7 +495,7 @@ Object *knh_DictMap_get__b(Ctx *ctx, DictMap *o, knh_bytes_t key)
 
 /* ------------------------------------------------------------------------ */
 
-Object *knh_DictMap_get(Ctx *ctx, DictMap *o, String *key)
+Object *knh_DictMap_get(Ctx *ctx, knh_DictMap_t *o, knh_String_t *key)
 {
 	KNH_ASSERT(IS_bDictMap(o));
 	knh_intptr_t loc = knh_dict_index_b(o->_list, o->size, o->fcmp, knh_String_tobytes(key));
@@ -467,7 +504,7 @@ Object *knh_DictMap_get(Ctx *ctx, DictMap *o, String *key)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictMap_append(Ctx *ctx, DictMap *o, String *key, Any *value)
+void knh_DictMap_append(Ctx *ctx, knh_DictMap_t *o, knh_String_t *key, Any *value)
 {
 	KNH_ASSERT(IS_bDictMap(o));
 	if(o->_list == NULL) {
@@ -487,7 +524,7 @@ void knh_DictMap_append(Ctx *ctx, DictMap *o, String *key, Any *value)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictMap_set(Ctx *ctx, DictMap *o, String *key, Any *v)
+void knh_DictMap_set(Ctx *ctx, knh_DictMap_t *o, knh_String_t *key, Any *v)
 {
 	knh_Object_t *value = v;
 	int loc = knh_dict_index_b(o->_list, o->size, o->fcmp, knh_String_tobytes(key));
@@ -502,7 +539,7 @@ void knh_DictMap_set(Ctx *ctx, DictMap *o, String *key, Any *v)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictMap_remove(Ctx *ctx, DictMap *o, String *key)
+void knh_DictMap_remove(Ctx *ctx, knh_DictMap_t *o, knh_String_t *key)
 {
 	int loc = knh_dict_index_b(o->_list, o->size, o->fcmp, knh_String_tobytes(key));
 	if(loc != 1) {
@@ -513,7 +550,7 @@ void knh_DictMap_remove(Ctx *ctx, DictMap *o, String *key)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictMap_clear(Ctx *ctx, DictMap *o)
+void knh_DictMap_clear(Ctx *ctx, knh_DictMap_t *o)
 {
 	size_t i, capacity = knh_dict_capacity(o->_list);
 	for(i = 0; i < capacity; i++) {
@@ -534,9 +571,7 @@ knh_dict_t *knh_dictset_malloc(Ctx *ctx, size_t init)
 
 /* ------------------------------------------------------------------------ */
 
-#define _new_DictSet(ctx, c)   new_DictSet0(ctx, c)
-
-DictSet* new_DictSet0(Ctx *ctx, knh_intptr_t capacity)
+knh_DictSet_t* new_DictSet0(Ctx *ctx, knh_intptr_t capacity)
 {
 	knh_DictSet_t *o = (knh_DictSet_t*)new_hObject(ctx, FLAG_DictSet, CLASS_DictSet, CLASS_DictSet);
 	o->_list = knh_dict_malloc(ctx, capacity, knh_dictset_finit, knh_dictset_ftraverse, knh_dict_fcmp);
@@ -548,14 +583,14 @@ DictSet* new_DictSet0(Ctx *ctx, knh_intptr_t capacity)
 /* ======================================================================== */
 /* [methods] */
 
-size_t knh_DictSet_size(DictSet *o)
+size_t knh_DictSet_size(knh_DictSet_t *o)
 {
 	return o->size;
 }
 
 /* ------------------------------------------------------------------------ */
 
-String* knh_DictSet_keyAt(DictSet *o, size_t n)
+knh_String_t* knh_DictSet_keyAt(knh_DictSet_t *o, size_t n)
 {
 	KNH_ASSERT(n < o->size);
 	return o->list[n].key;
@@ -563,7 +598,7 @@ String* knh_DictSet_keyAt(DictSet *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-knh_uintptr_t knh_DictSet_valueAt(DictSet *o, size_t n)
+knh_uintptr_t knh_DictSet_valueAt(knh_DictSet_t *o, size_t n)
 {
 	KNH_ASSERT(n < o->size);
 	return o->list[n].value;
@@ -571,7 +606,7 @@ knh_uintptr_t knh_DictSet_valueAt(DictSet *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictSet_removeAt(Ctx *ctx, DictSet *o, size_t n)
+void knh_DictSet_removeAt(Ctx *ctx, knh_DictSet_t *o, size_t n)
 {
 	KNH_ASSERT(n < o->size);
 	o->list[n].value = 0;
@@ -579,7 +614,7 @@ void knh_DictSet_removeAt(Ctx *ctx, DictSet *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-String *new_String__DictSet(Ctx *ctx, DictSet *o, knh_bytes_t key)
+knh_String_t *new_String__DictSet(Ctx *ctx, knh_DictSet_t *o, knh_bytes_t key)
 {
 	knh_index_t loc = knh_dict_index_b(o->_list, o->size, o->fcmp, key);
 	if(loc == -1) {
@@ -592,7 +627,7 @@ String *new_String__DictSet(Ctx *ctx, DictSet *o, knh_bytes_t key)
 
 /* ------------------------------------------------------------------------ */
 
-knh_uintptr_t knh_DictSet_get__b(DictSet *o, knh_bytes_t key)
+knh_uintptr_t knh_DictSet_get__b(knh_DictSet_t *o, knh_bytes_t key)
 {
 	knh_intptr_t loc = knh_dict_index_b(o->_list, o->size, o->fcmp, key);
 	return (loc == -1) ? 0 : o->list[loc].value;
@@ -600,7 +635,7 @@ knh_uintptr_t knh_DictSet_get__b(DictSet *o, knh_bytes_t key)
 
 /* ------------------------------------------------------------------------ */
 
-knh_uintptr_t knh_DictSet_get(Ctx *ctx, DictSet *o, String *key)
+knh_uintptr_t knh_DictSet_get(Ctx *ctx, knh_DictSet_t *o, knh_String_t *key)
 {
 	knh_intptr_t loc = knh_dict_index_b(o->_list, o->size, o->fcmp, knh_String_tobytes(key));
 	return (loc == -1) ? 0 : o->list[loc].value;
@@ -608,7 +643,7 @@ knh_uintptr_t knh_DictSet_get(Ctx *ctx, DictSet *o, String *key)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictSet_append(Ctx *ctx, DictSet *o, String *key, knh_uintptr_t n)
+void knh_DictSet_append(Ctx *ctx, knh_DictSet_t *o, knh_String_t *key, knh_uintptr_t n)
 {
 	if(o->_list == NULL) {
 		o->_list = knh_dict_malloc(ctx, 8, knh_dictmap_finit, knh_dictmap_ftraverse, knh_dict_fcmp);
@@ -627,7 +662,7 @@ void knh_DictSet_append(Ctx *ctx, DictSet *o, String *key, knh_uintptr_t n)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictSet_set(Ctx *ctx, DictSet *o, String *key, knh_uintptr_t n)
+void knh_DictSet_set(Ctx *ctx, knh_DictSet_t *o, knh_String_t *key, knh_uintptr_t n)
 {
 	int loc = knh_dict_index_b(o->_list, o->size, o->fcmp, knh_String_tobytes(key));
 	if(loc != -1) {
@@ -641,7 +676,7 @@ void knh_DictSet_set(Ctx *ctx, DictSet *o, String *key, knh_uintptr_t n)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictSet_add(Ctx *ctx, DictSet *o, String *key)
+void knh_DictSet_add(Ctx *ctx, knh_DictSet_t *o, knh_String_t *key)
 {
 	int loc = knh_dict_index_b(o->_list, o->size, o->fcmp, knh_String_tobytes(key));
 	if(loc != -1) {
@@ -655,7 +690,7 @@ void knh_DictSet_add(Ctx *ctx, DictSet *o, String *key)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictSet_remove(Ctx *ctx, DictSet *o, String *key)
+void knh_DictSet_remove(Ctx *ctx, knh_DictSet_t *o, knh_String_t *key)
 {
 	int loc = knh_dict_index_b(o->_list, o->size, o->fcmp, knh_String_tobytes(key));
 	if(loc != 1) {
@@ -666,7 +701,7 @@ void knh_DictSet_remove(Ctx *ctx, DictSet *o, String *key)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictSet_clear(Ctx *ctx, DictSet *o)
+void knh_DictSet_clear(Ctx *ctx, knh_DictSet_t *o)
 {
 	size_t i, capacity = knh_dict_capacity(o->_list);
 	for(i = 0; i < capacity; i++) {
