@@ -39,17 +39,17 @@ extern "C" {
 /* ======================================================================== */
 /* [constructors] */
 
-ResultSet* new_ResultSet(Ctx *ctx)
+knh_ResultSet_t* new_ResultSet(Ctx *ctx)
 {
-	return (ResultSet*)new_Object_bcid(ctx, CLASS_ResultSet, 0);
+	return (knh_ResultSet_t*)new_Object_bcid(ctx, CLASS_ResultSet, 0);
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_bool_t knh_ResultSet_next(Ctx *ctx, ResultSet *o)
+knh_bool_t knh_ResultSet_next(Ctx *ctx, knh_ResultSet_t *o)
 {
 	if(DP(o)->dbcur != NULL) {
-		if(DP(DP(o)->conn)->df->dbcurnext(ctx, DP(o)->dbcur, o)) {
+		if(DP(o)->conn->df->dbcurnext(ctx, DP(o)->dbcur, o)) {
 			DP(o)->count += 1;
 			return 1;
 		}
@@ -63,17 +63,8 @@ knh_bool_t knh_ResultSet_next(Ctx *ctx, ResultSet *o)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method Boolean! ResultSet.next() */
 
-METHOD knh__ResultSet_next(Ctx *ctx, knh_sfp_t *sfp)
-{
-	KNH_RETURN_Boolean(ctx, sfp, knh_ResultSet_next(ctx, (ResultSet*)sfp[0].o))
-}
-
-/* ------------------------------------------------------------------------ */
-/* @method void ResultSet.close() */
-
-void knh_ResultSet_close(Ctx *ctx, ResultSet *o)
+void knh_ResultSet_close(Ctx *ctx, knh_ResultSet_t *o)
 {
 	if(DP(o)->dbcur != NULL) {
 		DP(o)->dbcur_free(DP(o)->dbcur);
@@ -85,7 +76,7 @@ void knh_ResultSet_close(Ctx *ctx, ResultSet *o)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_ResultSet_initColumn(Ctx *ctx, ResultSet *o, size_t column_size)
+KNHAPI(void) knh_ResultSet_initColumn(Ctx *ctx, knh_ResultSet_t *o, size_t column_size)
 {
 	size_t i;
 	if(DP(o)->column_size != 0) {
@@ -136,14 +127,14 @@ KNHAPI(void) knh_ResultSet_initColumn(Ctx *ctx, ResultSet *o, size_t column_size
 /* ======================================================================== */
 /* [meta] */
 
-void knh_ResultSet_initTargetClass(ResultSet *o, knh_class_t tcid)
+void knh_ResultSet_initTargetClass(knh_ResultSet_t *o, knh_class_t tcid)
 {
 	DP(o)->tcid = tcid;
 }
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_ResultSet_setName(Ctx *ctx, ResultSet *o, size_t n, String *name)
+KNHAPI(void) knh_ResultSet_setName(Ctx *ctx, knh_ResultSet_t *o, size_t n, knh_String_t *name)
 {
 	KNH_ASSERT(n < DP(o)->column_size);
 	KNH_SETv(ctx, DP(o)->column[n].name, name);
@@ -151,7 +142,7 @@ KNHAPI(void) knh_ResultSet_setName(Ctx *ctx, ResultSet *o, size_t n, String *nam
 
 /* ------------------------------------------------------------------------ */
 
-String *knh_ResultSet_getName(Ctx *ctx, ResultSet *o, size_t n)
+knh_String_t *knh_ResultSet_getName(Ctx *ctx, knh_ResultSet_t *o, size_t n)
 {
 	KNH_ASSERT(n < DP(o)->column_size);
 	return DP(o)->column[n].name;
@@ -159,7 +150,7 @@ String *knh_ResultSet_getName(Ctx *ctx, ResultSet *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-int knh_ResultSet_findColumn(Ctx *ctx, ResultSet *o, knh_bytes_t name)
+int knh_ResultSet_findColumn(Ctx *ctx, knh_ResultSet_t *o, knh_bytes_t name)
 {
 	int i = 0;
 	for(i = 0; i < DP(o)->column_size; i++) {
@@ -170,7 +161,7 @@ int knh_ResultSet_findColumn(Ctx *ctx, ResultSet *o, knh_bytes_t name)
 
 /* ------------------------------------------------------------------------ */
 
-knh_type_t knh_ResultSet_get_type(Ctx *ctx, ResultSet *o, size_t n)
+knh_type_t knh_ResultSet_get_type(Ctx *ctx, knh_ResultSet_t *o, size_t n)
 {
 	KNH_ASSERT(n < DP(o)->column_size);
 	return DP(o)->column[n].type;
@@ -179,7 +170,7 @@ knh_type_t knh_ResultSet_get_type(Ctx *ctx, ResultSet *o, size_t n)
 /* ======================================================================== */
 /* [set] */
 
-KNHAPI(void) knh_ResultSet_initData(Ctx *ctx, ResultSet *o)
+KNHAPI(void) knh_ResultSet_initData(Ctx *ctx, knh_ResultSet_t *o)
 {
 	int i = 0;
 	for(i = 0; i < DP(o)->column_size; i++) {
@@ -192,7 +183,7 @@ KNHAPI(void) knh_ResultSet_initData(Ctx *ctx, ResultSet *o)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_ResultSet_setInt(Ctx *ctx, ResultSet *o, size_t n, knh_int_t value)
+KNHAPI(void) knh_ResultSet_setInt(Ctx *ctx, knh_ResultSet_t *o, size_t n, knh_int_t value)
 {
 	KNH_ASSERT(n < DP(o)->column_size);
 	knh_bytes_t t = {(knh_uchar_t*)(&value), sizeof(knh_int_t)};
@@ -204,7 +195,7 @@ KNHAPI(void) knh_ResultSet_setInt(Ctx *ctx, ResultSet *o, size_t n, knh_int_t va
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_ResultSet_setFloat(Ctx *ctx, ResultSet *o, size_t n, knh_float_t value)
+KNHAPI(void) knh_ResultSet_setFloat(Ctx *ctx, knh_ResultSet_t *o, size_t n, knh_float_t value)
 {
 	KNH_ASSERT(n < DP(o)->column_size);
 	knh_bytes_t t = {(knh_uchar_t*)(&value), sizeof(knh_float_t)};
@@ -218,7 +209,7 @@ KNHAPI(void) knh_ResultSet_setFloat(Ctx *ctx, ResultSet *o, size_t n, knh_float_
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_ResultSet_setText(Ctx *ctx, ResultSet *o, size_t n, knh_bytes_t t)
+KNHAPI(void) knh_ResultSet_setText(Ctx *ctx, knh_ResultSet_t *o, size_t n, knh_bytes_t t)
 {
 	KNH_ASSERT(n < DP(o)->column_size);
 	DP(o)->column[n].ctype = knh_ResultSet_CTYPE__text;
@@ -231,7 +222,7 @@ KNHAPI(void) knh_ResultSet_setText(Ctx *ctx, ResultSet *o, size_t n, knh_bytes_t
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_ResultSet_setBlob(Ctx *ctx, ResultSet *o, size_t n, knh_bytes_t t)
+KNHAPI(void) knh_ResultSet_setBlob(Ctx *ctx, knh_ResultSet_t *o, size_t n, knh_bytes_t t)
 {
 	KNH_ASSERT(n < DP(o)->column_size);
 	DP(o)->column[n].ctype = knh_ResultSet_CTYPE__bytes;
@@ -244,7 +235,7 @@ KNHAPI(void) knh_ResultSet_setBlob(Ctx *ctx, ResultSet *o, size_t n, knh_bytes_t
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_ResultSet_setNULL(Ctx *ctx, ResultSet *o, size_t n)
+KNHAPI(void) knh_ResultSet_setNULL(Ctx *ctx, knh_ResultSet_t *o, size_t n)
 {
 	KNH_ASSERT(n < DP(o)->column_size);
 	DP(o)->column[n].ctype = knh_ResultSet_CTYPE__null;
@@ -254,7 +245,7 @@ KNHAPI(void) knh_ResultSet_setNULL(Ctx *ctx, ResultSet *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-knh_int_t knh_ResultSet_getInt(Ctx *ctx, ResultSet *o, size_t n)
+knh_int_t knh_ResultSet_getInt(Ctx *ctx, knh_ResultSet_t *o, size_t n)
 {
 	KNH_ASSERT(n < DP(o)->column_size);
 	char *p = knh_Bytes_tochar(DP(o)->databuf) + DP(o)->column[n].start;
@@ -274,7 +265,7 @@ knh_int_t knh_ResultSet_getInt(Ctx *ctx, ResultSet *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-knh_float_t knh_ResultSet_getFloat(Ctx *ctx, ResultSet *o, size_t n)
+knh_float_t knh_ResultSet_getFloat(Ctx *ctx, knh_ResultSet_t *o, size_t n)
 {
 	KNH_ASSERT(n < DP(o)->column_size);
 	char *p = knh_Bytes_tochar(DP(o)->databuf) + DP(o)->column[n].start;
@@ -294,13 +285,13 @@ knh_float_t knh_ResultSet_getFloat(Ctx *ctx, ResultSet *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-String* knh_ResultSet_getString(Ctx *ctx, ResultSet *o, size_t n)
+knh_String_t* knh_ResultSet_getString(Ctx *ctx, knh_ResultSet_t *o, size_t n)
 {
 	KNH_ASSERT(n < DP(o)->column_size);
 	char *p = knh_Bytes_tochar(DP(o)->databuf) + DP(o)->column[n].start;
 	switch(DP(o)->column[n].ctype) {
 	case knh_ResultSet_CTYPE__null :
-		return (String*)KNH_NULL;
+		return (knh_String_t*)KNH_NULL;
 	case knh_ResultSet_CTYPE__integer :
 		return new_String__int(ctx, (knh_int_t)(*((knh_int_t*)p)));
 	case knh_ResultSet_CTYPE__float :
@@ -308,7 +299,7 @@ String* knh_ResultSet_getString(Ctx *ctx, ResultSet *o, size_t n)
 	case knh_ResultSet_CTYPE__text :
 		return new_String(ctx, B2(p, DP(o)->column[n].len), NULL);
 	}
-	return (String*)KNH_NULL;
+	return (knh_String_t*)KNH_NULL;
 }
 
 ///* ======================================================================== */
