@@ -35,16 +35,6 @@
 extern "C" {
 #endif
 
-/* ======================================================================== */
-/* [macro] */
-
-#define _NSN_main           0
-#define _FILEN_unknown      0
-
-/* ======================================================================== */
-/* [constructors] */
-
-
 /* ======================================================================== */
 /* [properties] */
 
@@ -66,9 +56,8 @@ knh_bytes_nzenvkey(knh_bytes_t t, char *buf, size_t bufsiz)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method Any System.getProperty(String! key) */
 
-Any* knh_System_getProperty(Ctx *ctx, System *o, knh_bytes_t key)
+Any* knh_System_getProperty(Ctx *ctx, knh_System_t *o, knh_bytes_t key)
 {
 	if(knh_bytes_startsWith(key, STEXT("env."))) {
 		char envkey[256];
@@ -83,19 +72,27 @@ Any* knh_System_getProperty(Ctx *ctx, System *o, knh_bytes_t key)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void System.setProperty(String! key, Any value) */
 
-void knh_System_setProperty(Ctx *ctx, System *o, String *key, Any *value)
+void knh_System_setProperty(Ctx *ctx, knh_System_t *o, knh_String_t *key, Any *value)
 {
-	knh_DictMap_set(ctx, DP(o)->props, key, value);
+	knh_type_t type = knh_getPropertyType(ctx, knh_String_tobytes(key));
+	if(type == TYPE_Any || knh_Object_cid(value) == CLASS_type(type)) {
+		knh_DictMap_set(ctx, DP(o)->props, key, value);
+	}
+	else {
+		knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
+		knh_printf(ctx, cwb->w, "PropertyType!!: $%s must be %T", knh_String_text(ctx, key), type);
+		knh_String_t *s = knh_cwb_newString(ctx, cwb);
+		KNH_THROW(ctx, s);
+	}
 }
 
 /* ------------------------------------------------------------------------ */
 
 KNHAPI(void) knh_setSystemPropertyText(Ctx *ctx, char *key, char *value)
 {
-	String *k = T(key);
-	String *v = T(value);
+	knh_String_t *k = T__(key);
+	knh_String_t *v = T__(value);
 	knh_DictMap_set(ctx, DP(ctx->sys)->props, k, UP(v));
 }
 
