@@ -79,7 +79,7 @@ KNHAPI(void) knh_println(Ctx *ctx, knh_OutputStream_t *w, knh_bytes_t s)
 
 KNHAPI(void) knh_write_EOL(Ctx *ctx, knh_OutputStream_t *w)
 {
-	knh_OutputStream_write(ctx, w, __tobytes(DP(w)->NEWLINE));
+	knh_OutputStream_write(ctx, w, knh_String_tobytes(DP(w)->NEWLINE));
 	if(knh_OutputStream_isAutoFlush(w)) {
 		knh_OutputStream_flush(ctx, w);
 	}
@@ -90,7 +90,7 @@ KNHAPI(void) knh_write_EOL(Ctx *ctx, knh_OutputStream_t *w)
 
 KNHAPI(void) knh_write_TAB(Ctx *ctx, knh_OutputStream_t *w)
 {
-	knh_OutputStream_write(ctx, w, __tobytes(DP(w)->TAB));
+	knh_OutputStream_write(ctx, w, knh_String_tobytes(DP(w)->TAB));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -105,7 +105,7 @@ KNHAPI(void) knh_write_BOL(Ctx *ctx, knh_OutputStream_t *w)
 		//knh_write(ctx, w, STEXT("/*"));
 		//knh_write_ifmt(ctx, w, KNH_INT_FMT, i);
 		//knh_write(ctx, w, STEXT("*/"));
-		knh_OutputStream_write(ctx, w, __tobytes(DP(w)->TAB));
+		knh_OutputStream_write(ctx, w, knh_String_tobytes(DP(w)->TAB));
 	}
 	knh_OutputStream_setBOL(w, 0);
 }
@@ -207,8 +207,8 @@ void knh_write_cid(Ctx *ctx, knh_OutputStream_t *w, knh_class_t cid)
 
 void knh_write_lcid(Ctx *ctx, knh_OutputStream_t *w, knh_class_t cid)
 {
-	DBG2_ASSERT_cid(cid);
-	knh_write(ctx, w, __tobytes(ClassTable(cid).lname));
+	KNH_ASSERT_cid(cid);
+	knh_write(ctx, w, knh_String_tobytes(ClassTable(cid).lname));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -244,7 +244,7 @@ void knh_write_mn(Ctx *ctx, knh_OutputStream_t *w, knh_type_t rtype, knh_methodn
 void knh_write_type_(Ctx *ctx, knh_OutputStream_t *w, knh_type_t type, int isLongName)
 {
 	knh_class_t cid = CLASS_type(type);
-	DBG2_ASSERT_cid(cid);
+	KNH_ASSERT_cid(cid);
 	if(type == TYPE_void) {
 		knh_write(ctx, w, STEXT("void"));
 	}
@@ -286,13 +286,10 @@ void knh_write_fline(Ctx *ctx, knh_OutputStream_t *w, char *file, int line)
 
 KNHAPI(void) knh_format(Ctx *ctx, knh_OutputStream_t *w, knh_methodn_t mn, Object *o, Any *m)
 {
-	knh_sfp_t sfpbuf;
-	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
-	sfpbuf.o = o;
-	sfpbuf.data = knh_Object_data(o);
-	knh_stack_w(ctx, lsfp, &sfpbuf, mn, w, m);
-	KNH_LOCALBACK(ctx, lsfp);
-
+	knh_sfp_t *esp = KNH_LOCAL(ctx);
+	KNH_SETv(ctx, esp[1].o, o);
+	esp[1].data = knh_Object_data(o);
+	knh_esp1_format(ctx, mn, w, m);
 }
 
 /* ======================================================================== */
@@ -300,7 +297,7 @@ KNHAPI(void) knh_format(Ctx *ctx, knh_OutputStream_t *w, knh_methodn_t mn, Objec
 
 static char* knh_vprintf_parseindex(char *p, int *index)
 {
-	char *ptr = p+1;
+    char *ptr = p+1;
 	if(ptr[0] == '{' && isdigit(ptr[1]) && ptr[2] == '}') {
 		*index = ptr[1] - '0';
 		ptr += 3;
@@ -356,13 +353,8 @@ void knh_vprintf(Ctx *ctx, knh_OutputStream_t *w, char *fmt, va_list ap)
 	while((ch = *c) != '\0') {
 		c++;
 		if(ch == '%') {
-			int index;
 			ch = *c;
-			if(ch == '%') {
-				c++;
-				continue;
-			}
-			index = bindex++;
+			int index = bindex++;
 			c = knh_vprintf_parseindex(c++, &index);
 			//DBG2_P("bindex=%d, index=%d", bindex, index);
 			switch(ch) {
@@ -400,12 +392,6 @@ void knh_vprintf(Ctx *ctx, knh_OutputStream_t *w, char *fmt, va_list ap)
 				case 'B':
 					args[index].atype = VA_BYTES;
 					break;
-				// TODO
-				// we should care if "fmt" has "%%".
-				// sometimes, next args is NULL.
-				case '%':
-					index--;
-					c++;
 				default:
 					bindex--;
 			}
@@ -492,13 +478,6 @@ void knh_vprintf(Ctx *ctx, knh_OutputStream_t *w, char *fmt, va_list ap)
 				  knh_print(ctx, w, b);
 				}
 				ch = *c;
-				if(ch == '%') {
-					knh_putc(ctx, w, '%');
-					c++;
-					b.buf = (knh_uchar_t*)c;
-					b.len = 0;
-					continue;
-				}
 				int index = bindex++;
 				c = knh_vprintf_parseindex(++c, &index);
 
@@ -560,9 +539,6 @@ void knh_vprintf(Ctx *ctx, knh_OutputStream_t *w, char *fmt, va_list ap)
 						KNH_ASSERT(args[index].atype == VA_BYTES);
 						knh_write(ctx,w, args[index].bvalue);
 						break;
-					case '%':
-						index--;
-						bindex--;
 					default:
 						//knh_putc(ctx, w, '%');
 						knh_putc(ctx, w, ch);
@@ -586,6 +562,7 @@ void knh_vprintf(Ctx *ctx, knh_OutputStream_t *w, char *fmt, va_list ap)
 
 /* ------------------------------------------------------------------------ */
 
+
 KNHAPI(void) knh_printf(Ctx *ctx, knh_OutputStream_t *w, char *fmt, ...)
 {
 	va_list ap;
@@ -709,6 +686,19 @@ METHOD knh_fmethod_formatter(Ctx *ctx, knh_sfp_t *sfp)
 	KNH_RETURN_void(ctx, sfp);
 }
 
+/* ------------------------------------------------------------------------ */
+
+knh_bool_t knh_stack_isRecuriveFormatting(Ctx *ctx, knh_sfp_t *sfp, Object *o, knh_OutputStream_t *w, void *mtdcode)
+{
+	knh_sfp_t *sp = sfp - 1;
+	while(ctx->stack < sp) {
+		if(sp[0].o == o && sp[1].w == w && IS_Method(sp[-1].mtd)) {
+			if(DP(sp[-1].mtd)->code == mtdcode) return 1;
+		}
+		sp--;
+	}
+	return 0;
+}
 
 /* ------------------------------------------------------------------------ */
 
