@@ -1,8 +1,10 @@
 #ifndef COMMONS_H_
 #define COMMONS_H_
 
-#include<konoha/konoha_dev.h>
-#include<konoha/gen/konohac_klr_.h>
+#define KONOHA_MACROS
+#include"../../include/konoha.h"
+#include"../../include/konoha/konoha_stmt_.h"
+#include"../../include/konoha/konoha_code_.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -11,26 +13,58 @@ extern "C" {
 /* ------------------------------------------------------------------------ */
 
 #undef IS_Token
-#define IS_Token(o)      (((Token*)o)->h.cid == CLASS_Token)
+#define IS_Token(o)      (((knh_Token_t*)o)->h.cid == CLASS_Token)
 #define TOKENNo(o)        knh_token_tochar(SP(o)->tt)
 
 #undef IS_Stmt
-#define IS_Stmt(o)       (((Stmt*)o)->h.cid == CLASS_Stmt)
+#define IS_Stmt(o)       (((knh_Stmt_t*)o)->h.cid == CLASS_Stmt)
 #define STMTNo(o)         knh_stmt_tochar(SP(o)->stt)
 
+
+#define knh_token_isNested(tt)  \
+	(tt == TT_BRACE || tt == TT_PARENTHESIS || tt == TT_BRANCET)
+
+#define TT_(tk)      SP(tk)->tt
+#define STT_(stmt)   SP(stmt)->stt
+
+#define IS_EMPTY(tc)   ((tc)->c >= (tc)->e)
+#define HAS_TOKEN(tc)    ((tc)->c < (tc)->e)
+
+#define TK0(tc)          (tc)->ts[(tc)->c]
+#define TK1(tc)          (tc)->ts[((tc)->c)+1]
+#define knh_Stmt_addT(ctx, stmt, tk)    knh_Stmt_add(ctx, stmt, TM(tk))
+#define IST_(tk, t)   knh_String_equals(DP(tk)->text, STEXT(t))
+#define TT_IN TT_HAS
+#define TTn_(tk)         DP(tk)->tt_next
+
 /* ------------------------------------------------------------------------ */
 
 #define FL(o)            ((Object*)o)
 #define TM(o)            ((Term*)o)
 
+/* error */
 #define KERR_ERROR     0
-#define KERR_EWARN     1
-#define KERR_DWARN     2
-#define KERR_ERRATA    3
-#define KERR_INFO      4
+#define KERR_TERROR    1
+/* warning */
+#define KERR_EWARN     2
+#define KERR_DWARN     3
+#define KERR_ERRATA    4
+/* info */
+#define KERR_BAD       5
+#define KERR_TINFO     6
+#define KERR_NOTICE    7
+#define KERR_INFO      8
 
 #define _(s)           s
 
+
+#define DECL_OUTERPARAM  0
+#define DECL_INNERPARAM  1
+#define DECL_FOREACH     2
+#define DECL_LOCAL       3
+#define DECL_FIELD       4
+#define DECL_SCRIPT      5
+
 /* ------------------------------------------------------------------------ */
 
 #define TWARN_  0
@@ -40,8 +74,7 @@ extern "C" {
 
 /* ------------------------------------------------------------------------ */
 
-#define TS_MN(mn)  knh_tfieldn_name(METHODN_TOFIELDN(mn))
-#define TS_FN(fn)  knh_tfieldn_name(fm)
+#define KNH_ASM_JMP(ctx, abr, l)  KNH_ASM_JMP_(ctx, abr, l);
 
 /* ------------------------------------------------------------------------ */
 
@@ -61,24 +94,24 @@ extern "C" {
 
 #define DEBUG3(fmt, ...) \
 	fflush(stdout); \
-	fprintf(stderr, "\nDEBUG3[%s:%d/%s]: ", KNH_SAFEFILE(__FILE__), __LINE__, __FUNCTION__); \
+	fprintf(stderr, "\nDEBUG3[%s:%d/%s]: ", knh_safefile(__FILE__), __LINE__, __FUNCTION__); \
 	fprintf(stderr, fmt, ## __VA_ARGS__); \
 	fprintf(stderr, "\n"); \
 
 #define TODO3(fmt, ...) \
 	fflush(stdout); \
-	fprintf(stderr, "\nTODO3[%s:%d/%s]: ", KNH_SAFEFILE(__FILE__), __LINE__, __FUNCTION__); \
+	fprintf(stderr, "\nTODO3[%s:%d/%s]: ", knh_safefile(__FILE__), __LINE__, __FUNCTION__); \
 	fprintf(stderr, fmt, ## __VA_ARGS__); \
 	fprintf(stderr, "\n"); \
 
 #define DEBUG3_PTC(name, tc) \
 	fflush(stdout); \
-	fprintf(stderr, "\nDEBUG3[%s:%d/%s]: ", KNH_SAFEFILE(__FILE__), __LINE__, __FUNCTION__); \
+	fprintf(stderr, "\nDEBUG3[%s:%d/%s]: ", knh_safefile(__FILE__), __LINE__, __FUNCTION__); \
 	fprintf(stderr, "%s: c=%d, e=%d\n", name, (tc)->c, (tc)->e); \
 
 #define DEBUG3_STC(name, tc) \
 	fflush(stdout); \
-	fprintf(stderr, "\nDEBUG3[%s:%d/%s]: ", KNH_SAFEFILE(__FILE__), __LINE__, __FUNCTION__); \
+	fprintf(stderr, "\nDEBUG3[%s:%d/%s]: ", knh_safefile(__FILE__), __LINE__, __FUNCTION__); \
 	fprintf(stderr, "%s: c=%d, e=%d\n", name, (tc).c, (tc).e); \
 
 #else
