@@ -39,7 +39,7 @@ extern "C" {
 /* [main] */
 
 static
-int knh_NameSpace_compile(Ctx *ctx, NameSpace *ns, Stmt *stmt, int isEval);
+int knh_NameSpace_compile(Ctx *ctx, knh_NameSpace_t *ns, knh_Stmt_t *stmt, int isEval);
 
 /* ------------------------------------------------------------------------ */
 /* [import script] */
@@ -58,10 +58,10 @@ knh_bool_t knh_bytes_isSystemScript(knh_bytes_t path)
 /* ------------------------------------------------------------------------ */
 
 static
-void knh_NameSpace_loaded(Ctx *ctx, NameSpace *ns, knh_uri_t uri)
+void knh_NameSpace_loaded(Ctx *ctx, knh_NameSpace_t *ns, knh_uri_t uri)
 {
 	KNH_ASSERT(IS_NameSpace(ns));
-	String *s = knh_getResourceName(ctx, uri);
+	knh_String_t *s = knh_getResourceName(ctx, uri);
 	knh_uintptr_t trust = KNH_PATH_UNTRUSTED;
 	if(URI_ISTRUSTED(uri)) {
 		trust = KNH_PATH_TRUSTED;
@@ -75,10 +75,10 @@ void knh_NameSpace_loaded(Ctx *ctx, NameSpace *ns, knh_uri_t uri)
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_NameSpace_isLoaded(Ctx *ctx, NameSpace *ns, knh_uri_t uri)
+int knh_NameSpace_isLoaded(Ctx *ctx, knh_NameSpace_t *ns, knh_uri_t uri)
 {
 	if(IS_NOTNULL(DP(ns)->pathTrustDictSet)) {
-		String *s = knh_getResourceName(ctx, uri);
+		knh_String_t *s = knh_getResourceName(ctx, uri);
 		knh_uintptr_t n = knh_DictSet_get__b(DP(ns)->pathTrustDictSet, knh_String_tobytes(s));
 		return (n != 0);
 	}
@@ -87,7 +87,7 @@ int knh_NameSpace_isLoaded(Ctx *ctx, NameSpace *ns, knh_uri_t uri)
 
 /* ------------------------------------------------------------------------ */
 
-InputStream* new_ScriptInputStream(Ctx *ctx, knh_bytes_t path, knh_cwb_t *cwb, NameSpace *ns, int isThrowable)
+knh_InputStream_t* new_ScriptInputStream(Ctx *ctx, knh_bytes_t path, knh_cwb_t *cwb, knh_NameSpace_t *ns, int isThrowable)
 {
 	knh_uri_t uri = 0;
 	knh_iodrv_t *drv = NULL;
@@ -102,7 +102,7 @@ InputStream* new_ScriptInputStream(Ctx *ctx, knh_bytes_t path, knh_cwb_t *cwb, N
 		drv = knh_getIODriver(ctx, STEXT("file"));
 		knh_cwb_ospath(ctx, cwb);
 		if(!knh_cwb_isfile(ctx, cwb) && knh_bytes_isSystemScript(path)) {
-			String *spath = (String*)knh_Context_getProperty(ctx, (Context*)ctx, STEXT("konoha.script.path"));
+			knh_String_t *spath = (knh_String_t*)knh_Context_getProperty(ctx, (knh_Context_t*)ctx, STEXT("konoha.script.path"));
 			if(IS_bString(spath)) {
 				knh_cwb_subclear(cwb, 0);
 				knh_cwb_write(ctx, cwb, knh_String_tobytes(spath));
@@ -115,7 +115,7 @@ InputStream* new_ScriptInputStream(Ctx *ctx, knh_bytes_t path, knh_cwb_t *cwb, N
 					uri = URI_TRUSTED(uri);
 				}
 			}
-			spath = (String*)knh_Context_getProperty(ctx, (Context*)ctx, STEXT("user.script.path"));
+			spath = (knh_String_t*)knh_Context_getProperty(ctx, (knh_Context_t*)ctx, STEXT("user.script.path"));
 			if(IS_bString(spath) && uri == 0) {
 				knh_cwb_subclear(cwb, 0);
 				knh_cwb_write(ctx, cwb, knh_String_tobytes(spath));
@@ -140,15 +140,15 @@ InputStream* new_ScriptInputStream(Ctx *ctx, knh_bytes_t path, knh_cwb_t *cwb, N
 	if(knh_NameSpace_isLoaded(ctx, ns, uri)) {
 		KNH_WARNING(ctx, "Already imported: %s", (char*)path.buf);
 		if(!knh_ask(ctx, "Do you want to reload it [y/N] ?", 0)) {
-			return (InputStream*)KNH_DEF(ctx, CLASS_InputStream);
+			return (knh_InputStream_t*)KNH_DEF(ctx, CLASS_InputStream);
 		}
 	}
 	{
-		String *spath = knh_getResourceName(ctx, uri);
+		knh_String_t *spath = knh_getResourceName(ctx, uri);
 		path = knh_String_tobytes(spath);
 		KNH_NOTICE(ctx, "importing script: %s", path.buf);
 		knh_io_t fd = drv->fopen(ctx, path, "r", isThrowable);
-		InputStream *in = new_InputStream__io(ctx, spath, fd, drv);
+		knh_InputStream_t *in = new_InputStream__io(ctx, spath, fd, drv);
 		if(!knh_InputStream_isClosed(ctx, in)) {
 			DP(in)->uri = uri;
 			if(!knh_bytes_startsWith(path, STEXT("http://"))) {
@@ -161,20 +161,23 @@ InputStream* new_ScriptInputStream(Ctx *ctx, knh_bytes_t path, knh_cwb_t *cwb, N
 
 /* ------------------------------------------------------------------------ */
 
-Stmt *knh_InputStream_parseStmt(Ctx *ctx, InputStream *in, int isData)
+knh_Stmt_t *knh_InputStream_parseStmt(Ctx *ctx, knh_InputStream_t *in, int isData)
 {
-	Token *tk = new_Token(ctx, 0, DP(in)->uri, 0, knh_char_totoken('{'));
+	knh_Token_t *tk = new_Token(ctx, 0, DP(in)->uri, 0, TT_BRACE);
 	KNH_LPUSH(ctx, tk);
 	KNH_LPUSH(ctx, in);
-	knh_Token_parse(ctx, tk, in);
-	//DBG2_DUMP(ctx, tk, KNH_NULL, "tokens");
-	return new_StmtINSTMT(ctx, tk, isData);
+	knh_InputStream_parseToken(ctx, in, tk);
+	DBG2_DUMP(ctx, tk, KNH_NULL, "tokens");
+	{
+		knh_tkc_t tcbuf, *tc = knh_Token_tc(ctx, tk, &tcbuf);
+		return new_StmtSTMTBLOCK(ctx, tc, isData);
+	}
 }
 
 /* ------------------------------------------------------------------------ */
 
 static
-void knh_Asm_openDynamicLinkLibrary(Ctx *ctx, Asm *abr, knh_uri_t uri)
+void knh_Asm_openDynamicLinkLibrary(Ctx *ctx, knh_Asm_t *abr, knh_uri_t uri)
 {
 	knh_bytes_t path = knh_String_tobytes(knh_getResourceName(ctx, uri));
 	if(path.buf[0] != '(' && !knh_bytes_startsWith(path, STEXT("http://"))) {  // check "(eval)", "(shell)", or o
@@ -203,7 +206,7 @@ void knh_Asm_openDynamicLinkLibrary(Ctx *ctx, Asm *abr, knh_uri_t uri)
 
 /* ------------------------------------------------------------------------ */
 
-int knh_NameSpace_load(Ctx *ctx, NameSpace *ns, InputStream *in, int isEval, int isThrowable)
+int knh_NameSpace_load(Ctx *ctx, knh_NameSpace_t *ns, knh_InputStream_t *in, int isEval, int isThrowable)
 {
 	int res = 0;
 	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
@@ -214,7 +217,7 @@ int knh_NameSpace_load(Ctx *ctx, NameSpace *ns, InputStream *in, int isEval, int
 	KNH_TRY(ctx, L_CATCH, lsfp, 1);
 	knh_Context_initAsm(ctx);   // initialization
 	{
-		Stmt *stmt = knh_InputStream_parseStmt(ctx, in, 0/*isData*/);
+		knh_Stmt_t *stmt = knh_InputStream_parseStmt(ctx, in, 0/*isData*/);
 		KNH_LPUSH(ctx, stmt);
 		if(URI_UNMASK(uri) != 0) {
 			DBG_DUMP(ctx, stmt, KNH_NULL, "stmt");
@@ -270,18 +273,18 @@ char *knh_cwb_packageScript(Ctx *ctx, knh_cwb_t *cwb, knh_bytes_t rootdir, knh_b
 static
 char *knh_lookupPackageScript(Ctx *ctx, knh_cwb_t *cwb, knh_bytes_t pkgname)
 {
-	String *spath;
+	knh_String_t *spath;
 	char *path = knh_getenv("KONOHA_PACKAGE");
 	if(path != NULL) {
 		path = knh_cwb_packageScript(ctx, cwb, B(path), pkgname);
 		if(path != NULL) return path;
 	}
-	spath = (String*)knh_Context_getProperty(ctx, (Context*)ctx, STEXT("konoha.package.path"));
+	spath = (knh_String_t*)knh_Context_getProperty(ctx, (knh_Context_t*)ctx, STEXT("konoha.package.path"));
 	if(IS_bString(spath)) {
 		path = knh_cwb_packageScript(ctx, cwb, knh_String_tobytes(spath), pkgname);
 		if(path != NULL) return path;
 	}
-	spath = (String*)knh_Context_getProperty(ctx, (Context*)ctx, STEXT("user.package.path"));
+	spath = (knh_String_t*)knh_Context_getProperty(ctx, (knh_Context_t*)ctx, STEXT("user.package.path"));
 	if(IS_bString(spath)){
 		path = knh_cwb_packageScript(ctx, cwb, knh_String_tobytes(spath), pkgname);
 		if(path != NULL) return path;
@@ -295,24 +298,24 @@ char *knh_lookupPackageScript(Ctx *ctx, knh_cwb_t *cwb, knh_bytes_t pkgname)
 /* ------------------------------------------------------------------------ */
 
 static
-NameSpace *knh_NameSpace_newPackageNULL(Ctx *ctx, knh_bytes_t pkgname)
+knh_NameSpace_t *knh_NameSpace_newPackageNULL(Ctx *ctx, knh_bytes_t pkgname)
 {
 	knh_uri_t uri = knh_getResourceId(ctx, pkgname);
-	String *nsname = knh_getResourceName(ctx, uri);
+	knh_String_t *nsname = knh_getResourceName(ctx, uri);
 	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
 	char *fpath = knh_lookupPackageScript(ctx, cwb, knh_String_tobytes(nsname));
-	NameSpace *ns = NULL;
+	knh_NameSpace_t *ns = NULL;
 	if(fpath != NULL) {
 		pkgname = knh_String_tobytes(nsname);
 		KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
-		ns = (NameSpace*)knh_DictMap_get__b(ctx,  DP(ctx->sys)->NameSpaceTableDictMap, pkgname);
+		ns = (knh_NameSpace_t*)knh_DictMap_get__b(ctx,  DP(ctx->sys)->NameSpaceTableDictMap, pkgname);
 		if(IS_NULL(ns)) {
 			ns = new_NameSpace(ctx, nsname);
 			knh_DictMap_set(ctx, DP(ctx->sys)->NameSpaceTableDictMap, nsname, UP(ns));
 			KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
 			knh_sfp_t *lsfp = KNH_LOCAL(ctx);
-			NameSpace *curns = knh_switchCurrentNameSpace(ctx, ns);
-			InputStream *in = new_ScriptInputStream(ctx, knh_cwb_tobytes(cwb), cwb, ctx->share->mainns, 0);
+			knh_NameSpace_t *curns = knh_switchAsmNameSpace(ctx, ns);
+			knh_InputStream_t *in = new_ScriptInputStream(ctx, knh_cwb_tobytes(cwb), cwb, ctx->share->mainns, 0);
 			KNH_LPUSH(ctx, in);
 			if(!knh_InputStream_isClosed(ctx, in)) {
 				knh_NameSpace_load(ctx, ns, in, 1/*isEval*/,0/*isThrowable*/);
@@ -321,7 +324,7 @@ NameSpace *knh_NameSpace_newPackageNULL(Ctx *ctx, knh_bytes_t pkgname)
 				KNH_WARNING(ctx, "package script does not exist");
 				ns = NULL;
 			}
-			knh_switchCurrentNameSpace(ctx, curns);
+			knh_switchAsmNameSpace(ctx, curns);
 			KNH_LOCALBACK(ctx, lsfp);
 		}
 		else {
@@ -332,53 +335,34 @@ NameSpace *knh_NameSpace_newPackageNULL(Ctx *ctx, knh_bytes_t pkgname)
 	return ns;
 }
 
-///* ------------------------------------------------------------------------ */
-//
-//Object *konohac_data(Ctx *ctx, InputStream *in, knh_class_t reqc)
-//{
-//	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
-//	KNH_LPUSH(ctx, in);
-//	KNH_LPUSH(ctx, new_ExceptionHandler(ctx));
-//	KNH_TRY(ctx, L_CATCH, lsfp, 1);
-//	{
-//		Stmt *stmt = knh_InputStream_parseStmt(ctx, in, 1/*isData*/);
-//		KNH_LPUSH(ctx, stmt);
-//		return knh_Stmt_toData(ctx, stmt, reqc);
-//	}
-//	/* catch */
-//	L_CATCH:;
-//	KNH_LOCALBACK(ctx, lsfp);
-//	//KNH_PRINT_STACKTRACE(ctx, lsfp, 1);
-//	return KNH_NULL;
-//}
 
 /* ======================================================================== */
 /* [CLASS] */
 
 static
-knh_flag_t knh_StmtCLASS_flag(Ctx *ctx, Stmt *o)
+knh_flag_t knh_StmtCLASS_flag(Ctx *ctx, knh_Stmt_t *o)
 {
 	knh_flag_t flag = 0;
 	if(IS_DictMap(DP(o)->metaDictMap)) {
 		Object *v = knh_DictMap_get__b(ctx, DP(o)->metaDictMap, STEXT("Final"));
 		if(IS_NOTNULL(v)) {
-			flag |= KNH_FLAG_CF_FINAL;
+			flag |= FLAG_Class_Final;
 		}
 		v = knh_DictMap_get__b(ctx, DP(o)->metaDictMap, STEXT("Private"));
 		if(IS_NOTNULL(v)) {
-			flag |= KNH_FLAG_CF_PRIVATE;
+			flag |= FLAG_Class_Private;
 		}
 		v = knh_DictMap_get__b(ctx, DP(o)->metaDictMap, STEXT("Interface"));
 		if(IS_NOTNULL(v)) {
-			flag |= KNH_FLAG_CF_INTERFACE;
+			flag |= FLAG_Class_Interface;
 		}
 		v = knh_DictMap_get__b(ctx, DP(o)->metaDictMap, STEXT("Singleton"));
 		if(IS_NOTNULL(v)) {
-			flag |= KNH_FLAG_CF_SINGLETON;
+			flag |= FLAG_Class_Singleton;
 		}
 		v = knh_DictMap_get__b(ctx, DP(o)->metaDictMap, STEXT("Release"));
 		if(IS_NOTNULL(v)) {
-			flag |= KNH_FLAG_CF_RELEASE;
+			flag |= FLAG_Class_Release;
 		}
 	}
 	return flag;
@@ -388,9 +372,9 @@ knh_flag_t knh_StmtCLASS_flag(Ctx *ctx, Stmt *o)
 /* [implements] */
 
 static
-Mapper* new_Mapper__interface(Ctx *ctx, knh_class_t scid, knh_class_t tcid)
+knh_Mapper_t* new_Mapper__interface(Ctx *ctx, knh_class_t scid, knh_class_t tcid)
 {
-	return new_Mapper(ctx, KNH_FLAG_MMF_INTERFACE, scid, tcid, knh_Mapper_fasis, KNH_NULL);
+	return new_Mapper(ctx, FLAG_Mapper_Interface, scid, tcid, knh_fmapper_asis, KNH_NULL);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -406,20 +390,18 @@ void knh_class_addInterface(Ctx *ctx, knh_class_t cid, knh_class_t icid)
 		knh_class_t isupcid = icid;
 		int allchecked = 1;
 		while(isupcid != CLASS_Object) {
-			ClassStruct *cs = ctx->share->ClassTable[isupcid].cstruct;
+			knh_ClassStruct_t *cs = ctx->share->ClassTable[isupcid].cstruct;
 			size_t i;
 			for(i = 0; i < knh_Array_size(cs->methods); i++) {
-				Method *imtd = (Method*)knh_Array_n(cs->methods, i);
+				knh_Method_t *imtd = (knh_Method_t*)knh_Array_n(cs->methods, i);
 				if(knh_Method_isPrivate(imtd)) {
 					continue;
 				}
 				else {
-					Method *mtd = knh_Class_getMethod(ctx, cid, DP(imtd)->mn);
+					knh_Method_t *mtd = knh_Class_getMethod(ctx, cid, DP(imtd)->mn);
 					if(IS_NOTNULL(mtd)) {
 						if(!knh_MethodField_equalsType(DP(mtd)->mf, DP(imtd)->mf)) {
-							char bufcm[CLASSNAME_BUFSIZ];
-							knh_format_cmethodn(ctx, bufcm, sizeof(bufcm), DP(imtd)->cid, DP(imtd)->mn);
-							DBG2_P("mismatch!! %s", bufcm);
+							DBG2_P("mismatch!! %s.%s", CLASSN(DP(imtd)->cid), METHODN(DP(imtd)->mn));
 							allchecked = 0;
 						}
 					}
@@ -430,15 +412,15 @@ void knh_class_addInterface(Ctx *ctx, knh_class_t cid, knh_class_t icid)
 		if(allchecked) {
 			isupcid = icid;
 			while(isupcid != CLASS_Object) {
-				ClassStruct *cs = ctx->share->ClassTable[isupcid].cstruct;
+				knh_ClassStruct_t *cs = ctx->share->ClassTable[isupcid].cstruct;
 				size_t i;
 				for(i = 0; i < knh_Array_size(cs->methods); i++) {
-					Method *imtd = (Method*)knh_Array_n(cs->methods, i);
+					knh_Method_t *imtd = (knh_Method_t*)knh_Array_n(cs->methods, i);
 					if(knh_Method_isPrivate(imtd)) {
 						continue;
 					}
 					else {
-						Method *mtd = knh_Class_getMethod(ctx, cid, DP(imtd)->mn);
+						knh_Method_t *mtd = knh_Class_getMethod(ctx, cid, DP(imtd)->mn);
 						if(IS_NULL(mtd)) {
 							mtd = new_Method(ctx, 0, cid, DP(imtd)->mn, NULL);
 							KNH_SETv(ctx, DP(mtd)->mf, DP(imtd)->mf);
@@ -458,17 +440,17 @@ void knh_class_addInterface(Ctx *ctx, knh_class_t cid, knh_class_t icid)
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_StmtCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
+int knh_StmtCLASS_decl(Ctx *ctx, knh_Stmt_t *stmt, knh_Asm_t *abr, knh_NameSpace_t *ns)
 {
 	char bufn[CLASSNAME_BUFSIZ];
 	knh_snprintf(bufn, sizeof(bufn), "%s.%s", knh_String_tochar(DP(ns)->nsname), sToken(StmtCLASS_class(stmt)));
 	knh_class_t cid  = knh_NameSpace_getcid(ctx, ns, B(bufn));
 	if(cid == CLASS_unknown) {
 		cid = knh_ClassTable_newId(ctx);
-		KNH_ASSERT(ClassTable(cid).class_cid == NULL);
+		KNH_ASSERT(ClassTable(cid).sname == NULL);
 	}
 	else {
-		knh_Asm_perror(ctx, abr, KERR_ERROR, _("cannot redefine %C"), cid);
+		knh_Asm_perror(ctx, abr, KERR_ERROR, _("re-definition of %C"), cid);
 		knh_Stmt_done(ctx, stmt);
 		return 0;
 	}
@@ -488,67 +470,76 @@ int knh_StmtCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 	}
 
 	DP(StmtCLASS_class(stmt))->cid = cid;
-
-	knh_ClassTable_t *TC = (knh_ClassTable_t*)&(ClassTable(cid));
-	TC->cflag  = knh_StmtCLASS_flag(ctx, stmt);
-	TC->oflag  = KNH_FLAG_CF2OF(TC->cflag);
-	if(SP(StmtCLASS_instmt(stmt))->stt == STT_DONE) {
-		TC->bcid   = CLASS_Any;    /* Glue */
-	}
-	else {
-		TC->bcid   = CLASS_Object;
-	}
-	TC->supcid = supcid;
-	TC->offset = 0; /* will be extended in CLASS_typing */
-	KNH_ASSERT(TC->class_cid == NULL);
-	knh_setClassName(ctx, cid, new_String(ctx, B(bufn), NULL));
-	KNH_INITv(TC->cstruct, new_ClassStruct0(ctx, 0, 8));
-	KNH_INITv(TC->cmap, ctx->share->ClassTable[CLASS_Any].cmap);
-	knh_setClassDefaultValue(ctx, cid, KNH_NULL, NULL);
-	knh_NameSpace_setLocalName(ctx, ns, cid);
-
-	if(SP(StmtCLASS_interface(stmt))->stt != STT_DONE) {
-		Stmt *istmt = StmtCLASS_interface(stmt);
-		int i, n = DP(istmt)->size;
-		for(i = 0; i < n; i++) {
-			Token *tk = DP(stmt)->tokens[i];
-			knh_class_t icid = knh_NameSpace_getcid(ctx, ns, knh_Token_tobytes(ctx, tk));
-			if(icid == CLASS_unknown) {
-				knh_Token_perror(ctx, tk, KERR_EWARN, _("unknown class: %s"), sToken(tk));
-				continue;
-			}
-			if(!knh_class_isInterface(icid)) {
-				knh_Token_perror(ctx, tk, KERR_EWARN, _("cannot implements %C: this class is not @Interface"), icid);
-				continue;
-			}
-			knh_class_addInterface(ctx, cid, icid);
+	{
+		knh_ClassTable_t *t = (knh_ClassTable_t*)&(ClassTable(cid));
+		t->cflag  = knh_StmtCLASS_flag(ctx, stmt);
+		t->oflag  = knh_flag_oflag(t->cflag);
+		if(SP(StmtCLASS_instmt(stmt))->stt == STT_DONE) {
+			t->bcid   = CLASS_Any;    /* Glue */
 		}
+		else {
+			t->bcid   = CLASS_Object;
+		}
+		t->supcid = supcid;
+		t->offset = 0; /* will be extended in CLASS_typing */
+		KNH_ASSERT_cid(supcid);
+		t->keyidx = ClassTable(supcid).keyidx;
+		KNH_ASSERT(t->sname == NULL);
+		knh_setClassName(ctx, cid, new_String(ctx, B(bufn), NULL));
+		KNH_INITv(t->cstruct, new_ClassStruct0(ctx, 0, 8));
+		KNH_INITv(t->cmap, ctx->share->ClassTable[CLASS_Any].cmap);
+		knh_setClassDefaultValue(ctx, cid, new_hObject(ctx, t->oflag | FLAG_Object_Undefined, t->bcid, cid), NULL);
+		if(t->bcid == CLASS_Any) {
+			knh_Glue_init(ctx, (knh_Glue_t*)t->cspec, NULL, NULL);
+		}
+		knh_NameSpace_setLocalName(ctx, ns, cid);
+		if(SP(StmtCLASS_interface(stmt))->stt != STT_DONE) {
+			knh_Stmt_t *istmt = StmtCLASS_interface(stmt);
+			int i, n = DP(istmt)->size;
+			for(i = 0; i < n; i++) {
+				knh_Token_t *tk = DP(stmt)->tokens[i];
+				knh_class_t icid = knh_NameSpace_getcid(ctx, ns, knh_Token_tobytes(ctx, tk));
+				if(icid == CLASS_unknown) {
+					knh_Token_perror(ctx, tk, KERR_EWARN, _("unknown class: %s"), sToken(tk));
+					continue;
+				}
+				if(!knh_class_isInterface(icid)) {
+					knh_Token_perror(ctx, tk, KERR_EWARN, _("cannot implements %C: this class is not @Interface"), icid);
+					continue;
+				}
+				knh_class_addInterface(ctx, cid, icid);
+			}
+		}
+		if(SP(StmtCLASS_instmt(stmt))->stt == STT_DONE) {
+			knh_Stmt_done(ctx, stmt);
+		}
+		KNH_NOTICE(ctx, "added new class: %s", CLASSN(cid));
 	}
-	if(SP(StmtCLASS_instmt(stmt))->stt == STT_DONE) {
-		knh_Stmt_done(ctx, stmt);
-	}
-	KNH_NOTICE(ctx, "added new class: %s", CLASSN(cid));
 	return 1;
 }
 
-
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_StmtIMPORT_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
+int knh_StmtIMPORT_decl(Ctx *ctx, knh_Stmt_t *stmt, knh_Asm_t *abr, knh_NameSpace_t *ns)
 {
 	int res = 1;
 	knh_bytes_t path = knh_String_tobytes(DP(StmtIMPORT_file(stmt))->text);
 	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
 	if(path.buf[0] != '/' && path.buf[0] != '\\' && !knh_bytes_startsWith(path, STEXT("http://"))) {
-		knh_cwb_write(ctx, cwb, knh_String_tobytes(knh_getResourceName(ctx, SP(stmt)->uri)));
-		knh_cwb_parentpath(ctx, cwb, NULL);
-		knh_cwb_putc(ctx, cwb, '/');
+		knh_bytes_t t = knh_String_tobytes(knh_getResourceName(ctx, SP(stmt)->uri));
+		if(t.buf[0] != '(') {  /* NOT (eval) */
+			knh_cwb_write(ctx, cwb, t);
+			knh_bool_t ret = knh_cwb_parentpath(ctx, cwb, NULL);
+			if (ret) {
+			  knh_cwb_putc(ctx, cwb, '/');
+			}
+		}
 	}
 	knh_cwb_write(ctx, cwb, path);
 	{
 		knh_sfp_t *lsfp = KNH_LOCAL(ctx);
-		InputStream *in = new_ScriptInputStream(ctx, path, cwb, ctx->share->mainns, 0);
+		knh_InputStream_t *in = new_ScriptInputStream(ctx, path, cwb, ctx->share->mainns, 0);
 		KNH_LPUSH(ctx, in);
 		if(!knh_InputStream_isClosed(ctx, in)) {
 			res = knh_NameSpace_load(ctx, ctx->share->mainns, in, 1/*isEval*/,0/*isThrowable*/);
@@ -565,9 +556,9 @@ int knh_StmtIMPORT_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_StmtUIMPORT_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
+int knh_StmtUCLASS_decl(Ctx *ctx, knh_Stmt_t *stmt, knh_Asm_t *abr, knh_NameSpace_t *ns)
 {
-	Token *tk = DP(stmt)->tokens[0];
+	knh_Token_t *tk = DP(stmt)->tokens[0];
 	knh_bytes_t name = knh_Token_tobytes(ctx, tk);
 	knh_index_t loc = knh_bytes_rindex(name, '.');
 	if(loc != -1 && isupper(name.buf[loc+1])) { /* using math.Math */
@@ -599,7 +590,7 @@ int knh_StmtUIMPORT_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_StmtUALIAS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
+int knh_StmtUALIAS_decl(Ctx *ctx, knh_Stmt_t *stmt, knh_Asm_t *abr, knh_NameSpace_t *ns)
 {
 	DBG2_DUMP(ctx, stmt, KNH_NULL, "decl");
 	TODO();
@@ -618,7 +609,7 @@ char *knh_format_classurn(Ctx *ctx, char *buf, size_t bufsiz, knh_class_t bcid,
 /* ------------------------------------------------------------------------ */
 
 static
-void knh_NameSpace_setTagName(Ctx *ctx, NameSpace *o, String *name, knh_class_t cid)
+void knh_NameSpace_setTagName(Ctx *ctx, knh_NameSpace_t *o, knh_String_t *name, knh_class_t cid)
 {
 	KNH_ASSERT(IS_NameSpace(o));
 	KNH_ASSERT_cid(cid);
@@ -636,10 +627,10 @@ void knh_NameSpace_setTagName(Ctx *ctx, NameSpace *o, String *name, knh_class_t
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_StmtXCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_class_t bcid)
+int knh_StmtXCLASS_decl(Ctx *ctx, knh_Stmt_t *stmt, knh_Asm_t *abr, knh_NameSpace_t *ns, knh_class_t bcid)
 {
-	Token *tkclassn = DP(stmt)->tokens[0];
-	Token *tkurn = DP(stmt)->tokens[1];
+	knh_Token_t *tkclassn = DP(stmt)->tokens[0];
+	knh_Token_t *tkurn = DP(stmt)->tokens[1];
 
 	char bufcu[CLASSNAME_BUFSIZ];
 	knh_format_classurn(ctx, bufcu, sizeof(bufcu), bcid, knh_getURNAlias(ctx, knh_Token_tobytes(ctx, tkurn)));
@@ -663,7 +654,7 @@ int knh_StmtXCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_class
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_StmtUVOCAB_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
+int knh_StmtUVOCAB_decl(Ctx *ctx, knh_Stmt_t *stmt, knh_Asm_t *abr, knh_NameSpace_t *ns)
 {
 	return knh_StmtXCLASS_decl(ctx, stmt, abr, ns, CLASS_String);
 }
@@ -671,7 +662,7 @@ int knh_StmtUVOCAB_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_StmtUENUM_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
+int knh_StmtUENUM_decl(Ctx *ctx, knh_Stmt_t *stmt, knh_Asm_t *abr, knh_NameSpace_t *ns)
 {
 	return knh_StmtXCLASS_decl(ctx, stmt, abr, ns, CLASS_Int);
 }
@@ -679,7 +670,7 @@ int knh_StmtUENUM_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_StmtUUNIT_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
+int knh_StmtUUNIT_decl(Ctx *ctx, knh_Stmt_t *stmt, knh_Asm_t *abr, knh_NameSpace_t *ns)
 {
 	return knh_StmtXCLASS_decl(ctx, stmt, abr, ns, CLASS_Float);
 }
@@ -687,9 +678,9 @@ int knh_StmtUUNIT_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_StmtUFUNC_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
+int knh_StmtUFUNC_decl(Ctx *ctx, knh_Stmt_t *stmt, knh_Asm_t *abr, knh_NameSpace_t *ns)
 {
-	Token *tk = DP(stmt)->tokens[0];
+	knh_Token_t *tk = DP(stmt)->tokens[0];
 	knh_bytes_t name = knh_Token_tobytes(ctx, tk);
 	if(SP(tk)->tt == TT_CMETHODN) {
 		knh_index_t loc = knh_bytes_rindex(name, '.');
@@ -713,10 +704,10 @@ int knh_StmtUFUNC_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 		}
 		else {
 			KNH_ASSERT_cid(cid);
-			Array *a = (ClassTable(cid).cstruct)->methods;
+			knh_Array_t *a = (ClassTable(cid).cstruct)->methods;
 			size_t i;
 			for(i = 0; i < knh_Array_size(a); i++) {
-				Method *mtd = (Method*)knh_Array_n(a, i);
+				knh_Method_t *mtd = (knh_Method_t*)knh_Array_n(a, i);
 				KNH_ASSERT(IS_Method(mtd));
 				if(!knh_Method_isStatic(mtd)) continue;
 				knh_NameSpace_setFuncClass(ctx, ns, DP(mtd)->mn, cid);
@@ -732,27 +723,26 @@ int knh_StmtUFUNC_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_StmtUMAPMAP_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
+int knh_StmtUMAPMAP_decl(Ctx *ctx, knh_Stmt_t *stmt, knh_Asm_t *abr, knh_NameSpace_t *ns)
 {
 	DBG2_DUMP(ctx, stmt, KNH_NULL, "decl");
-	TODO();
 	return 1;
 }
 
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_Stmt_eval(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int isEval)
+int knh_Stmt_eval(Ctx *ctx, knh_Stmt_t *stmt, knh_Asm_t *abr, knh_NameSpace_t *ns, int isEval)
 {
-	Script *scr = knh_getCurrentScript(ctx);
-	Method *mtd = knh_Class_getMethod(ctx, knh_Object_cid(scr), METHODN_lambda);
+	knh_Script_t *scr = knh_getAsmScript(ctx);
+	knh_Method_t *mtd = knh_Class_getMethod(ctx, knh_Object_cid(scr), METHODN_LAMBDA);
 	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
 
 	int isExpr = knh_stmt_isExpr(SP(stmt)->stt);
-
 	knh_methodn_t mt = METHODN__k;
+
 	if(SP(stmt)->stt == STT_MT) {
-		Token *tk0 = DP(stmt)->tokens[0];
+		knh_Token_t *tk0 = DP(stmt)->tokens[0];
 		knh_methodn_t mn = knh_getmn(ctx, knh_Token_tobytes(ctx, tk0), METHODN_NEWID);
 		char *name = FIELDN(METHODN_TOFIELDN(mn));
 		if(name[1] != 0) mt = METHODN__s;
@@ -760,7 +750,7 @@ int knh_Stmt_eval(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int isEval)
 
 	if(isExpr) {
 		if(SP(stmt)->stt == STT_CALL1) {
-			Token *tk0 = DP(stmt)->tokens[0];
+			knh_Token_t *tk0 = DP(stmt)->tokens[0];
 			if(SP(tk0)->tt == TT_STR) {
 				if(isEval) {
 					knh_write(ctx, KNH_STDOUT, knh_String_tobytes(DP(tk0)->text));
@@ -777,13 +767,17 @@ int knh_Stmt_eval(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int isEval)
 			isExpr = 0;
 		}
 		else {
-			Stmt *newstmt = new_Stmt(ctx, 0, STT_RETURN);
+			knh_Stmt_t *newstmt = new_Stmt(ctx, 0, STT_RETURN);
 			knh_Stmt_add(ctx, newstmt, UP(stmt));
 			stmt = newstmt;
 			KNH_LPUSH(ctx, stmt);
 		}
 	}
 
+	if(STT_(stmt) == STT_ERR) {
+		KNH_LOCALBACK(ctx, lsfp);
+		return 0;
+	}
 	KNH_ASM_METHOD(ctx, abr, mtd, NULL, stmt, 0 /* isIteration */);
 	if(knh_Method_isAbstract(mtd) || SP(stmt)->stt == STT_ERR) {
 		KNH_LOCALBACK(ctx, lsfp);
@@ -792,7 +786,7 @@ int knh_Stmt_eval(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int isEval)
 
 	int isVOID = knh_Stmt_isVOID(stmt);
 	if(isEval) {
-		ExceptionHandler *hdr = new_ExceptionHandler(ctx);
+		knh_ExceptionHandler_t *hdr = new_ExceptionHandler(ctx);
 		KNH_MOV(ctx, lsfp[0].o, hdr);
 		KNH_TRY(ctx, L_CATCH, lsfp, 0);
 		{
@@ -800,8 +794,8 @@ int knh_Stmt_eval(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int isEval)
 			KNH_MOV(ctx, lsfp[3].o, scr);
 			KNH_SCALL(ctx, lsfp, 2, mtd, 0/*args*/);
 			if(isExpr && !isVOID) {
-				//DBG2_P("returning %s %lld", CLASSNo(lsfp[2].o), lsfp[2].ivalue);
-				((Context*)ctx)->esp = lsfp+1;
+				DBG2_P("returning %p %s %lld %ld", lsfp+2, CLASSNo(lsfp[2].o), lsfp[2].ivalue, lsfp[2].bvalue);
+				((knh_Context_t*)ctx)->esp = lsfp+1;
 				knh_esp1_format(ctx, mt, KNH_STDOUT, KNH_NULL);
 				knh_write_EOL(ctx, KNH_STDOUT);
 			}
@@ -820,13 +814,13 @@ int knh_Stmt_eval(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int isEval)
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_NameSpace_compile(Ctx *ctx, NameSpace *ns, Stmt *stmt, int isEval)
+int knh_NameSpace_compile(Ctx *ctx, knh_NameSpace_t *ns, knh_Stmt_t *stmt, int isEval)
 {
-	Asm *abr = knh_Context_getAsm(ctx);
+	knh_Asm_t *abr = knh_Context_getAsm(ctx);
 	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
 	KNH_LPUSH(ctx, stmt);
 
-	Stmt *cur = stmt;
+	knh_Stmt_t *cur = stmt;
 	DP(abr)->uri = SP(stmt)->uri;
 	while(IS_Stmt(cur)) {
 		knh_stmt_t stt = SP(cur)->stt;
@@ -841,8 +835,8 @@ int knh_NameSpace_compile(Ctx *ctx, NameSpace *ns, Stmt *stmt, int isEval)
 			res = knh_StmtIMPORT_decl(ctx, cur, abr, ns);
 			knh_Stmt_done(ctx, cur);
 			break;
-		case STT_UIMPORT:
-			res = knh_StmtUIMPORT_decl(ctx, cur, abr, ns);
+		case STT_UCLASS:
+			res = knh_StmtUCLASS_decl(ctx, cur, abr, ns);
 			knh_Stmt_done(ctx, cur);
 			break;
 		case STT_UALIAS:
@@ -893,14 +887,29 @@ int knh_NameSpace_compile(Ctx *ctx, NameSpace *ns, Stmt *stmt, int isEval)
 //			tm = knh_StmtFORMAT_typing(ctx, cur, abr, ns);
 //			break;
 		case STT_DECL: {
-			knh_Asm_initThisScript(ctx, abr);
-			tm = knh_StmtDECL_typing(ctx, cur, abr, ns);
-			break;
+				knh_Asm_initThisScript(ctx, abr);
+				tm = knh_StmtDECL_typing(ctx, cur, abr, ns, DECL_SCRIPT);
+				break;
 			}
 		case STT_LET: {
-			knh_Asm_initThisScript(ctx, abr);
-			tm = knh_StmtLET_typing(ctx, cur, abr, ns, TYPE_void);
-			break;
+				knh_Asm_initThisScript(ctx, abr);
+				tm = knh_StmtLET_typing(ctx, cur, abr, ns, TYPE_void);
+				break;
+			}
+		case STT_LETM: {
+				knh_Asm_initThisScript(ctx, abr);
+				tm = knh_StmtLETM_typing(ctx, cur, abr, ns);
+				break;
+			}
+		case STT_SEPARATOR: {
+				knh_Asm_initThisScript(ctx, abr);
+				tm = knh_StmtSEPARATOR_typing(ctx, cur, abr, ns);
+				break;
+			}
+		case STT_FUNCTION: {
+				knh_Asm_initThisScript(ctx, abr);
+				tm = knh_StmtFUNCTION_typing(ctx, cur, abr, ns, TYPE_void);
+				break;
 			}
 		}
 		if(tm == NULL) {
@@ -950,27 +959,206 @@ int knh_NameSpace_compile(Ctx *ctx, NameSpace *ns, Stmt *stmt, int isEval)
 	return 0;
 }
 
-/* ------------------------------------------------------------------------ */
+/* ======================================================================== */
+/* [Data] */
 
-Object *knh_Stmt_toData(Ctx *ctx, Stmt *stmt, knh_class_t reqc)
+int knh_bytes_checkStmtLine(knh_bytes_t line)
 {
-//	Asm *abr = knh_Context_getAsm(ctx);
-//	DP(abr)->level = 0;
-//	NameSpace *ns = ctx->ns;
-//	if(IS_Stmt(stmt)) {
-//		KNH_ASM_SETLINE(ctx, abr, SP(stmt)->line);
-//		if(knh_stmt_isExpr(SP(stmt)->stt)) {
-//			Token *tk = knh_StmtEXPR_typing(ctx, stmt, abr, ns, reqc);
-//			if(tk == NULL || IS_Stmt(tk)) return KNH_NULL;
-//		}
-//		else {
-//			knh_Stmt_done(ctx, stmt);
-//		}
-//	}
-	TODO();
-	return KNH_NULL;
+	char *ln = (char*)line.buf;
+	size_t i = 0, len = line.len;
+	int ch, quote = 0, nest =0;
+	L_NORMAL:
+	for(; i < len; i++) {
+		ch = ln[i];
+		if(ch == '{' || ch == '[' || ch == '(') nest++;
+		if(ch == '}' || ch == ']' || ch == ')') nest--;
+		if(ch == '\'' || ch == '"' || ch == '`') {
+			quote = ch; i++;
+			goto L_QUOTE;
+		}
+	}
+	return nest;
+
+	L_QUOTE:
+	DBG2_ASSERT(i > 0);
+	for(; i < len; i++) {
+		ch = ln[i];
+		if(ln[i-1] != '\\' && ch == quote) {
+			i++;
+			goto L_NORMAL;
+		}
+	}
+	return 1;
 }
 
+/* ------------------------------------------------------------------------ */
+
+static
+Object *knh_InputStream_parseDataNULL(Ctx *ctx, knh_InputStream_t *in)
+{
+	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
+	Object *rVALUE = NULL;
+	knh_Stmt_t *stmt = knh_InputStream_parseStmt(ctx, in, 1/*isData*/);
+	KNH_LPUSH(ctx, in); //lsfp[0]
+	KNH_LPUSH(ctx, stmt); // lsfp[1]
+	if(knh_stmt_isExpr(STT_(stmt)) && STT_(stmt) != STT_LET) {
+		knh_Asm_t *abr = knh_Context_getAsm(ctx);
+		knh_Script_t *scr = knh_getAsmScript(ctx);
+		knh_Method_t *mtd = knh_Class_getMethod(ctx, knh_Object_cid(scr), METHODN_LAMBDA);
+		KNH_ASM_METHOD(ctx, abr, mtd, NULL, stmt, 0 /* isIteration */);
+		if(knh_Method_isAbstract(mtd) || SP(stmt)->stt == STT_ERR) {
+			goto L_RETURN;
+		}
+		DBG_P("rtype=%s%s", TYPEQN(DP(stmt)->type));
+		//knh_ExceptionHandler_t *hdr = new_ExceptionHandler(ctx);
+		//KNH_MOV(ctx, lsfp[0].o, hdr);
+		//KNH_TRY(ctx, L_CATCH, lsfp, 0);
+		KNH_MOV(ctx, lsfp[1].o, DP(mtd)->code); // TO AVOID RCGC
+		KNH_MOV(ctx, lsfp[2].o, scr);
+		KNH_SCALL(ctx, lsfp, 2, mtd, 0/*args*/);
+		rVALUE = lsfp[4].o;
+		//L_CATCH:;
+		//KNH_PRINT_STACKTRACE(ctx, lsfp, 0);
+	}
+	L_RETURN:
+	KNH_LOCALBACK(ctx, lsfp);
+	return rVALUE;
+}
+
+/* ------------------------------------------------------------------------ */
+
+Object* knh_InputStream_readData(Ctx *ctx, knh_InputStream_t *in)
+{
+	int ch, linenum = DP(in)->line;
+	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
+	L_READLINE:;
+	int quote = 0, prev = 0;
+	knh_cwb_subclear(cwb, 0);
+	while((ch = knh_InputStream_getc(ctx, in)) != EOF) {
+		if(ch == ';' && quote == 0) {
+			if(knh_cwb_size(cwb) > 0) {
+				goto L_STMT;
+			}
+		}
+		knh_Bytes_putc(ctx, cwb->ba, ch);
+		if(quote == ch && prev != '\\') {
+			quote = 0;
+			continue;
+		}
+		if(quote == 0) {
+			if(ch == '"' || ch == '\'' || ch == '`') {
+				quote = ch;
+			}
+			if(ch == '#') {
+				quote = '\n';
+			}
+		}
+	}
+	return KNH_NULL;
+	L_STMT:;
+	{
+		knh_InputStream_t *bin = new_BytesInputStream(ctx, cwb->ba, cwb->pos, knh_Bytes_size(cwb->ba));
+		Object *value = NULL;
+		DP(bin)->uri = DP(in)->uri;
+		DP(bin)->line = linenum;
+		knh_InputStream_setEncoding(ctx, bin, DP(in)->enc);
+		value = knh_InputStream_parseDataNULL(ctx, bin);
+		knh_cwb_close(cwb);
+		if(value == NULL) goto L_READLINE;
+		return value;
+	}
+}
+
+/* ------------------------------------------------------------------------ */
+/* [Extension] */
+
+static
+knh_Array_t* knh_DictMap_makeArray(Ctx *ctx, knh_DictMap_t *o, knh_bytes_t key)
+{
+	knh_index_t idx = knh_DictMap_firstIndex(o, key);
+	if(idx != -1) {
+		knh_Array_t *a = new_Array0(ctx, 0);
+		int i;
+		for(i = idx; i < knh_DictMap_size(o); i++) {
+			knh_String_t *k = knh_DictMap_keyAt(o, i);
+			Object *v = knh_DictMap_valueAt(o, i);
+			if(knh_String_startsWith(k, key) && IS_NOTNULL(v)) {
+				knh_Array_add(ctx, a, UP(k));
+				knh_Array_add(ctx, a, v);
+			}
+		}
+		return a;
+	}
+	return (knh_Array_t*)KNH_NULL;
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+void knh_invokeMethodListener(Ctx *ctx, knh_DictMap_t *meta, knh_Method_t *mtd, knh_bytes_t key1, knh_bytes_t key2)
+{
+	knh_Closure_t *cc;
+	KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
+	{
+		knh_DictMap_t *dm = DP(ctx->sys)->listenerDictMap;
+		cc = (knh_Closure_t*)knh_DictMap_get__b(ctx, dm, key1);
+	}
+	KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
+	if(IS_NOTNULL(cc)) {
+		knh_sfp_t *lsfp = KNH_LOCAL(ctx);
+		if(IS_NOTNULL(meta)) {
+			KNH_SETv(ctx, lsfp[2].o, meta);   /* cc(DictMap, Method) */
+			KNH_SETv(ctx, lsfp[3].o, mtd);
+			knh_Closure_invokesfp(ctx, cc, lsfp, /*argc*/2);
+		}
+		else {
+			knh_DictMap_t *dmeta = (knh_DictMap_t*)KNH_DEF(ctx, CLASS_DictMap);
+			KNH_SETv(ctx, lsfp[2].o, dmeta);
+			KNH_SETv(ctx, lsfp[3].o, mtd);
+			knh_Closure_invokesfp(ctx, cc, lsfp, /*argc*/2);
+			knh_DictMap_clear(ctx, dmeta);
+		}
+	}
+	if(IS_NOTNULL(meta)) {
+		int i;
+		knh_Array_t *a;
+		knh_sfp_t *lsfp = KNH_LOCAL(ctx);
+		KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
+		{
+			knh_DictMap_t *dm = DP(ctx->sys)->listenerDictMap;
+			a = knh_DictMap_makeArray(ctx, dm, key2);
+		}
+		KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
+		if(IS_NULL(a)) return;
+
+		KNH_LPUSH(ctx, a); // TO AVOID GC;
+		for(i = 0; i < knh_Array_size(a); i+=2) {
+			knh_String_t *key = (knh_String_t*)knh_Array_n(a, i);
+			knh_bytes_t anno = knh_bytes_rmod(knh_String_tobytes(key), '@');
+			if(IS_NOTNULL(knh_DictMap_get__b(ctx, meta, anno))) {
+				cc = (knh_Closure_t*)knh_Array_n(a, i+1);
+				KNH_SETv(ctx, lsfp[3].o, meta);   /* cc(DictMap, Method) */
+				KNH_SETv(ctx, lsfp[4].o, mtd);
+				knh_Closure_invokesfp(ctx, cc, lsfp+1, /*argc*/2);
+			}
+		}
+		KNH_LOCALBACK(ctx, lsfp);
+	}
+}
+
+/* ------------------------------------------------------------------------ */
+
+void knh_invokeMethodTypingListener(Ctx *ctx, knh_DictMap_t *meta, knh_Method_t *mtd)
+{
+	knh_invokeMethodListener(ctx, meta, mtd, STEXT("MethodT"), STEXT("MethodT@"));
+}
+
+/* ------------------------------------------------------------------------ */
+
+void knh_invokeMethodCompilationListener(Ctx *ctx, knh_DictMap_t *meta, knh_Method_t *mtd)
+{
+	knh_invokeMethodListener(ctx, meta, mtd, STEXT("MethodC"), STEXT("MethodC@"));
+}
 
 /* ------------------------------------------------------------------------ */
 
