@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -29,22 +29,22 @@
 
 
 #include"commons.h"
-#define KNH_USING_NOAPI 1
 
 #ifdef KNH_USING_POSIX
-#undef KNH_USING_NOAPI
-#include <unistd.h>
+#include<unistd.h>
 #include<dlfcn.h>
-
-#include<time.h>
-#include<sys/time.h>
 #endif
 
 #ifdef KNH_USING_WINDOWS
-#undef KNH_USING_NOAPI
 #include<windows.h>
 #endif
 
+#ifdef KNH_USING_BTRON
+#include<btron/file.h>
+#include<btron/dynload.h>
+#include<tstring.h>
+#endif
+
 
 /* ************************************************************************ */
 
@@ -55,18 +55,46 @@ extern "C" {
 /* ======================================================================== */
 /* [dlopen] */
 
-void *knh_dlopen(Ctx *ctx, const char* path, int mode)
+
+void *knh_cwb_dlopen(Ctx *ctx, knh_cwb_t *cwb)
 {
-	char buff[FILENAME_BUFSIZ];
-	knh_format_ospath(ctx, buff, sizeof(buff), B((char*)path));
-	DBG_P("dlopen .. '%s'\n", buff);
-#ifdef KNH_USING_WINDOWS
-	return (void*)LoadLibraryA((LPCTSTR)buff);
-#endif
-#ifdef KNH_USING_POSIX
-	return dlopen(buff, mode);
-#endif
-#ifdef KNH_USING_NOAPI
+	char *file;
+	if(!knh_bytes_endsWith(knh_cwb_tobytes(cwb), STEXT(KONOHA_OS_DLLEXT))) {
+		knh_Bytes_concatZero(ctx, cwb->ba);
+		knh_Bytes_write(ctx, cwb->ba, STEXT(KONOHA_OS_DLLEXT));
+	}
+	file = knh_cwb_ospath(ctx, cwb);
+#if defined(KNH_USING_WINDOWS)
+	DBG_P("knh_dlopen .. '%s'\n", file);
+	return (void*)LoadLibraryA((LPCTSTR)file);
+#elif defined(KNH_USING_POSIX)
+	DBG_P("knh_dlopen .. '%s'\n", file);
+	return dlopen(file, RTLD_LAZY);
+#elif defined(KNH_USING_BTRON)
+//	// YASHORO KUN ZENZEN WAKARIMASEN
+//        char buff[FILEPATH_BUFSIZ];
+//        TC buff_tc[FILEPATH_BUFSIZ];
+//        LINK lnk;
+//        W err;
+//	if(knh_bytes_endsWith(libname, STEXT(KONOHA_OS_DLLEXT))) {
+//		knh_format_ospath(ctx, buff, sizeof(buff), libname);
+//	}
+//	else {
+//		knh_format_ospath2(ctx, buff, sizeof(buff), libname, KONOHA_OS_DLLEXT);
+//	}
+//	DBG_P("knh_dlopen .. '%s'\n", buff);
+//        eucstotcs(buff_tc, buff);
+//        err = b_get_lnk(buff_tc, &lnk, F_NORM);
+//        if (err < 0) {
+//            return NULL;
+//        }
+//        err = b_dlopen(&lnk, DL_LAZY);
+//        if (err <= 0) {
+//            return NULL;
+//        }
+//        // Be careful that BTRON dlopen handles are of type W, not void*!
+//        return (void*)err;
+#else
 	return NULL;
 #endif
 }
@@ -75,14 +103,19 @@ void *knh_dlopen(Ctx *ctx, const char* path, int mode)
 
 void *knh_dlsym(Ctx *ctx, void* hdr, const char* symbol)
 {
-#ifdef KNH_USING_WINDOWS
-	return GetProcAddress((HMODULE)hdr, (LPCSTR)symbol);
-#endif
-#ifdef KNH_USING_POSIX
-	return dlsym(hdr, symbol);
-#endif
-#ifdef KNH_USING_NOAPI
-	return NULL;
+#if defined(KNH_USING_WINDOWS)
+    return GetProcAddress((HMODULE)hdr, (LPCSTR)symbol);
+#elif defined(KNH_USING_POSIX)
+    return dlsym(hdr, symbol);
+#elif defined(KNH_USING_BTRON)
+    UW val = NULL;
+    W err = b_dlsym((W)hdr, symbol, &val);
+    if (err < 0) {
+        return NULL;
+    }
+    return val;
+#else
+    return NULL;
 #endif
 }
 
@@ -90,14 +123,12 @@ void *knh_dlsym(Ctx *ctx, void* hdr, const char* symbol)
 
 const char *knh_dlerror(Ctx *ctx)
 {
-#ifdef KNH_USING_WINDOWS
-	return "TODO: dlerror()";
-#endif
-#ifdef KNH_USING_POSIX
-	return dlerror();
-#endif
-#ifdef KNH_USING_NOAPI
-	return "Unsupported dlopen APIs";
+#if defined(KNH_USING_WINDOWS)
+    return "TODO: dlerror()";
+#elif defined(KNH_USING_POSIX)
+    return dlerror();
+#else
+    return "Unsupported dlopen APIs";
 #endif
 }
 
@@ -105,14 +136,14 @@ const char *knh_dlerror(Ctx *ctx)
 
 int knh_dlclose(Ctx *ctx, void* hdr)
 {
-#ifdef KNH_USING_WINDOWS
-	return (int)FreeLibrary((HMODULE)hdr);
-#endif
-#ifdef KNH_USING_POSIX
-	return dlclose(hdr);
-#endif
-#ifdef KNH_USING_NOAPI
-	return 1;
+#if defined(KNH_USING_WINDOWS)
+    return (int)FreeLibrary((HMODULE)hdr);
+#elif defined(KNH_USING_POSIX)
+    return dlclose(hdr);
+#elif defined(KNH_USING_BTRON)
+    return b_dlclose((W)hdr);
+#else
+    return 0;
 #endif
 }
 
