@@ -190,7 +190,7 @@ static
 knh_bool_t knh_cwb_mkdir(Ctx *ctx, knh_cwb_t *cwb, char *subpath, int isThrowable)
 {
 	char *pathname;
-	if(!knh_cwb_isdir(ctx, cwb)) {
+	if(knh_cwb_isdir(ctx, cwb)) {
 		char subbuf[SUBPATH_BUFSIZ];
 		if(knh_cwb_parentpath(ctx, cwb, subbuf)) {
 			if(knh_cwb_mkdir(ctx, cwb, subpath, isThrowable) == 0) {
@@ -285,6 +285,44 @@ knh_bool_t knh_rename(Ctx *ctx, knh_bytes_t on, knh_bytes_t nn, int isThrowable)
 	return res;
 }
 
+//knh_bool_t knh_rename(Ctx *ctx, knh_bytes_t on, knh_bytes_t nn, int isThrowable)
+//{
+//    knh_cwb_t cwbbuf1, *cwb1;
+//    knh_cwb_t cwbbuf2, *cwb2;
+//    knh_bytes_t path1 , path2;
+//    char pathname1[FILEPATH_BUFSIZ] = {0};
+//    char pathname2[FILEPATH_BUFSIZ] = {0};
+//
+//    cwb1 = knh_cwb_openinit(ctx, &cwbbuf1, on);
+//    knh_cwb_ospath(ctx, cwb1);
+//    path1 = knh_cwb_tobytes(cwb1);
+//    knh_memcpy(pathname1, path1.buf, path1.len);
+//
+//    cwb2 = knh_cwb_openinit(ctx, &cwbbuf2, nn);
+//    knh_cwb_ospath(ctx, cwb2);
+//    path2 = knh_cwb_tobytes(cwb2);
+//    knh_memcpy(pathname2, path2.buf, path2.len);
+//    int res = 1;
+//
+//#if defined(KNH_USING_POSIX)
+//    if(rename(pathname1, pathname2) == -1) {
+//        KNH_PERRNO(ctx, cwb1, "OS!!", "rename", isThrowable);
+//        res = 0;
+//    }
+//#elif defined(KNH_USING_WINDOWS)
+//    if(MoveFileA(pathname1, pathname2) == 0) {
+//        KNH_PERRNO(ctx, cwb1, "OS!!", "MoveFile", isThrowable);
+//        res = 0;
+//    }
+//#else
+//    KNH_NOAPI(ctx, cwb1, isThrowable);
+//#endif
+//    knh_cwb_close(cwb2);
+//    knh_cwb_close(cwb1);
+//    return res;
+//}
+
+
 /* ======================================================================== */
 /* [homepath] */
 
@@ -293,12 +331,12 @@ knh_bool_t knh_rename(Ctx *ctx, knh_bytes_t on, knh_bytes_t nn, int isThrowable)
 // $konoha.package.path {$konoha.path}/package
 // $konoha.script.path  {$konoha.path}/script
 
-void knh_System_initPath(Ctx *ctx, System *o)
+void knh_System_initPath(Ctx *ctx, knh_System_t *o)
 {
 	knh_System_struct *sys = DP(o);
 	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
 	char *homepath = knh_getenv("KONOHA_HOME");
-	String *shome;
+	knh_String_t *shome;
 	knh_bytes_t home = {NULL, 0};
 #if defined(KNH_PREFIX)
 	if(homepath == NULL) {
@@ -307,7 +345,7 @@ void knh_System_initPath(Ctx *ctx, System *o)
 	}
 #endif
 	if(homepath != NULL) {
-		knh_DictMap_set(ctx, sys->props, T("konoha.path"), UP(T(homepath)));
+		knh_DictMap_set(ctx, sys->props, T__("konoha.path"), UP(T__(homepath)));
 		home = B(homepath);
 	}
 #if defined(KNH_USING_WINDOWS)
@@ -318,15 +356,15 @@ void knh_System_initPath(Ctx *ctx, System *o)
 		GetModuleFileNameA(h, buf, bufsiz);
 		knh_cwb_write(ctx, cwb, B(buf));
 		knh_DictMap_set(ctx, sys->props,
-			T("konoha.bin.path"), UP(new_String__cwb(ctx, cwb)));
+			T__("konoha.bin.path"), UP(knh_cwb_newString(ctx, cwb)));
 		if(homepath == NULL) {
 			GetModuleFileNameA(h, buf, bufsiz);
 			knh_cwb_write(ctx, cwb, B(buf));
 			knh_cwb_parentpath(ctx, cwb, NULL);
 			knh_cwb_parentpath(ctx, cwb, NULL);
-			shome = new_String__cwb(ctx, cwb);
+			shome = knh_cwb_newString(ctx, cwb);
 			home = knh_String_tobytes(shome);
-			knh_DictMap_set(ctx, sys->props, T("konoha.path"), UP(shome));
+			knh_DictMap_set(ctx, sys->props, T__("konoha.path"), UP(shome));
 		}
 	}
 #elif defined(KONOHA_ON_LINUX)
@@ -338,15 +376,15 @@ void knh_System_initPath(Ctx *ctx, System *o)
 		readlink("/proc/self/exe", buf, bufsiz);
 		knh_cwb_write(ctx, cwb, B(buf));
 		knh_DictMap_set(ctx, sys->props,
-			T("konoha.bin.path"), UP(new_String__cwb(ctx, cwb)));
+			T__("konoha.bin.path"), UP(knh_cwb_newString(ctx, cwb)));
 		if(homepath == NULL) {
 			readlink("/proc/self/exe", buf, bufsiz);
 			knh_cwb_write(ctx, cwb, B(buf));
 			knh_cwb_parentpath(ctx, cwb, NULL);
 			knh_cwb_parentpath(ctx, cwb, NULL);
-			shome = new_String__cwb(ctx, cwb);
+			shome = knh_cwb_newString(ctx, cwb);
 			home = knh_String_tobytes(shome);
-			knh_DictMap_set(ctx, sys->props, T("konoha.path"), UP(shome));
+			knh_DictMap_set(ctx, sys->props, T__("konoha.path"), UP(shome));
 		}
 	}
 #elif defined(KONOHA_ON_MACOSX)
@@ -356,33 +394,33 @@ void knh_System_initPath(Ctx *ctx, System *o)
 		s = realpath(s, buf);
 		knh_cwb_write(ctx, cwb, B(buf));
 		knh_DictMap_set(ctx, sys->props,
-			T("konoha.bin.path"), UP(new_String__cwb(ctx, cwb)));
+			T__("konoha.bin.path"), UP(knh_cwb_newString(ctx, cwb)));
 		if(homepath == NULL) {
 			knh_cwb_write(ctx, cwb, B(s));
 			knh_cwb_parentpath(ctx, cwb, NULL);
 			knh_cwb_parentpath(ctx, cwb, NULL);
-			shome = new_String__cwb(ctx, cwb);
+			shome = knh_cwb_newString(ctx, cwb);
 			home = knh_String_tobytes(shome);
-			knh_DictMap_set(ctx, sys->props, T("konoha.path"), UP(shome));
+			knh_DictMap_set(ctx, sys->props, T__("konoha.path"), UP(shome));
 		}
 		//free(s);
 	}
 #else
 	home = STEXT("/konoha");
-	knh_DictMap_set(ctx, sys->props, T("konoha.path"), UP(T("/konoha")));
+	knh_DictMap_set(ctx, sys->props, T__("konoha.path"), UP(T__("/konoha")));
 #endif
 	KNH_ASSERT(home. buf != NULL);
 	/* $konoha.package.path {$konoha.path}/package */
 	knh_cwb_subclear(cwb, 0);
 	knh_cwb_write(ctx, cwb, home);
-	knh_cwb_write(ctx, cwb, STEXT("/package"));
+	knh_cwb_write(ctx, cwb, STEXT("/package-" LIBKONOHA_VERSION));
 	knh_DictMap_set(ctx, sys->props,
-		T("konoha.package.path"), UP(new_String__cwb(ctx, cwb)));
+		T__("konoha.package.path"), UP(knh_cwb_newString(ctx, cwb)));
 
 	/* $konoha.script.path {$konoha.path}/script */
 	knh_cwb_write(ctx, cwb, home);
-	knh_cwb_write(ctx, cwb, STEXT("/script"));
-	knh_DictMap_set(ctx, sys->props, T("konoha.script.path"), UP(new_String__cwb(ctx, cwb)));
+	knh_cwb_write(ctx, cwb, STEXT("/script-" LIBKONOHA_VERSION));
+	knh_DictMap_set(ctx, sys->props, T__("konoha.script.path"), UP(knh_cwb_newString(ctx, cwb)));
 
 	homepath = knh_getenv("HOME");
 	if(homepath != NULL) {
@@ -390,23 +428,23 @@ void knh_System_initPath(Ctx *ctx, System *o)
 		knh_cwb_write(ctx, cwb, B(homepath));
 		knh_cwb_putc(ctx, cwb, '/');
 		knh_cwb_write(ctx, cwb, STEXT(KONOHA_FOLDER));
-		shome = new_String__cwb(ctx, cwb);
+		shome = knh_cwb_newString(ctx, cwb);
 		home = knh_String_tobytes(shome);
-		knh_DictMap_set(ctx, sys->props, T("user.path"), UP(shome));
+		knh_DictMap_set(ctx, sys->props, T__("user.path"), UP(shome));
 		/* $konoha.temp.path ${user.path}/temp */
 		knh_cwb_write(ctx, cwb, home);
 		knh_cwb_write(ctx, cwb, STEXT("/temp"));
-		knh_DictMap_set(ctx, sys->props, T("konoha.temp.path"), UP(new_String__cwb(ctx, cwb)));
+		knh_DictMap_set(ctx, sys->props, T__("konoha.temp.path"), UP(knh_cwb_newString(ctx, cwb)));
 
 		/* $user.package.path ${user.path}/package */
 		knh_cwb_write(ctx, cwb, home);
-		knh_cwb_write(ctx, cwb, STEXT("/package"));
-		knh_DictMap_set(ctx, sys->props, T("user.package.path"), UP(new_String__cwb(ctx, cwb)));
+		knh_cwb_write(ctx, cwb, STEXT("/package-" LIBKONOHA_VERSION));
+		knh_DictMap_set(ctx, sys->props, T__("user.package.path"), UP(knh_cwb_newString(ctx, cwb)));
 
 		/* $user.script.path ${user.path}/script */
 		knh_cwb_write(ctx, cwb, home);
-		knh_cwb_write(ctx, cwb, STEXT("/script"));
-		knh_DictMap_set(ctx, sys->props, T("user.script.path"), UP(new_String__cwb(ctx, cwb)));
+		knh_cwb_write(ctx, cwb, STEXT("/script-" LIBKONOHA_VERSION));
+		knh_DictMap_set(ctx, sys->props, T__("user.script.path"), UP(knh_cwb_newString(ctx, cwb)));
 	}
 	else {
 #if defined(KONOHA_ON_WINDOWS)
@@ -414,7 +452,7 @@ void knh_System_initPath(Ctx *ctx, System *o)
 #else
 		knh_cwb_write(ctx, cwb, STEXT("/tmp"));
 #endif
-		knh_DictMap_set(ctx, sys->props, T("konoha.temp.path"), UP(new_String__cwb(ctx, cwb)));
+		knh_DictMap_set(ctx, sys->props, T__("konoha.temp.path"), UP(knh_cwb_newString(ctx, cwb)));
 	}
 }
 
