@@ -33,6 +33,11 @@
 #include<sqlite3.h>
 #endif
 
+#ifdef KNH_USING_MYSQL
+#include<mysql/mysql.h>
+#endif
+
+
 /* ************************************************************************ */
 
 #ifdef __cplusplus
@@ -51,7 +56,7 @@ knh_db_t *knh_dbopen__NOP(Ctx *ctx, knh_bytes_t url)
 /* ------------------------------------------------------------------------ */
 
 static
-knh_dbcur_t *knh_dbquery__NOP(Ctx *ctx, knh_db_t *hdr, knh_bytes_t sql, ResultSet *b)
+knh_dbcur_t *knh_dbquery__NOP(Ctx *ctx, knh_db_t *hdr, knh_bytes_t sql, knh_ResultSet_t *b)
 {
 	return NULL;
 }
@@ -94,7 +99,7 @@ static knh_db_drvapi_t DB__NOP = {
 /* ======================================================================== */
 /* [sqlite3] */
 
-void knh_write_USING_SQLITE3(Ctx *ctx, OutputStream *w)
+void knh_write_USING_SQLITE3(Ctx *ctx, knh_OutputStream_t *w)
 {
 #ifdef KNH_USING_SQLITE3
 	knh_write_char(ctx, w, " sqlite3");
@@ -177,7 +182,7 @@ int knh_dbcurnext__sqlite3(Ctx *ctx, knh_dbcur_t *dbcur, struct knh_ResultSet_t
 /* ------------------------------------------------------------------------ */
 
 static
-knh_dbcur_t *knh_dbquery__sqlite3(Ctx *ctx, knh_db_t *hdr, knh_bytes_t sql, ResultSet *rs)
+knh_dbcur_t *knh_dbquery__sqlite3(Ctx *ctx, knh_db_t *hdr, knh_bytes_t sql, knh_ResultSet_t *rs)
 {
 	if(rs == NULL) {
 		int r = sqlite3_exec((sqlite3*)hdr, (const char*)sql.buf, NULL, NULL, NULL);
@@ -248,6 +253,157 @@ static knh_db_drvapi_t DB__sqlite3 = {
 
 #endif/*KNH_USING_SQLITE3*/
 
+/* ======================================================================== */
+/* [mysql] */
+
+void knh_write_USING_MYSQL(Ctx *ctx, knh_OutputStream_t *w)
+{
+#ifdef KNH_USING_MYSQL
+	knh_write_char(ctx, w, " mysql");
+#endif
+}
+
+#ifdef KNH_USING_MYSQL
+
+static
+void knh_mysql_perror(Ctx *ctx, MYSQL *db, int r)
+{
+	char *msg = "SQL!!", buf[512];
+//	if(r == SQLITE_PERM || r == SQLITE_AUTH) {
+//		msg = "Security!!";
+//	}
+	knh_snprintf(buf, sizeof(buf), "%s: %s", msg, mysql_errmsg(db));
+	KNH_WARNING(ctx, buf);
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+knh_db_t *knh_dbopen__mysql(Ctx *ctx, knh_bytes_t url)
+{
+    MYSQL *db = NULL;
+    char* server = "localhost";
+    char* user   = "user";
+    char* passwd = "passwd";
+    char* dbname = "dbname";
+    int port = 1024;
+
+    url = knh_bytes_skipscheme(url);
+    db  = mysql_init(NULL);
+    if(mysql_real_connect(db,  server, user, passwd, dbname, port, NULL, 0)==NULL) {
+        KNH_THROWs(ctx, "SQL!!: connection error");
+    }
+    return (knh_db_t*)db;
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+int knh_dbcurnext__mysql(Ctx *ctx, knh_dbcur_t *dbcur, struct knh_ResultSet_t *rs)
+{
+    MYSQL_ROW row;
+    int i = 0;
+    if((row = mysql_fetch_row((MYSQL_RES*) dbcur)) != NULL) {
+        knh_ResultSet_initData(ctx, rs); /* DO NOT TOUCH */
+        for(i = 0; i < rs->column_size; i++) {
+            if(row[i] == NULL) {
+                knh_ResultSet_setNull(ctx, rs, i);
+                continue;
+            }
+            switch(rs->column[i].dbtype) {
+                case MYSQL_TYPE_TINY:
+                case MYSQL_TYPE_SHORT:
+                case MYSQL_TYPE_LONG:
+                case MYSQL_TYPE_INT24:
+                case MYSQL_TYPE_BIT:
+                    knh_ResultSet_setInt(ctx, rs, i, knh_bytes_toint(B(row[i])));
+                    break;
+                case MYSQL_TYPE_DECIMAL:
+                case MYSQL_TYPE_FLOAT:
+                case MYSQL_TYPE_DOUBLE:
+                    knh_ResultSet_setFloat(ctx, rs, i, knh_bytes_tofloat(B(row[i])));
+                    break;
+                case MYSQL_TYPE_LONGLONG:
+                    knh_ResultSet_setInt(ctx, rs, i, knh_bytes_toint64(B(row[i])));
+                    break;
+                default:
+                    knh_ResultSet_setBytes(ctx, rs, i, B(row[i]));
+                    break;
+            }
+        }
+        return 1; /* if you have a tuple */
+    }
+    return 0;  /* NOMORE */
+}
+
+
+/* ------------------------------------------------------------------------ */
+
+static
+knh_dbcur_t *knh_dbquery__mysql(Ctx *ctx, knh_db_t *hdr, knh_bytes_t sql, knh_ResultSet_t *rs)
+{
+    if(rs == NULL) {
+        // rs is NULL
+        knh_mysql_perror(ctx, (MYSQL*)hdr, r);
+    }
+    else {
+        MYSQL_RES *res = NULL;
+        MYSQL_FIELD *field = NULL;
+        int i = 0;
+        int r = mysql_query((MYSQL*)hdr, (const char*)sql.buf);
+        if(r > 0) {
+            // query error
+            knh_mysql_perror(ctx, (MYSQL*)hdr, r);
+        }
+        res = mysql_store_result((MYSQL*)hdr);
+        if(res == NULL) {
+            if(mysql_error((MYSQL*)hdr)) {
+                KNH_THROWs(ctx, "result is NULL");
+            } else {
+                KNH_THROWs(ctx, "unknown error");
+            }
+        }
+        while((field = mysql_fetch_field(res)) != NULL) {
+            rs->column[i].dbtype = field->type;
+            i++;
+        }
+        return (knh_dbcur_t *) res;
+    }
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+void knh_dbclose__mysql(Ctx *ctx, knh_db_t *hdr)
+{
+    mysql_close((MYSQL*)hdr);
+}
+
+
+/* ------------------------------------------------------------------------ */
+
+static
+void knh_dbcurfree__mysql(knh_dbcur_t *dbcur)
+{
+	MYSQL_STMT *stmt = (MYSQL_STMT*)dbcur;
+	mysql_stmt_close(stmt);
+}
+
+/* ------------------------------------------------------------------------ */
+/* @data */
+
+static knh_db_drvapi_t DB__mysql = {
+	KNH_DRVAPI_TYPE__DB, "mysql",
+	knh_dbopen__mysql,
+	knh_dbquery__mysql,
+	knh_dbclose__mysql,
+	knh_dbcurnext__mysql,
+	knh_dbcurfree__mysql
+};
+
+#endif/*KNH_USING_MYSQL*/
+
+
 /* ======================================================================== */
 /* [drivers] */
 
@@ -260,17 +416,21 @@ knh_db_drvapi_t *knh_System_getDefaultDBDriver(void)
 
 knh_db_drvapi_t *knh_System_getDBDriver(Ctx *ctx, knh_bytes_t name)
 {
-	if(ctx == NULL) {
-		return &DB__NOP;
-	}
-	else {
-		knh_db_drvapi_t *p = (knh_db_drvapi_t *)knh_getDriverAPI(ctx, KNH_DRVAPI_TYPE__DB, name);
-		if(p == NULL) {
-			KNH_WARNING(ctx, "DB: unsupported scheme '%s'", name);
-			p = &DB__NOP;
-		}
-		return p;
-	}
+    if(ctx == NULL) {
+        return &DB__NOP;
+    }
+    else {
+        knh_db_drvapi_t *p = (knh_db_drvapi_t *)knh_getDriverAPI(ctx, KNH_DRVAPI_TYPE__DB, name);
+        if(p == NULL) {
+#ifdef KNHX_SQLITE3_EXCEPTION
+            KNH_THROWf(ctx, "DB!!: unsupported scheme '%s'", name);
+#else
+            KNH_WARNING(ctx, "DB: unsupported scheme '%s'", name);
+#endif
+            p = &DB__NOP;
+        }
+        return p;
+    }
 }
 
 /* ------------------------------------------------------------------------ */
@@ -283,13 +443,17 @@ KNHAPI(void) knh_addDBDriver(Ctx *ctx, char *alias, knh_db_drvapi_t *d)
 /* ======================================================================== */
 /* [init] */
 
-void KNHINIT init_DB(Ctx *ctx)
+void knh_Connection_newClass(Ctx *ctx, knh_class_t cid)
 {
 	knh_addDBDriver(ctx, NULL, &DB__NOP);
 #ifdef KNH_USING_SQLITE3
 	knh_addDBDriver(ctx, NULL, &DB__sqlite3);
 	knh_addDBDriver(ctx, "sqlite", &DB__sqlite3);
 #endif
+
+#ifdef KNH_USING_MYSQL
+	knh_addDBDriver(ctx, "mysql", &DB__mysql);
+#endif
 }
 
 /* ------------------------------------------------------------------------ */
