@@ -40,10 +40,6 @@
 #endif
 
 
-#ifdef KONOHA_ON_WINDOWS
-#include <process.h>
-#endif
-
 /* ************************************************************************ */
 
 //機能 	pthread関数名 	win32 スレッドapi名
@@ -72,8 +68,6 @@ knh_thread_t knh_thread_self(void)
 	return (knh_thread_t)pthread_self();
 #elif defined(KNH_USING_BTRON)
         return b_get_tid();
-#elif defined (KONOHA_ON_WINDOWS)
-	return (knh_thread_t)GetCurrentThread();
 #else
 	return 0;
 #endif
@@ -102,12 +96,10 @@ static void knh_thread_btronEntryPoint(knh_thread_target_btron* arg)
 #endif /* KNH_USING_BTRON */
 
 
-/* ------------------------------------------------------------------------ */
-
-int knh_thread_create(Ctx *ctx, knh_thread_t *thread, void *attr, threadfunc_t func, void * arg)
+int knh_thread_create(Ctx *ctx, knh_thread_t *thread, void *attr, void *(*frun)(void *), void * arg)
 {
 #if defined(KNH_USING_PTHREAD)
-	return pthread_create((pthread_t*)thread, attr, func, arg);
+	return pthread_create((pthread_t*)thread, attr, frun, arg);
 #elif defined(KNH_USING_BTRON)
         // FIXME: attr is ignored
         W err;
@@ -116,7 +108,7 @@ int knh_thread_create(Ctx *ctx, knh_thread_t *thread, void *attr, threadfunc_t f
         if (target == NULL) {
             return -1;
         }
-        target->func = func;
+        target->func = frun;
         target->arg = arg;
         err = b_cre_tsk((FP)knh_thread_btronEntryPoint, -1, (W)target);
         if (err < 0) {
@@ -125,9 +117,6 @@ int knh_thread_create(Ctx *ctx, knh_thread_t *thread, void *attr, threadfunc_t f
         }
         *thread = err;
         return 0;
-#elif defined(KONOHA_ON_WINDOWS)
-	//	return _beginthreadex(NULL, 0, (unsigned *(*)(void))func, arg, 0, thread);
-	return _beginthread((void( *)( void * ))func, 1024, arg);
 #else
 	return -1;
 #endif
@@ -146,10 +135,11 @@ int knh_thread_detach(Ctx *ctx, knh_thread_t th)
 
 /* ------------------------------------------------------------------------ */
 
-int knh_thread_join(Ctx *ctx, knh_thread_t thread, void **ret)
+int knh_thread_join(Ctx *ctx, knh_thread_t *thread, void **ret)
 {
 #if defined(KNH_USING_PTHREAD)
-    return pthread_join((pthread_t)thread, ret);
+    // int pthread_join(pthread_t th, void **thread_return);
+    return pthread_join((pthread_t*)thread, ret);
 #else
     return -1;
 #endif
@@ -206,7 +196,7 @@ void knh_stack_threadRun(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_thread_t th;
 	knh_threadcc_t ta = {ctx, sfp};
-	knh_thread_create(ctx, &th, NULL, (threadfunc_t)threading, (void*)&ta);
+	knh_thread_create(ctx, &th, NULL, threading, (void*)&ta);
 	knh_thread_detach(ctx, th);
 }
 
