@@ -21,19 +21,18 @@
    Engineering a sort function; Jon Bentley and M. Douglas McIlroy;
    Software - Practice and Experience; Vol. 23 (11), 1249-1265, 1993.  */
 
-#include <konoha.h>
-#if defined(KONOHA_ON_LKM)
-#ifdef __linux__
+#include<konoha.h>   // added to integrate konoha
+
+#ifdef KONOHA_ON_LKM
 #include <linux/string.h>
-#elif defined(__APPLE__)
-#include <string.h>
-#endif
 #else
 #include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 #endif
 
+#ifdef K_USING_QSORT
+
 /* Byte-wise swap two items of size SIZE. */
 #define SWAP(a, b, size)						      \
     do {									      \
@@ -258,8 +257,8 @@ jump_over:;
     }
 }
 
-void knh_qsort_r (void *const pbase, size_t total_elems, size_t size,
-        int (*cmp)(void* ,const void*,const void*),void* thunk)
+void knh_qsort_r (void *const pbase, size_t total_elems, size_t size, 
+        void* thunk,int (*cmp)(void* ,const void*,const void*))
 {
     register char *base_ptr = (char *) pbase;
 
@@ -419,3 +418,6 @@ jump_over:;
         }
     }
 }
+
+#endif /* K_USING_QSORT */
+
