@@ -62,7 +62,7 @@ void *knh_malloc(Ctx *ctx, size_t size)
 	if (unlikely(block == NULL)) {
 		DBG2_ABORT();
 		KNH_EXIT("OutOfMemory!!: %d bytes used", (int)ctx->stat->usedMemorySize);
-		//KNH_THROW__T(ctx, "OutOfMemory!!");
+		//KNH_THROWs(ctx, "OutOfMemory!!");
 	}
 	knh_stat_incUsedMemorySize(ctx, size);
 	return block;
@@ -87,7 +87,7 @@ void *DBG2_malloc(Ctx *ctx, size_t size, char *func)
 	if (unlikely(block == NULL)) {
 		DBG2_ABORT();
 		KNH_EXIT("OutOfMemory!!: %d bytes used", (int)ctx->stat->usedMemorySize);
-		//KNH_THROW__T(ctx, "OutOfMemory!!");
+		//KNH_THROWs(ctx, "OutOfMemory!!");
 	}
 	knh_stat_incUsedMemorySize(ctx, size);
 	block[0] = size;
@@ -119,7 +119,6 @@ void DBG2_free(Ctx *ctx, void *p, size_t size, char *func)
 		((knh_Context_t*)ctx)->unusedObjectSize += 1;\
 	}\
 
-#define SIZEOF_OBJECTTABLE(ctx) (ctx->share->ObjectPageTableMaxSize * sizeof(knh_ObjectPageTable_t))
 
 /* ------------------------------------------------------------------------ */
 
@@ -133,8 +132,8 @@ knh_Object_t *new_UnusedObject(Ctx *ctx)
 		knh_ObjectPageTable_t *newpage
 			= (knh_ObjectPageTable_t*)KNH_MALLOC(ctx, sizeof(knh_ObjectPageTable_t) * newsize);
 		knh_bzero(newpage, sizeof(knh_ObjectPageTable_t) * newsize);
-		knh_memcpy(newpage, ctx->share->ObjectPageTable, SIZEOF_OBJECTTABLE(ctx));
-		KNH_FREE(ctx, ctx->share->ObjectPageTable, SIZEOF_OBJECTTABLE(ctx));
+		knh_memcpy(newpage, ctx->share->ObjectPageTable, ctx->share->ObjectPageTableMaxSize);
+		KNH_FREE(ctx, ctx->share->ObjectPageTable, ctx->share->ObjectPageTableMaxSize);
 		((knh_SharedData_t*)ctx->share)->ObjectPageTable = newpage;
 		((knh_SharedData_t*)ctx->share)->ObjectPageTableMaxSize = newsize;
 	}
@@ -203,7 +202,7 @@ static void knh_unsetFastMallocMemory(void *p)
 
 /* ------------------------------------------------------------------------ */
 
-static inline int knh_isFastMallocMemory(void *p)
+int knh_isFastMallocMemory(void *p)
 {
 	knh_uintptr_t *b = (knh_uintptr_t*)((((knh_uintptr_t)p) / KONOHA_PAGESIZE) * KONOHA_PAGESIZE);
 	int n = ((Object*)p - (Object*)b); /*((knh_uintptr_t)p % KONOHA_PAGESIZE) / sizeof(knh_Object_t); */
@@ -213,19 +212,14 @@ static inline int knh_isFastMallocMemory(void *p)
 }
 
 /* ------------------------------------------------------------------------ */
-#define CHECK_UNUSED_OBJECT(ctx) { \
-if(unlikely(ctx->unusedObject == NULL)) { \
-		KNH_ASSERT(ctx->unusedObjectSize == 0); \
-		((knh_Context_t*)ctx)->unusedObject = new_UnusedObject(ctx); \
-	} \
-} \
-/* ------------------------------------------------------------------------ */
-
 
 void *knh_fastmalloc(Ctx *ctx, size_t size)
 {
 	if(size <= KNH_FASTMALLOC_SIZE) {
-		CHECK_UNUSED_OBJECT(ctx);
+		if(unlikely(ctx->unusedObject == NULL)) {
+			KNH_ASSERT(ctx->unusedObjectSize == 0);
+			((knh_Context_t*)ctx)->unusedObject = new_UnusedObject(ctx);
+		}
 		Object *o = ctx->unusedObject;
 		((knh_Context_t*)ctx)->unusedObject = (knh_Object_t*)o->ref;
 		((knh_Context_t*)ctx)->unusedObjectSize -= 1;
@@ -238,7 +232,7 @@ void *knh_fastmalloc(Ctx *ctx, size_t size)
 		if (unlikely(block == NULL)) {
 			DBG2_ABORT();
 			KNH_EXIT("OutOfMemory!!: %d bytes used", (int)ctx->stat->usedMemorySize);
-			//KNH_THROW__T(ctx, "OutOfMemory!!");
+			//KNH_THROWs(ctx, "OutOfMemory!!");
 		}
 		knh_stat_incUsedMemorySize(ctx, size);
 		return block;
@@ -274,7 +268,10 @@ knh_Object_t *new_hObject(Ctx *ctx, knh_flag_t flag, knh_class_t bcid, knh_class
 	DBG2_ASSERT(bcid != CLASS_Context);
 	DBG2_ASSERT(bcid < KNH_TSTRUCT_SIZE);
 	DBG2_ASSERT(ctx != NULL);
-	CHECK_UNUSED_OBJECT(ctx);
+	if(unlikely(ctx->unusedObject == NULL)) {
+		KNH_ASSERT(ctx->unusedObjectSize == 0);
+		((knh_Context_t*)ctx)->unusedObject = new_UnusedObject(ctx);
+	}
 	{
 		knh_Object_t *o = ctx->unusedObject;
 		((knh_Context_t*)ctx)->unusedObject = (knh_Object_t*)o->ref;
@@ -298,7 +295,10 @@ knh_Object_t *new_Object_bcid(Ctx *ctx, knh_class_t bcid, int init)
 {
 	DBG2_ASSERT(bcid != CLASS_Context);
 	DBG2_ASSERT(bcid < KNH_TSTRUCT_SIZE);
-	CHECK_UNUSED_OBJECT(ctx);
+	if(unlikely(ctx->unusedObject == NULL)) {
+		KNH_ASSERT(ctx->unusedObjectSize == 0);
+		((knh_Context_t*)ctx)->unusedObject = new_UnusedObject(ctx);
+	}
 	{
 		knh_Object_t *o = ctx->unusedObject;
 		((knh_Context_t*)ctx)->unusedObject = (knh_Object_t*)o->ref;
@@ -329,18 +329,10 @@ knh_Object_t *new_Object_bcid(Ctx *ctx, knh_class_t bcid, int init)
 knh_Object_t *new_Object_init(Ctx *ctx, knh_flag_t flag, knh_class_t cid, int init)
 {
 	DBG2_ASSERT(cid != CLASS_Context);
-#ifdef KNH_USING_MARKGC
-	size_t maxsize = ctx->share->ObjectPageTableMaxSize;
-	size_t gc_threshold = maxsize / 4 * 3;
-	if (unlikely(ctx->share->ObjectPageTableSize == gc_threshold))
-	{
-		size_t tsize  = ctx->share->ObjectPageTableSize;
-		knh_System_gc(ctx);
-		knh_Object_t *old = ctx->unusedObject;
-		knh_Object_t *o = new_UnusedObject(ctx);
+	if(ctx->unusedObject == NULL) {
+		KNH_ASSERT(ctx->unusedObjectSize == 0);
+		((knh_Context_t*)ctx)->unusedObject = new_UnusedObject(ctx);
 	}
-#endif
-	CHECK_UNUSED_OBJECT(ctx);
 	{
 		knh_Object_t *o = ctx->unusedObject;
 		((knh_Context_t*)ctx)->unusedObject = (knh_Object_t*)o->ref;
@@ -348,7 +340,7 @@ knh_Object_t *new_Object_init(Ctx *ctx, knh_flag_t flag, knh_class_t cid, int in
 		knh_stat_incUsedObjectSize(ctx, 1);
 		o->h.magic = KNH_OBJECT_MAGIC;
 		knh_Object_RCset(o, KNH_RCGC_INIT);
-		DBG2_ASSERT_cid(cid);
+		KNH_ASSERT_cid(cid);
 		knh_class_t bcid = ClassTable(cid).bcid;
 		o->h.bcid = bcid;
 		o->h.flag = ClassTable(cid).oflag | flag;
@@ -372,7 +364,10 @@ knh_Object_t *new_Object_init(Ctx *ctx, knh_flag_t flag, knh_class_t cid, int in
 KNHAPI(Object*) new_Object_boxing(Ctx *ctx, knh_class_t cid, knh_sfp_t *sfp)
 {
 	KNH_ASSERT(ctx != NULL);
-	CHECK_UNUSED_OBJECT(ctx);
+	if(unlikely(ctx->unusedObject == NULL)) {
+		KNH_ASSERT(ctx->unusedObjectSize == 0);
+		((knh_Context_t*)ctx)->unusedObject = new_UnusedObject(ctx);
+	}
 	{
 		knh_Object_t *o = ctx->unusedObject;
 		((knh_Context_t*)ctx)->unusedObject = (knh_Object_t*)o->ref;
@@ -428,7 +423,7 @@ void knh_Object_traverse(Ctx *ctx, knh_Object_t *o, knh_ftraverse ftr)
 
 #define BSHIFT ((KONOHA_PAGESIZE / sizeof(knh_Object_t)) / (sizeof(knh_uintptr_t) * 8))
 
-static volatile size_t markedObjectSize = 0;
+volatile static size_t markedObjectSize = 0;
 
 /* ------------------------------------------------------------------------ */
 
@@ -481,6 +476,7 @@ void knh_System_gc(Ctx *ctx)
 		char *max = ctx->share->ObjectPageTable[tindex].thead + SIZEOF_OBJECTPAGE;
 		while(t + KONOHA_PAGESIZE < max) {
 			knh_memcpy(t + ((BSHIFT) * sizeof(knh_uintptr_t)), t, (BSHIFT) * sizeof(knh_uintptr_t));
+			//knh_bzero(t + ((BSHIFT) * sizeof(knh_uintptr_t)), (BSHIFT) * sizeof(knh_uintptr_t));
 			t += KONOHA_PAGESIZE;
 		}
 	}
@@ -531,10 +527,7 @@ void knh_System_gc(Ctx *ctx)
 void knh_Object_RCsweep(Ctx *ctx, Object *o)
 {
 	knh_Object_RCdec(o);
-#if defined(KNH_USING_RCGC)
-	if (knh_Object_isRC0(o))
-#endif
-	{
+	if(knh_Object_isRC0(o)) {
 		knh_Object_free(ctx, o);
 	}
 }
