@@ -43,7 +43,54 @@ extern "C" {
 
 int knh_stack_argc(Ctx *ctx, knh_sfp_t *v)
 {
-	return (((Context*)ctx)->esp - v);
+	return (((knh_Context_t*)ctx)->esp - v);
+}
+
+/* ------------------------------------------------------------------------ */
+
+knh_Array_t* knh_stack_toArray(Ctx *ctx, knh_sfp_t *sfp, knh_class_t cid)
+{
+	KNH_ASSERT_cid(cid);
+	if(cid == CLASS_Any) {
+		knh_Array_t *a = new_Array0(ctx, ctx->esp - sfp);
+		while(sfp < ctx->esp) {
+			knh_sfp_boxing(ctx, sfp);
+			knh_Array_add(ctx, a, sfp->o);
+			sfp++;
+		}
+		return a;
+	}
+	else if(cid == CLASS_Int || ClassTable(cid).bcid == CLASS_Int) {
+		knh_IArray_t *a = new_IArray(ctx, cid, ctx->esp - sfp);
+		while(sfp < ctx->esp) {
+			if(IS_NOTNULL(sfp->o)) {
+				knh_IArray_add(ctx, a, sfp->ivalue);
+			}
+			sfp++;
+		}
+		return (knh_Array_t*)a;
+	}
+	else if(cid == CLASS_Float || ClassTable(cid).bcid == CLASS_Float) {
+		knh_FArray_t *a = new_FArray(ctx, cid, ctx->esp - sfp);
+		while(sfp < ctx->esp) {
+			if(IS_NOTNULL(sfp->o)) {
+				knh_FArray_add(ctx, a, sfp->fvalue);
+			}
+			sfp++;
+		}
+		return (knh_Array_t*)a;
+	}
+	else {
+		knh_Array_t *a = new_Array(ctx, cid, ctx->esp - sfp);
+		while(sfp < ctx->esp) {
+			if(IS_NOTNULL(sfp->o)) {
+				knh_sfp_boxing(ctx, sfp);
+				knh_Array_add(ctx, a, sfp->o);
+			}
+			sfp++;
+		}
+		return a;
+	}
 }
 
 /* ------------------------------------------------------------------------ */
@@ -65,7 +112,7 @@ KNHAPI(void) knh_sfp_unboxing(Ctx *ctx, knh_sfp_t *sfp)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_esp1_sformat(Ctx *ctx, Method *mtd, OutputStream *w, Any *m)
+KNHAPI(void) knh_esp1_sformat(Ctx *ctx, knh_Method_t *mtd, knh_OutputStream_t *w, Any *m)
 {
 	DBG2_ASSERT(IS_Method(mtd));
 	knh_sfp_t *esp = KNH_LOCAL(ctx);
@@ -76,10 +123,10 @@ KNHAPI(void) knh_esp1_sformat(Ctx *ctx, Method *mtd, OutputStream *w, Any *m)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_esp1_format(Ctx *ctx, knh_methodn_t mn, OutputStream *w, Any *m)
+KNHAPI(void) knh_esp1_format(Ctx *ctx, knh_methodn_t mn, knh_OutputStream_t *w, Any *m)
 {
 	knh_sfp_t *esp = KNH_LOCAL(ctx);
-	Method *mtd = knh_lookupFormatter(ctx, knh_Object_cid(esp[1].o), mn);
+	knh_Method_t *mtd = knh_lookupFormatter(ctx, knh_Object_cid(esp[1].o), mn);
 	DBG2_ASSERT(esp == ctx->esp);
 	KNH_SETv(ctx, esp[2].o, w);
 	KNH_SETv(ctx, esp[3].o, m);
@@ -89,35 +136,35 @@ KNHAPI(void) knh_esp1_format(Ctx *ctx, knh_methodn_t mn, OutputStream *w, Any *m
 /* ------------------------------------------------------------------------ */
 /* @method void System.stackdump() */
 
-METHOD knh__System_stackdump(Ctx *ctx, knh_sfp_t *sfp)
+METHOD System_stackdump(Ctx *ctx, knh_sfp_t *sfp)
 {
-	fprintf(stderr, "\n** ** ** ** ** ** ** ** **\n");
-	int i;
-	if(sfp-1 > ctx->stack && IS_Method(sfp[-1].mtd)) {
-		char buf[CLASSNAME_BUFSIZ];
-		knh_format_cmethodn(ctx, buf, sizeof(buf), DP(sfp[-1].mtd)->cid, DP(sfp[-1].mtd)->mn);
-		fprintf(stderr, "sfp[-1] %s data=%d\n", buf, (int)sfp[-1].data);
-	}
-	for(i = 0; sfp + i < ctx->esp; i++) {
-		fprintf(stderr, "sfp[%2d] %s data=%d\n", i, CLASSN(sfp[i].o->h.cid), (int)sfp[i].data);
-	}
-	for(i = 0; i < 2; i++) {
-		fprintf(stderr, "esp[%2d] %s data=%d\n", i, CLASSN(ctx->esp[i].o->h.cid), (int)sfp[i].data);
-	}
-	fprintf(stderr, "** ** ** ** ** ** ** ** **\n");
+//	fprintf(stderr, "\n** ** ** ** ** ** ** ** **\n");
+//	int i;
+//	if(sfp-1 > ctx->stack && IS_Method(sfp[-1].mtd)) {
+//		char buf[CLASSNAME_BUFSIZ];
+//		knh_format_cmethodn(ctx, buf, sizeof(buf), DP(sfp[-1].mtd)->cid, DP(sfp[-1].mtd)->mn);
+//		fprintf(stderr, "sfp[-1] %s data=%d\n", buf, (int)sfp[-1].data);
+//	}
+//	for(i = 0; sfp + i < ctx->esp; i++) {
+//		fprintf(stderr, "sfp[%2d] %s data=%d\n", i, CLASSN(sfp[i].o->h.cid), (int)sfp[i].data);
+//	}
+//	for(i = 0; i < 2; i++) {
+//		fprintf(stderr, "esp[%2d] %s data=%d\n", i, CLASSN(ctx->esp[i].o->h.cid), (int)sfp[i].data);
+//	}
+//	fprintf(stderr, "** ** ** ** ** ** ** ** **\n");
 }
 
 /* ======================================================================== */
 /* [call] */
 
-void knh_sfp_typecheck(Ctx *ctx, knh_sfp_t *sfp, Method *mtd, knh_code_t *pc)
+void knh_stack_typecheck(Ctx *ctx, knh_sfp_t *sfp, knh_Method_t *mtd, knh_code_t *pc)
 {
 	DBG2_ASSERT(IS_Method(sfp[-1].mtd));
 	char *emsg;
 	knh_class_t this_cid = knh_Object_cid(sfp[0].o);
 	int i, argc = knh_Method_isVarArgs(mtd) ? (ctx->esp - sfp) : knh_Method_psize(mtd);
 	for(i = 1; i < argc; i++) {
-		knh_type_t type = knh_Method_ptype(ctx, mtd, this_cid, i);
+		knh_type_t type = knh_Method_ptype(ctx, mtd, this_cid, i - 1);
 		if(IS_NULL(sfp[i].o)) {
 			if(IS_NNTYPE(type)) {
 				emsg = "Null!!: the parameter %d of %M"; goto L_THROWERR;
@@ -126,7 +173,7 @@ void knh_sfp_typecheck(Ctx *ctx, knh_sfp_t *sfp, Method *mtd, knh_code_t *pc)
 		else {
 			knh_class_t reqc = CLASS_type(type);
 			if(!knh_class_instanceof(ctx, knh_Object_cid(sfp[i].o), reqc)) {
-				emsg = "Null!!: the parameter %d of %M"; goto L_THROWERR;
+				emsg = "Type!!: the parameter %d of %M"; goto L_THROWERR;
 			}
 		}
 	}
@@ -136,31 +183,11 @@ void knh_sfp_typecheck(Ctx *ctx, knh_sfp_t *sfp, Method *mtd, knh_code_t *pc)
 	{
 		knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
 		knh_printf(ctx, cwb->w, emsg, (knh_intptr_t)i, DP(mtd)->mn);
-		String *s = new_String__cwb(ctx, cwb);
+		knh_String_t *s = knh_cwb_newString(ctx, cwb);
 		knh_throw(ctx, (Object*)s, knh_Method_file(ctx, sfp[-1].mtd), knh_Method_pctoline(sfp[-1].mtd, pc));
 	}
 }
 
-/* ======================================================================== */
-/* [macros] */
-
-#define _KNH_THROW(ctx, e) knh_throw(ctx, UP(e), knh_safefile(__FILE__), __LINE__)
-
-#define _KNH_THROWs(ctx, s) \
-	knh_throwException(ctx, new_Exception__b(ctx, B(s)), knh_safefile(__FILE__), __LINE__); \
-
-#define _KNH_THROWf(ctx, fmt, ...) \
-	char throcwb_[256]; \
-	knh_snprintf(throcwb_, sizeof(throcwb_), fmt, ## __VA_ARGS__); \
-	knh_throwException(ctx, new_Exception__b(ctx, B(throcwb_)), knh_safefile(__FILE__), __LINE__); \
-
-#define DEBUG_THROWf(ctx, fmt, ...) \
-	fprintf(stderr, "THROW?[%s:%d]: ", __FUNCTION__, __LINE__); \
-	fprintf(stderr, fmt, ## __VA_ARGS__); \
-	fprintf(stderr, "\n"); \
-
-#define _TODO_THROW(ctx) knh_throw_TODO(ctx, (char*)__FILE__, __LINE__, (char*)__FUNCTION__)
-
 /* ======================================================================== */
 /* [throw] */
 
@@ -187,7 +214,7 @@ KNHAPI(void) knh_throw_bugstop(Ctx *ctx, char *file, int line, char *func)
 /* [throw] */
 
 static
-void knh_Exception_addStackTrace(Ctx *ctx, Exception *o, String *msg)
+void knh_Exception_addStackTrace(Ctx *ctx, knh_Exception_t *o, knh_String_t *msg)
 {
 	KNH_ASSERT(IS_Exception(o));
 	if(IS_NOTNULL(msg)) {
@@ -201,9 +228,9 @@ void knh_Exception_addStackTrace(Ctx *ctx, Exception *o, String *msg)
 /* ------------------------------------------------------------------------ */
 
 static
-String *knh_stack_newStackTrace(Ctx *ctx, knh_sfp_t *sfp, Exception *e)
+knh_String_t *knh_stack_newStackTrace(Ctx *ctx, knh_sfp_t *sfp, knh_Exception_t *e)
 {
-	Method *mtd = sfp[-1].mtd;
+	knh_Method_t *mtd = sfp[-1].mtd;
 	knh_code_t *pc = sfp[-1].pc;
 	char *file = "-";
 	int  line = 0;
@@ -218,17 +245,17 @@ String *knh_stack_newStackTrace(Ctx *ctx, knh_sfp_t *sfp, Exception *e)
 			}
 		}
 		else {
-			return (String*)KNH_NULL;
+			return (knh_String_t*)KNH_NULL;
 		}
 	}
 	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
 	knh_write_fline(ctx, cwb->w, file, line);
 	knh_putc(ctx, cwb->w, ':');
 	knh_write_cid(ctx, cwb->w, DP(mtd)->cid);
-	if(DP(mtd)->mn != METHODN_lambda) {
+	if(DP(mtd)->mn != METHODN_LAMBDA) {
 		int i = 0;
 		knh_putc(ctx, cwb->w, '.');
-		knh_write_mn(ctx, cwb->w, DP(mtd)->mn);
+		knh_write_mn(ctx, cwb->w, knh_Method_rztype(mtd), DP(mtd)->mn);
 		knh_putc(ctx, cwb->w, '(');
 		for(i = 0; i < knh_Method_psize(mtd); i++) {
 			knh_mparam_t p = knh_Method_param(mtd, i);
@@ -260,14 +287,14 @@ String *knh_stack_newStackTrace(Ctx *ctx, knh_sfp_t *sfp, Exception *e)
 		}
 		knh_putc(ctx, cwb->w, ')');
 	}
-	String *s = new_String__cwb(ctx, cwb);
+	knh_String_t *s = knh_cwb_newString(ctx, cwb);
 	//DBG2_P("stacktrace=%s", knh_String_tochar(s));
 	return s;
 }
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_throwException(Ctx *ctx, Exception *e, char *file, int line)
+KNHAPI(void) knh_throwException(Ctx *ctx, knh_Exception_t *e, char *file, int line)
 {
 	KNH_ASSERT(IS_Exception(e));
 //	if(file != NULL) {
@@ -306,9 +333,9 @@ KNHAPI(void) knh_throw(Ctx *ctx, Object *e, char *file, int line)
 	if(IS_NULL(e)) {
 		knh_throwException(ctx, new_NullException (ctx, e), file, line);
 	}else if(IS_Exception(e)) {
-		knh_throwException(ctx, (Exception*)e, file, line);
+		knh_throwException(ctx, (knh_Exception_t*)e, file, line);
 	}else if(IS_bString(e)) {
-		knh_throwException(ctx, new_Exception(ctx, (String*)e), file, line);
+		knh_throwException(ctx, new_Exception(ctx, (knh_String_t*)e), file, line);
 	}else {
 		DBG2_P("Cannot throw %s", CLASSN(knh_Object_cid(e)));
 		knh_throwException(ctx, new_Exception__b(ctx, STEXT("Type!!")), file, line);
@@ -327,10 +354,9 @@ KNHAPI(void) knh_throwStupidException(Ctx *ctx)
 KNHAPI(void) konoha_throwSecurityException(void)
 {
 	Ctx *ctx = knh_getCurrentContext();
-	knh_throwException(ctx, new_Exception(ctx, TS_SecurityException), NULL, 0);
+	knh_throwException(ctx, new_Exception(ctx, knh_getExptName(ctx, EXPT_Security)), NULL, 0);
 }
 
-
 /* ------------------------------------------------------------------------ */
 
 #ifdef __cplusplus
