@@ -120,7 +120,7 @@ void knh_ObjectPageTable_free(Ctx *ctx, char *thead)
 				default:
 					fprintf(stderr, "async mem cid=%s(%d), refc=%d\n", STRUCTN(o->h.bcid), (int)o->h.cid, (int)o->h.refc);
 				}
-			} );
+			} )
 			o->h.refc = 0;
 #endif
 			/* BUGS: We cannot free cyclic objects at this stage */
@@ -177,7 +177,7 @@ void knh_Context_initCommon(Ctx *ctx, knh_Context_t *o, size_t stacksize)
 	KNH_INITv(o->in,  DP(ctx->sys)->in);
 	KNH_INITv(o->out, DP(ctx->sys)->out);
 	KNH_INITv(o->err, DP(ctx->sys)->err);
-	KNH_INITv(o->kc, new_Object_bcid(ctx, CLASS_Gamma, 0));
+	KNH_INITv(o->abr, KNH_NULL);
 	KNH_INITv(o->msgError, UP(TS_EMPTY));
 }
 
@@ -200,11 +200,9 @@ void knh_Context_traverseCommon(Ctx *ctx, knh_Context_t *o, knh_ftraverse ftr)
 	ftr(ctx, UP(o->in));
 	ftr(ctx, UP(o->out));
 	ftr(ctx, UP(o->err));
-	ftr(ctx, UP(o->kc));
+	ftr(ctx, UP(o->abr));
 	ftr(ctx, UP(o->msgError));
 
-	if(o->lines) ftr(ctx, UP(o->lines));
-
 	if(IS_SWEEP(ftr)) {
 		DBG2_P("freeing stack (ctxid=%d)...", o->ctxid);
 		KNH_FREE(ctx, o->stack, sizeof(knh_sfp_t) * o->stacksize);
@@ -219,7 +217,10 @@ void knh_Context_traverseCommon(Ctx *ctx, knh_Context_t *o, knh_ftraverse ftr)
 		knh_mutex_destroy(o->ctxlock);
 		KNH_FREE(ctx, o->ctxlock, sizeof(knh_mutex_t));
 		o->ctxlock = NULL;
+	} else {
+	  ftr(ctx, UP(o->lines));
 	}
+
 }
 
 /* ------------------------------------------------------------------------ */
@@ -229,16 +230,17 @@ static Ctx *new_Context0(Ctx *parent)
 	knh_Context_t *ctx;
 	if(parent == NULL) {
 		ctx = (knh_Context_t*)malloc(sizeof(knh_Context_t));
-		knh_bzero(ctx, sizeof(knh_Context_t));
 		parent = ctx;
 	}
 	else {
 		ctx = (knh_Context_t*)knh_malloc(parent, sizeof(knh_Context_t));
-		knh_bzero(ctx, sizeof(knh_Context_t));
 	}
+	knh_bzero(ctx, sizeof(knh_Context_t));
 	ctx->h.magic = KNH_OBJECT_MAGIC;
-	knh_Object_RCset(ctx, KNH_RCGC_INIT);
-	ctx->h.flag = FLAG_CTX(FLAG_Context);
+#ifdef KNH_HOBJECT_REFC
+	ctx->h.refc = KNH_RCGC_INIT;
+#endif
+	ctx->h.flag = 	FLAG_CTX(FLAG_Context);
 	ctx->h.bcid = CLASS_Context;
 	ctx->h.cid  = CLASS_Context;
 	ctx->h.ctxid = 0;
@@ -345,7 +347,11 @@ void knh_initSharedData(knh_Context_t *ctx)
 	}
 	{
 		knh_Float_t *fo = (knh_Float_t*)new_hObject(ctx, FLAG_Float, CLASS_Float, CLASS_Float);
-		(fo)->n.fvalue = KNH_FLOAT_ZERO;
+#ifndef KNH_USING_NOFLOAT
+		(fo)->n.fvalue = 0.0;
+#else
+		(fo)->n.fvalue = 0;
+#endif
 		KNH_INITv(share->constFloat0, fo);
 	}
 
@@ -376,7 +382,7 @@ Ctx *knh_createRootContext(size_t stacksize)
 /* ------------------------------------------------------------------------ */
 
 static
-void knh_ClassField_toAbstractAll(Ctx *ctx, knh_ClassField_t *cs)
+void knh_ClassStruct_toAbstractAll(Ctx *ctx, knh_ClassStruct_t *cs)
 {
 	knh_Array_t *a = cs->methods;
 	if(IS_bArray(a)) {
@@ -402,7 +408,7 @@ void knh_ClassField_toAbstractAll(Ctx *ctx, knh_ClassField_t *cs)
 static
 void knh_traverseSharedData(Ctx *ctx, knh_SharedData_t *share, knh_ftraverse ftr)
 {
-	size_t i;
+	int i;
 	ftr(ctx, share->constNull);
 	ftr(ctx, share->constTrue);
 	ftr(ctx, share->constFalse);
@@ -424,7 +430,7 @@ void knh_traverseSharedData(Ctx *ctx, knh_SharedData_t *share, knh_ftraverse ftr
 		}
 	}
 
-	for(i = 0; i < share->ExptTableSize; i++) {
+	for(i = 0; i < (int)(share->ExptTableSize); i++) {
 		if(ExptTable(i).name != NULL) {
 			ftr(ctx, UP(ExptTable(i).name));
 		}
@@ -434,11 +440,11 @@ void knh_traverseSharedData(Ctx *ctx, knh_SharedData_t *share, knh_ftraverse ftr
 	if(IS_SWEEP(ftr)) {
 		for(i = 0; i < share->StructTableSize; i++) {
 			DBG2_ASSERT(ClassTable(i).sname != NULL);
-			knh_ClassField_toAbstractAll(ctx, ClassTable(i).cstruct);
+			knh_ClassStruct_toAbstractAll(ctx, ClassTable(i).cstruct);
 		}
 		for(i = share->ClassTableSize; i < KNH_TCLASS_SIZE; i++) {
 			DBG2_ASSERT(ClassTable(i).sname != NULL);
-			knh_ClassField_toAbstractAll(ctx, ClassTable(i).cstruct);
+			knh_ClassStruct_toAbstractAll(ctx, ClassTable(i).cstruct);
 		}
 	}
 
@@ -508,7 +514,7 @@ void knh_traverseSharedData(Ctx *ctx, knh_SharedData_t *share, knh_ftraverse ftr
 
 		((knh_Context_t*)ctx)->fsweep = knh_Object_finalSweep;
 		KNH_ASSERT(share->ObjectPageTable != NULL);
-		for(i = 0; i < share->ObjectPageTableSize; i++) {
+		for(i = 0; i < (int)(share->ObjectPageTableSize); i++) {
 			knh_ObjectPageTable_free(ctx, share->ObjectPageTable[i].thead);
 			share->ObjectPageTable[i].thead = NULL;
 		}
@@ -611,7 +617,7 @@ Ctx *new_ThreadContext(Ctx *parent)
 			KNH_SETv(root, ctx->in,  DP(root->sys)->in);
 			KNH_SETv(root, ctx->out, DP(root->sys)->out);
 			KNH_SETv(root, ctx->err, DP(root->sys)->err);
-			KNH_SETv(root, ctx->kc, KNH_NULL);
+			KNH_SETv(root, ctx->abr, KNH_NULL);
 		}
 	}
 	if(ctx == NULL) {
@@ -659,7 +665,7 @@ static int isFirstVirtualMachine = 1;
 
 KNHAPI(konoha_t) konoha_open(size_t stacksize)
 {
-	konoha_t k = {KONOHA_MAGIC, NULL};
+	konoha_t k = {KONOHA_MAGIC};
 	if(isFirstVirtualMachine) {
 		isFirstVirtualMachine = 0;
 		konoha_init();
@@ -687,10 +693,8 @@ KNHAPI(void) konoha_close(konoha_t konoha)
 		return;
 	}
 	{
-		knh_System_gc(ctx);
 		knh_ObjectField_t *scr = (knh_ObjectField_t*)knh_NameSpace_getScript(ctx, ctx->share->mainns);
-		//knh_ObjectField_traverse(konoha.ctx, scr, ctx->fsweep);
-		StructTable(CLASS_ObjectField).ftraverse(ctx,UP(scr), ctx->fsweep);
+		knh_ObjectField_traverse(konoha.ctx, scr, ctx->fsweep);
 		(scr)->h.cid = CLASS_Object;
 		(scr)->h.bcid = CLASS_Object;
 		KNH_FREE(ctx, scr->fields, sizeof(Object*) * KNH_SCRIPT_FIELDSIZE);
